/*
 * Automatically generated file; DO NOT EDIT.
 * Linux/powerpc 7.0.13 Kernel Configuration
 */
#define CONFIG_AIC79XX_RESET_DELAY_MS 15000
#define CONFIG_SERIAL_8250_RT288X 1
#define CONFIG_VIDEO_PVRUSB2_MODULE 1
#define CONFIG_POWERNV_CPUFREQ 1
#define CONFIG_NFC_MRVL_MODULE 1
#define CONFIG_IPV6_IOAM6_LWTUNNEL 1
#define CONFIG_PATA_OPTI_MODULE 1
#define CONFIG_SERIAL_8250_RUNTIME_UARTS 32
#define CONFIG_KUNIT_DEFAULT_FILTER ""
#define CONFIG_USB_GSPCA_PAC7311_MODULE 1
#define CONFIG_IIO_GTS_HELPER_MODULE 1
#define CONFIG_GENERIC_PHY_MIPI_DPHY 1
#define CONFIG_PPC_RADIX_BROADCAST_TLBIE 1
#define CONFIG_NET_CLS_BPF_MODULE 1
#define CONFIG_DHT11_MODULE 1
#define CONFIG_USB_LD_MODULE 1
#define CONFIG_UBSAN_BOUNDS 1
#define CONFIG_ARCH_USES_HIGH_VMA_FLAGS 1
#define CONFIG_LEDS_AS3645A_MODULE 1
#define CONFIG_USB_GSPCA_SQ930X_MODULE 1
#define CONFIG_RFKILL_MODULE 1
#define CONFIG_SPARSEMEM_VMEMMAP_ENABLE 1
#define CONFIG_STRING_KUNIT_TEST_MODULE 1
#define CONFIG_NET_9P_VIRTIO_MODULE 1
#define CONFIG_USB_COMMON 1
#define CONFIG_SND_SB_COMMON_MODULE 1
#define CONFIG_MTD_MAP_BANK_WIDTH_1 1
#define CONFIG_TABLET_SERIAL_WACOM4_MODULE 1
#define CONFIG_SCSI_ENCLOSURE_MODULE 1
#define CONFIG_CRYPTO_FIPS 1
#define CONFIG_MODULE_UNLOAD_TAINT_TRACKING 1
#define CONFIG_NFT_FIB_IPV4_MODULE 1
#define CONFIG_NET_ACT_PEDIT_MODULE 1
#define CONFIG_MMC_MODULE 1
#define CONFIG_CM32181_MODULE 1
#define CONFIG_GPIOLIB 1
#define CONFIG_DVB_LGS8GXX_MODULE 1
#define CONFIG_NF_CONNTRACK_EVENTS 1
#define CONFIG_NET_EMATCH_NBYTE_MODULE 1
#define CONFIG_DM_CACHE_SMQ_MODULE 1
#define CONFIG_IR_SANYO_DECODER_MODULE 1
#define CONFIG_NFS_V3_MODULE 1
#define CONFIG_INPUT_KXTJ9_MODULE 1
#define CONFIG_VIDEO_DEV_MODULE 1
#define CONFIG_USB_GSPCA_OV534_MODULE 1
#define CONFIG_VDPA_MODULE 1
#define CONFIG_PPC_64K_PAGES 1
#define CONFIG_ARCH_HAS_MEMREMAP_COMPAT_ALIGN 1
#define CONFIG_NETFILTER_XT_TARGET_MASQUERADE_MODULE 1
#define CONFIG_VL6180_MODULE 1
#define CONFIG_QSEMI_PHY_MODULE 1
#define CONFIG_RFKILL_INPUT 1
#define CONFIG_USB_GSPCA_TOUPTEK_MODULE 1
#define CONFIG_USB_ROLE_SWITCH 1
#define CONFIG_SCSI_MYRB_MODULE 1
#define CONFIG_IRQ_BYPASS_MANAGER 1
#define CONFIG_NET_HANDSHAKE 1
#define CONFIG_NF_CONNTRACK_OVS 1
#define CONFIG_NETFILTER_XT_TARGET_RATEEST_MODULE 1
#define CONFIG_HTS221_I2C_MODULE 1
#define CONFIG_LIBFDT 1
#define CONFIG_R8169_MODULE 1
#define CONFIG_USB_OHCI_HCD_PPC_OF_LE 1
#define CONFIG_IWLMVM_MODULE 1
#define CONFIG_ATH10K_CE 1
#define CONFIG_RDS_TCP_MODULE 1
#define CONFIG_NFC_PN533_I2C_MODULE 1
#define CONFIG_PAGE_SHIFT 16
#define CONFIG_NLS_ASCII 1
#define CONFIG_GPIO_CADENCE_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_TIME_MODULE 1
#define CONFIG_HOTPLUG_PCI_POWERNV_MODULE 1
#define CONFIG_AF8133J_MODULE 1
#define CONFIG_DRIVER_PE_KUNIT_TEST_MODULE 1
#define CONFIG_PPC 1
#define CONFIG_SATA_SVW_MODULE 1
#define CONFIG_NFS_COMMON 1
#define CONFIG_PPPOATM_MODULE 1
#define CONFIG_WL12XX_MODULE 1
#define CONFIG_CGROUP_NET_CLASSID 1
#define CONFIG_NET_DSA_TAG_EDSA_MODULE 1
#define CONFIG_VIDEO_OV2735_MODULE 1
#define CONFIG_LOGIG940_FF 1
#define CONFIG_DVB_ISL6423_MODULE 1
#define CONFIG_SENSORS_MAX6697_MODULE 1
#define CONFIG_PHY_CADENCE_DPHY_RX_MODULE 1
#define CONFIG_IKCONFIG 1
#define CONFIG_TREE_RCU 1
#define CONFIG_ARCH_NEEDS_DEFER_KASAN 1
#define CONFIG_CRYPTO_CRYPTD_MODULE 1
#define CONFIG_VIDEO_SAA711X_MODULE 1
#define CONFIG_INFINIBAND_USER_MAD_MODULE 1
#define CONFIG_CRYPTO_ECC 1
#define CONFIG_PPC_PERF_CTRS 1
#define CONFIG_BMC150_ACCEL_MODULE 1
#define CONFIG_CRC_CCITT 1
#define CONFIG_USB_NET_AX8817X_MODULE 1
#define CONFIG_SYSTEM_BLACKLIST_HASH_LIST ""
#define CONFIG_INFINIBAND_QEDR_MODULE 1
#define CONFIG_NLS_CODEPAGE_437 1
#define CONFIG_SENSORS_GL518SM_MODULE 1
#define CONFIG_RMI4_F03_SERIO_MODULE 1
#define CONFIG_MMC5633_MODULE 1
#define CONFIG_VIDEO_TC358743_CEC 1
#define CONFIG_CRYPTO_GCM 1
#define CONFIG_ICE_SWITCHDEV 1
#define CONFIG_CRYPTO_WP512_MODULE 1
#define CONFIG_ARCH_USE_BUILTIN_BSWAP 1
#define CONFIG_SBITMAP 1
#define CONFIG_DRM_PANEL_STARTEK_KD070FHFID015_MODULE 1
#define CONFIG_USB_SERIAL_AIRCABLE_MODULE 1
#define CONFIG_USB_NET_CDCETHER_MODULE 1
#define CONFIG_KVM_BOOK3S_64_HANDLER 1
#define CONFIG_NET_NCSI 1
#define CONFIG_LRU_CACHE_MODULE 1
#define CONFIG_CC_HAS_ASSUME 1
#define CONFIG_AS_IS_GNU 1
#define CONFIG_STACKPROTECTOR_STRONG 1
#define CONFIG_RTW88_MODULE 1
#define CONFIG_PAGE_EXTENSION 1
#define CONFIG_IR_RC6_DECODER_MODULE 1
#define CONFIG_ENCRYPTED_KEYS 1
#define CONFIG_DVB_USB_UMT_010_MODULE 1
#define CONFIG_USB_EZUSB_FX2_MODULE 1
#define CONFIG_IIO_TRIGGER 1
#define CONFIG_USB_XHCI_PCI_RENESAS 1
#define CONFIG_EXPORTFS_BLOCK_OPS 1
#define CONFIG_TERANETICS_PHY_MODULE 1
#define CONFIG_V4L2_FWNODE_MODULE 1
#define CONFIG_SENSORS_INA209_MODULE 1
#define CONFIG_INIT_STACK_ALL_ZERO 1
#define CONFIG_SENSORS_DELTA_AHE50DC_FAN_MODULE 1
#define CONFIG_IMA_KEXEC_EXTRA_MEMORY_KB 0
#define CONFIG_HID_STEAM_MODULE 1
#define CONFIG_RANDSTRUCT_NONE 1
#define CONFIG_HTS221_MODULE 1
#define CONFIG_RTC_I2C_AND_SPI 1
#define CONFIG_RADIO_SHARK2_MODULE 1
#define CONFIG_VIDEO_OV9282_MODULE 1
#define CONFIG_ROMFS_ON_BLOCK 1
#define CONFIG_TMPFS 1
#define CONFIG_FUNCTION_ALIGNMENT_4B 1
#define CONFIG_PMBUS_MODULE 1
#define CONFIG_MUX_MMIO_MODULE 1
#define CONFIG_LOGO_LINUX_CLUT224_FILE "drivers/video/logo/logo_linux_clut224.ppm"
#define CONFIG_NETFILTER_NETLINK_MODULE 1
#define CONFIG_EPAPR_PARAVIRT 1
#define CONFIG_DM_MIRROR 1
#define CONFIG_RTW89_8852BT_MODULE 1
#define CONFIG_VIDEO_OV02A10_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_IRTOUCH 1
#define CONFIG_INFINIBAND_ISER_MODULE 1
#define CONFIG_VIDEO_OV02E10_MODULE 1
#define CONFIG_CARL9170_MODULE 1
#define CONFIG_IPV6 1
#define CONFIG_RTC_DRV_FM3130_MODULE 1
#define CONFIG_MCTP_SERIAL_MODULE 1
#define CONFIG_SCSI_DMA 1
#define CONFIG_USB_EHCI_HCD_PPC_OF 1
#define CONFIG_SIGNALFD 1
#define CONFIG_SENSORS_INTEL_M10_BMC_HWMON_MODULE 1
#define CONFIG_RTC_DRV_M48T59_MODULE 1
#define CONFIG_DVB_DDBRIDGE_MODULE 1
#define CONFIG_HID_ORTEK_MODULE 1
#define CONFIG_USB_PWC_INPUT_EVDEV 1
#define CONFIG_ASYNC_RAID6_RECOV_MODULE 1
#define CONFIG_MLX5_SF_MANAGER 1
#define CONFIG_CPU_FREQ_GOV_PERFORMANCE 1
#define CONFIG_F2FS_FS_LZORLE 1
#define CONFIG_SND_HDA_CODEC_HDMI_NVIDIA_MCP_MODULE 1
#define CONFIG_CHARGER_LT3651_MODULE 1
#define CONFIG_CFG80211_KUNIT_TEST_MODULE 1
#define CONFIG_MMC_CB710_MODULE 1
#define CONFIG_NFT_TPROXY_MODULE 1
#define CONFIG_INFINIBAND_RTRS_SERVER_MODULE 1
#define CONFIG_PPC_TRANSACTIONAL_MEM 1
#define CONFIG_INPUT_CS40L50_VIBRA_MODULE 1
#define CONFIG_NET_VENDOR_MICREL 1
#define CONFIG_RT2X00_LIB_DEBUGFS 1
#define CONFIG_PAPR_SCM_MODULE 1
#define CONFIG_DRM_PANEL_SAMSUNG_S6E63J0X03_MODULE 1
#define CONFIG_MTD_MAP_BANK_WIDTH_2 1
#define CONFIG_NET_VENDOR_ASIX 1
#define CONFIG_SND_HDA_INTEL_MODULE 1
#define CONFIG_VIDEO_OV5645_MODULE 1
#define CONFIG_SYSFS_SYSCALL 1
#define CONFIG_USB_SERIAL_SAFE_PADDED 1
#define CONFIG_KVM_GUEST 1
#define CONFIG_MFD_RT5120_MODULE 1
#define CONFIG_REGMAP_I3C_MODULE 1
#define CONFIG_MTD_NAND_ECC_MXIC 1
#define CONFIG_BRIDGE_EBT_IP6_MODULE 1
#define CONFIG_ARCH_SUPPORTS_DEBUG_PAGEALLOC 1
#define CONFIG_VIDEO_AR0521_MODULE 1
#define CONFIG_UHID_MODULE 1
#define CONFIG_IR_GPIO_CIR_MODULE 1
#define CONFIG_HID_RAZER_MODULE 1
#define CONFIG_SECURITY_NETWORK 1
#define CONFIG_CRYPTO_DRBG 1
#define CONFIG_SENSORS_RM3100_I2C_MODULE 1
#define CONFIG_HOTPLUG_SMT 1
#define CONFIG_IPE_PROP_DM_VERITY 1
#define CONFIG_NET_IPIP_MODULE 1
#define CONFIG_WDTPCI_MODULE 1
#define CONFIG_MTK_NET_PHYLIB_MODULE 1
#define CONFIG_AIO 1
#define CONFIG_HID_SPEEDLINK_MODULE 1
#define CONFIG_DRM_AMDGPU_USERPTR 1
#define CONFIG_NET_SCH_DUALPI2_MODULE 1
#define CONFIG_DRM_AMDGPU_SI 1
#define CONFIG_GENTOO_LINUX_INIT_SCRIPT 1
#define CONFIG_GENERIC_IRQ_SHOW 1
#define CONFIG_PATA_EFAR_MODULE 1
#define CONFIG_VFIO_IOMMU_SPAPR_TCE_MODULE 1
#define CONFIG_SCSI_DH_RDAC_MODULE 1
#define CONFIG_JFFS2_FS_SECURITY 1
#define CONFIG_BLOCK_LEGACY_AUTOLOAD 1
#define CONFIG_GPIB_INES_MODULE 1
#define CONFIG_VIDEO_GC2145_MODULE 1
#define CONFIG_PPC_HAVE_PMU_SUPPORT 1
#define CONFIG_PCI_SW_SWITCHTEC_MODULE 1
#define CONFIG_PPC_BOOK3S 1
#define CONFIG_SECURITY_TOMOYO_MAX_ACCEPT_ENTRY 2048
#define CONFIG_MPU3050_I2C_MODULE 1
#define CONFIG_VIDEO_TDA7432_MODULE 1
#define CONFIG_CDNS_I3C_MASTER_MODULE 1
#define CONFIG_NETFS_SUPPORT_MODULE 1
#define CONFIG_CAN_KVASER_USB_MODULE 1
#define CONFIG_MTD_RAW_NAND_MODULE 1
#define CONFIG_HID_WINWING_MODULE 1
#define CONFIG_PCI_PWRCTRL_MODULE 1
#define CONFIG_RPCSEC_GSS_KRB5_MODULE 1
#define CONFIG_INPUT_POWERMATE_MODULE 1
#define CONFIG_DP83848_PHY_MODULE 1
#define CONFIG_PRINTK_RINGBUFFER_KUNIT_TEST_MODULE 1
#define CONFIG_GPIO_QIXIS_FPGA_MODULE 1
#define CONFIG_PCI_PASID 1
#define CONFIG_WL1251_SDIO_MODULE 1
#define CONFIG_F2FS_FS_MODULE 1
#define CONFIG_IP_VS_MH_TAB_INDEX 12
#define CONFIG_CRYPTO_KPP 1
#define CONFIG_GENERIC_STRNLEN_USER 1
#define CONFIG_NET_FOU_MODULE 1
#define CONFIG_OF_IRQ 1
#define CONFIG_VIDEO_DW9714_MODULE 1
#define CONFIG_SND_LAYLA20_MODULE 1
#define CONFIG_BLK_SED_OPAL 1
#define CONFIG_PGTABLE_LEVELS 4
#define CONFIG_QFMT_V2 1
#define CONFIG_CRYPTO_ECB 1
#define CONFIG_DRM_AST_MODULE 1
#define CONFIG_NET_VENDOR_MICROSOFT 1
#define CONFIG_IDLE_PAGE_TRACKING 1
#define CONFIG_RTAS_ERROR_LOGGING 1
#define CONFIG_B44_MODULE 1
#define CONFIG_BT_BNEP_MODULE 1
#define CONFIG_MFD_QNAP_MCU_MODULE 1
#define CONFIG_REGMAP 1
#define CONFIG_SLAB_OBJ_EXT 1
#define CONFIG_BT_HCIBPA10X_MODULE 1
#define CONFIG_SENSORS_MAX1619_MODULE 1
#define CONFIG_PCI_ATS 1
#define CONFIG_SCSI_SYM53C8XX_MAX_TAGS 64
#define CONFIG_NET_VENDOR_8390 1
#define CONFIG_NLS_MAC_ROMANIAN_MODULE 1
#define CONFIG_USB_NET_GL620A_MODULE 1
#define CONFIG_DVB_ZL10039_MODULE 1
#define CONFIG_LED_TRIGGER_PHY 1
#define CONFIG_DRM_FBDEV_EMULATION 1
#define CONFIG_DRM_ANALOGIX_DP_MODULE 1
#define CONFIG_USER_NS 1
#define CONFIG_RT73USB_MODULE 1
#define CONFIG_NFSD_V4_2_INTER_SSC 1
#define CONFIG_TOUCHSCREEN_CY8CTMA140_MODULE 1
#define CONFIG_HWMON_VID_MODULE 1
#define CONFIG_MWL8K_MODULE 1
#define CONFIG_RTW89_MODULE 1
#define CONFIG_MODULE_DEBUGFS 1
#define CONFIG_GENERIC_HWEIGHT 1
#define CONFIG_LEDS_LM3601X_MODULE 1
#define CONFIG_SECURITY_YAMA 1
#define CONFIG_VIDEO_SONY_BTF_MPX_MODULE 1
#define CONFIG_RTW88_8822BE_MODULE 1
#define CONFIG_ARCH_HAS_ADD_PAGES 1
#define CONFIG_DRM_PANIC_SCREEN "user"
#define CONFIG_PPC_COPRO_BASE 1
#define CONFIG_USB_SERIAL_OPTION_MODULE 1
#define CONFIG_ARCH_WANT_IPC_PARSE_VERSION 1
#define CONFIG_SCD30_SERIAL_MODULE 1
#define CONFIG_IEEE802154_ATUSB_MODULE 1
#define CONFIG_NVME_AUTH 1
#define CONFIG_CRYPTO_DEV_QAT_C62XVF_MODULE 1
#define CONFIG_EROFS_FS_POSIX_ACL 1
#define CONFIG_SND_AU8810_MODULE 1
#define CONFIG_ARCH_HAS_MEMBARRIER_SYNC_CORE 1
#define CONFIG_USB_RTL8152_MODULE 1
#define CONFIG_VIDEO_OV8856_MODULE 1
#define CONFIG_GAMEPORT_FM801_MODULE 1
#define CONFIG_I2C_CP2615_MODULE 1
#define CONFIG_W1_SLAVE_DS28E04_MODULE 1
#define CONFIG_DRM_ITE_IT6505_MODULE 1
#define CONFIG_HAVE_C_RECORDMCOUNT 1
#define CONFIG_NET_VENDOR_VIA 1
#define CONFIG_NF_TABLES_IPV4 1
#define CONFIG_MAC80211_STA_HASH_MAX_SIZE 0
#define CONFIG_DEBUG_WX 1
#define CONFIG_TICK_ONESHOT 1
#define CONFIG_INET_ESP_MODULE 1
#define CONFIG_USB_SERIAL_SAFE_MODULE 1
#define CONFIG_NFC_NXP_NCI_I2C_MODULE 1
#define CONFIG_INFINIBAND_MODULE 1
#define CONFIG_VIDEO_TW9903_MODULE 1
#define CONFIG_RSI_USB_MODULE 1
#define CONFIG_RSI_COEX 1
#define CONFIG_DRM 1
#define CONFIG_BCMA_POSSIBLE 1
#define CONFIG_NF_REJECT_IPV6_MODULE 1
#define CONFIG_VIDEO_OV8865_MODULE 1
#define CONFIG_NFC_MODULE 1
#define CONFIG_NFSD_V4_SECURITY_LABEL 1
#define CONFIG_MLXSW_MINIMAL_MODULE 1
#define CONFIG_STMMAC_LIBPCI_MODULE 1
#define CONFIG_MFD_TPS6594_MODULE 1
#define CONFIG_CRYPTO_LIB_CHACHA_ARCH 1
#define CONFIG_MT76_USB_MODULE 1
#define CONFIG_TCG_TPM 1
#define CONFIG_USB_ISIGHTFW_MODULE 1
#define CONFIG_NET_DSA_TAG_DSA_COMMON_MODULE 1
#define CONFIG_BMI270_MODULE 1
#define CONFIG_STMMAC_ETH_MODULE 1
#define CONFIG_USB_YUREX_MODULE 1
#define CONFIG_USB_IPHETH_MODULE 1
#define CONFIG_USB_GSPCA_PAC7302_MODULE 1
#define CONFIG_NET_VENDOR_VERTEXCOM 1
#define CONFIG_IIO_TIGHTLOOP_TRIGGER_MODULE 1
#define CONFIG_BLK_DEV_MD 1
#define CONFIG_NET_DSA_TAG_QCA_MODULE 1
#define CONFIG_CPU_LITTLE_ENDIAN 1
#define CONFIG_MEMSTICK_R592_MODULE 1
#define CONFIG_VFIO_PCI_MODULE 1
#define CONFIG_TEHUTI_MODULE 1
#define CONFIG_I2C_ALGOPCA_MODULE 1
#define CONFIG_ISCSI_TCP_MODULE 1
#define CONFIG_MIPI_I3C_HCI_PCI_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_TRACE_MODULE 1
#define CONFIG_USB_SERIAL_GENERIC 1
#define CONFIG_NFC_DIGITAL_MODULE 1
#define CONFIG_MLX5_SF 1
#define CONFIG_BRIDGE_EBT_NFLOG_MODULE 1
#define CONFIG_INFINIBAND_MTHCA_MODULE 1
#define CONFIG_HID_SIGMAMICRO_MODULE 1
#define CONFIG_NF_DUP_IPV4_MODULE 1
#define CONFIG_CRYPTO_LIB_NH_KUNIT_TEST_MODULE 1
#define CONFIG_IR_IMON_RAW_MODULE 1
#define CONFIG_SCSI_AACRAID_MODULE 1
#define CONFIG_CAN_IFI_CANFD_MODULE 1
#define CONFIG_BRIDGE_EBT_SNAT_MODULE 1
#define CONFIG_RELAY 1
#define CONFIG_BLACKHOLE_DEV_KUNIT_TEST_MODULE 1
#define CONFIG_HAVE_KPROBES_ON_FTRACE 1
#define CONFIG_CPUSETS 1
#define CONFIG_MTD_NAND_CADENCE_MODULE 1
#define CONFIG_DRM_NOUVEAU_MODULE 1
#define CONFIG_USB_KC2190 1
#define CONFIG_PPC_RADIX_MMU 1
#define CONFIG_SND_HDA_CODEC_VIA_MODULE 1
#define CONFIG_CPU_FREQ_STAT 1
#define CONFIG_LOOPBACK_TARGET_MODULE 1
#define CONFIG_SCSI_MVSAS_MODULE 1
#define CONFIG_AL3000A_MODULE 1
#define CONFIG_MLX5_INFINIBAND_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_CLASSIFY_MODULE 1
#define CONFIG_LXT_PHY_MODULE 1
#define CONFIG_NET_VENDOR_DAVICOM 1
#define CONFIG_NVME_TARGET_AUTH 1
#define CONFIG_NFS_V3_ACL 1
#define CONFIG_DVB_SMIPCIE_MODULE 1
#define CONFIG_CRC_T10DIF 1
#define CONFIG_ATA_BMDMA 1
#define CONFIG_USB_TMC_MODULE 1
#define CONFIG_PPPOE_HASH_BITS_4 1
#define CONFIG_BCMA_HOST_PCI 1
#define CONFIG_QCA83XX_PHY_MODULE 1
#define CONFIG_SECURITY_LANDLOCK 1
#define CONFIG_NET_DSA_TAG_DSA_MODULE 1
#define CONFIG_X509_CERTIFICATE_PARSER 1
#define CONFIG_MAC80211_RC_DEFAULT "minstrel_ht"
#define CONFIG_HID_CYPRESS_MODULE 1
#define CONFIG_MLX4_INFINIBAND_MODULE 1
#define CONFIG_RICHTEK_RTQ6056_MODULE 1
#define CONFIG_DP83822_PHY_MODULE 1
#define CONFIG_SKGE_GENESIS 1
#define CONFIG_EVENTFD 1
#define CONFIG_GENERIC_CPU_VULNERABILITIES 1
#define CONFIG_FXAS21002C_MODULE 1
#define CONFIG_SYSTEM_BLACKLIST_KEYRING 1
#define CONFIG_SMI330_MODULE 1
#define CONFIG_DRM_PARADE_PS8640_MODULE 1
#define CONFIG_NFSD_V4 1
#define CONFIG_NUMA 1
#define CONFIG_VIDEO_HEXIUM_GEMINI_MODULE 1
#define CONFIG_HID_VIVALDI_COMMON_MODULE 1
#define CONFIG_NET_SHAPER 1
#define CONFIG_ZLIB_DEFLATE 1
#define CONFIG_BNXT_HWMON 1
#define CONFIG_ATM_HE_MODULE 1
#define CONFIG_USB_NET_DM9601_MODULE 1
#define CONFIG_SCSI_ARCMSR_MODULE 1
#define CONFIG_WLCORE_MODULE 1
#define CONFIG_IP_NF_NAT_MODULE 1
#define CONFIG_B43_PHY_HT 1
#define CONFIG_SATA_AHCI 1
#define CONFIG_HID_SONY_MODULE 1
#define CONFIG_CRYPTO_PCRYPT_MODULE 1
#define CONFIG_NET_VENDOR_CHELSIO 1
#define CONFIG_SND_HDA_CODEC_CMEDIA_MODULE 1
#define CONFIG_NF_DEFRAG_IPV6_MODULE 1
#define CONFIG_SENSORS_ADP1050_MODULE 1
#define CONFIG_PCI_NPEM 1
#define CONFIG_IXGBE_IPSEC 1
#define CONFIG_ZSMALLOC 1
#define CONFIG_CRYPTO_RNG 1
#define CONFIG_HID_SENSOR_ALS_MODULE 1
#define CONFIG_TCP_CONG_DCTCP_MODULE 1
#define CONFIG_REGULATOR_MAX77650_MODULE 1
#define CONFIG_NLMON_MODULE 1
#define CONFIG_FILE_LOCKING 1
#define CONFIG_NOUVEAU_DEBUG 5
#define CONFIG_PATA_IT8213_MODULE 1
#define CONFIG_IRQ_ALL_CPUS 1
#define CONFIG_VIDEO_ADV7511_MODULE 1
#define CONFIG_PATA_SIS_MODULE 1
#define CONFIG_RTC_DRV_ABEOZ9_MODULE 1
#define CONFIG_CRYPTO_HMAC 1
#define CONFIG_PCIE_MICROCHIP_HOST 1
#define CONFIG_ADXL355_I2C_MODULE 1
#define CONFIG_DMA_VIRTUAL_CHANNELS_MODULE 1
#define CONFIG_SCSI_INITIO_MODULE 1
#define CONFIG_REGULATOR_FIXED_VOLTAGE_MODULE 1
#define CONFIG_GP_PCI1XXXX_MODULE 1
#define CONFIG_GPIO_VIRTIO_MODULE 1
#define CONFIG_PGTABLE_HAS_HUGE_LEAVES 1
#define CONFIG_FS_ENCRYPTION_ALGS 1
#define CONFIG_TOUCHSCREEN_CYTTSP5_MODULE 1
#define CONFIG_STACKINIT_KUNIT_TEST_MODULE 1
#define CONFIG_CRYPTO_USER_API_SKCIPHER 1
#define CONFIG_CRYPTO_FCRYPT_MODULE 1
#define CONFIG_CRYPTO_AES 1
#define CONFIG_SND_HDA_PREALLOC_SIZE 2048
#define CONFIG_USB_XUSBATM_MODULE 1
#define CONFIG_NFSD_V4_DELEG_TIMESTAMPS 1
#define CONFIG_I2C_CHARDEV_MODULE 1
#define CONFIG_BCM_NET_PHYPTP_MODULE 1
#define CONFIG_SENSORS_MAX31760_MODULE 1
#define CONFIG_VIDEO_VIM2M_MODULE 1
#define CONFIG_AF_UNIX_OOB 1
#define CONFIG_QCOM_NET_PHYLIB_MODULE 1
#define CONFIG_INPUT_JOYDEV_MODULE 1
#define CONFIG_DVB_MN88472_MODULE 1
#define CONFIG_HAVE_DYNAMIC_FTRACE 1
#define CONFIG_SECURITY_APPARMOR_EXPORT_BINARY 1
#define CONFIG_AFS_FS_MODULE 1
#define CONFIG_BT_VIRTIO_MODULE 1
#define CONFIG_ARCH_SUPPORTS_CRASH_DUMP 1
#define CONFIG_DEV_SYNC_PROBE_MODULE 1
#define CONFIG_VIRTIO_CONSOLE 1
#define CONFIG_MDIO_I2C_MODULE 1
#define CONFIG_PATA_ATP867X_MODULE 1
#define CONFIG_VIDEO_OV13858_MODULE 1
#define CONFIG_SMC_MODULE 1
#define CONFIG_IDPF_MODULE 1
#define CONFIG_DVB_OR51211_MODULE 1
#define CONFIG_MFD_ROHM_BD96801_MODULE 1
#define CONFIG_NET_VENDOR_SUN 1
#define CONFIG_NET_IFE_SKBTCINDEX_MODULE 1
#define CONFIG_USB_SERIAL_UPD78F0730_MODULE 1
#define CONFIG_HID_ROCCAT_MODULE 1
#define CONFIG_MAX34408_MODULE 1
#define CONFIG_ARCH_HAS_MEMBARRIER_CALLBACKS 1
#define CONFIG_ATH10K_DEBUGFS 1
#define CONFIG_BOOTPARAM_SOFTLOCKUP_PANIC 0
#define CONFIG_TCM_PSCSI_MODULE 1
#define CONFIG_HID_KYSONA_MODULE 1
#define CONFIG_IP6_NF_TARGET_SYNPROXY_MODULE 1
#define CONFIG_VIDEO_OV5647_MODULE 1
#define CONFIG_NET_SCH_HFSC_MODULE 1
#define CONFIG_SENSORS_DRIVETEMP_MODULE 1
#define CONFIG_GENERIC_MSI_IRQ 1
#define CONFIG_FB_SYS_COPYAREA 1
#define CONFIG_OF_DYNAMIC 1
#define CONFIG_POWER_SUPPLY 1
#define CONFIG_HAVE_MOD_ARCH_SPECIFIC 1
#define CONFIG_ROCKER_MODULE 1
#define CONFIG_DEVTMPFS 1
#define CONFIG_HAVE_DEBUG_KMEMLEAK 1
#define CONFIG_BLK_DEV_BSG 1
#define CONFIG_MEDIA_PCI_SUPPORT 1
#define CONFIG_AT803X_PHY_MODULE 1
#define CONFIG_EXTRA_TARGETS ""
#define CONFIG_MACVTAP_MODULE 1
#define CONFIG_IPE_PROP_FS_VERITY 1
#define CONFIG_IPV6_FOU_MODULE 1
#define CONFIG_DRM_DISPLAY_HDMI_HELPER 1
#define CONFIG_NETFILTER_XT_MATCH_PHYSDEV_MODULE 1
#define CONFIG_USB_ANNOUNCE_NEW_DEVICES 1
#define CONFIG_USB_NET_QMI_WWAN_MODULE 1
#define CONFIG_RT2800PCI_RT53XX 1
#define CONFIG_USB_USBNET_MODULE 1
#define CONFIG_INPUT_EVDEV 1
#define CONFIG_IPV6_ROUTE_INFO 1
#define CONFIG_TI_ADS1015_MODULE 1
#define CONFIG_HUGETLBFS 1
#define CONFIG_SCTP_DEFAULT_COOKIE_HMAC_SHA256 1
#define CONFIG_CIFS_XATTR 1
#define CONFIG_MHI_BUS_MODULE 1
#define CONFIG_MLX5_CORE_EN_DCB 1
#define CONFIG_ZRAM_BACKEND_842 1
#define CONFIG_SENSORS_NPCM7XX_MODULE 1
#define CONFIG_PATA_SERVERWORKS_MODULE 1
#define CONFIG_USB_NET_RNDIS_HOST_MODULE 1
#define CONFIG_NETWORK_FILESYSTEMS 1
#define CONFIG_PATA_SIL680_MODULE 1
#define CONFIG_MISC_RTSX_PCI_MODULE 1
#define CONFIG_XFS_QUOTA 1
#define CONFIG_RTW88_8814AE_MODULE 1
#define CONFIG_COMPACT_UNEVICTABLE_DEFAULT 1
#define CONFIG_ARCH_HAS_DEBUG_WX 1
#define CONFIG_VIDEO_HI556_MODULE 1
#define CONFIG_NET_EGRESS 1
#define CONFIG_SND_USB_CAIAQ_MODULE 1
#define CONFIG_MLX5_SW_STEERING 1
#define CONFIG_W1_MASTER_DS2490_MODULE 1
#define CONFIG_CROSS_MEMORY_ATTACH 1
#define CONFIG_LD_ORPHAN_WARN 1
#define CONFIG_EROFS_FS_BACKED_BY_FILE 1
#define CONFIG_USB_SERIAL_CH341_MODULE 1
#define CONFIG_COMMON_CLK_RS9_PCIE_MODULE 1
#define CONFIG_NVMEM_QNAP_MCU_EEPROM_MODULE 1
#define CONFIG_INET_TCP_DIAG 1
#define CONFIG_DRM_EXEC_MODULE 1
#define CONFIG_SENSORS_ASC7621_MODULE 1
#define CONFIG_MXC6255_MODULE 1
#define CONFIG_VIDEO_AK881X_MODULE 1
#define CONFIG_HID_NTRIG 1
#define CONFIG_ARCH_WANT_LD_ORPHAN_WARN 1
#define CONFIG_EROFS_FS_MODULE 1
#define CONFIG_VIDEO_SAA7164_MODULE 1
#define CONFIG_DTPM_DEVFREQ 1
#define CONFIG_RTC_DRV_STK17TA8_MODULE 1
#define CONFIG_SENSORS_QNAP_MCU_HWMON_MODULE 1
#define CONFIG_USB_NET_CDC_SUBSET_ENABLE_MODULE 1
#define CONFIG_NF_CONNTRACK_TFTP_MODULE 1
#define CONFIG_CMM 1
#define CONFIG_BPF_PRELOAD 1
#define CONFIG_PARAVIRT_TIME_ACCOUNTING 1
#define CONFIG_I2C_NVIDIA_GPU_MODULE 1
#define CONFIG_OBJAGG_MODULE 1
#define CONFIG_SND_GINA20_MODULE 1
#define CONFIG_DVB_BUDGET_CORE_MODULE 1
#define CONFIG_BLK_DEV_INTEGRITY 1
#define CONFIG_DM_INIT 1
#define CONFIG_PPPOL2TP_MODULE 1
#define CONFIG_CRYPTO_LIB_BENCHMARK_VISIBLE 1
#define CONFIG_BACKLIGHT_ARCXCNN_MODULE 1
#define CONFIG_SENSORS_DPS920AB_MODULE 1
#define CONFIG_EXTCON_LC824206XA_MODULE 1
#define CONFIG_VIDEO_MT9M114_MODULE 1
#define CONFIG_VIDEO 1
#define CONFIG_VIDEO_GC0310_MODULE 1
#define CONFIG_ALIM7101_WDT_MODULE 1
#define CONFIG_DRM_EXPORT_FOR_TESTS 1
#define CONFIG_RADIO_SAA7706H_MODULE 1
#define CONFIG_NET_ACT_CONNMARK_MODULE 1
#define CONFIG_GENERIC_EARLY_IOREMAP 1
#define CONFIG_RTL8821AE_MODULE 1
#define CONFIG_BT_RFCOMM_MODULE 1
#define CONFIG_XFRM_ESP_MODULE 1
#define CONFIG_CACHEFILES_ONDEMAND 1
#define CONFIG_PATA_OPTIDMA_MODULE 1
#define CONFIG_KEYBOARD_TM2_TOUCHKEY_MODULE 1
#define CONFIG_TRACE_SYSCALL_BUF_SIZE_DEFAULT 63
#define CONFIG_RTL8188EE_MODULE 1
#define CONFIG_CXL_SUSPEND 1
#define CONFIG_DEBUG_SECTION_MISMATCH 1
#define CONFIG_ZRAM_BACKEND_LZ4HC 1
#define CONFIG_CAN_SLCAN_MODULE 1
#define CONFIG_USB_GSPCA_NW80X_MODULE 1
#define CONFIG_SND_USB_CAIAQ_INPUT 1
#define CONFIG_RT2800USB_RT35XX 1
#define CONFIG_OF_FLATTREE 1
#define CONFIG_IIO_GTS_KUNIT_TEST_MODULE 1
#define CONFIG_GPIB_NI_USB_MODULE 1
#define CONFIG_SPEAKUP_SYNTH_SOFT_MODULE 1
#define CONFIG_IP_NF_MATCH_ECN_MODULE 1
#define CONFIG_DVB_B2C2_FLEXCOP_PCI_MODULE 1
#define CONFIG_SND_USB_VARIAX_MODULE 1
#define CONFIG_SERIAL_DEV_BUS 1
#define CONFIG_USB_RTL8150_MODULE 1
#define CONFIG_L2TP_MODULE 1
#define CONFIG_WWAN_DEBUGFS 1
#define CONFIG_CAN_ESD_USB_MODULE 1
#define CONFIG_GPIB_NI_PCI_ISA_MODULE 1
#define CONFIG_USB_SEVSEG_MODULE 1
#define CONFIG_DRM_SIMPLE_BRIDGE_MODULE 1
#define CONFIG_PPC_PSERIES 1
#define CONFIG_NETROM_MODULE 1
#define CONFIG_NETFILTER_XT_SET_MODULE 1
#define CONFIG_NF_TABLES_IPV6 1
#define CONFIG_BSD_PROCESS_ACCT 1
#define CONFIG_GAMEPORT_EMU10K1_MODULE 1
#define CONFIG_SND_SEQ_MIDI_MODULE 1
#define CONFIG_SCSI_STEX_MODULE 1
#define CONFIG_DVB_MN88473_MODULE 1
#define CONFIG_INPUT_FF_MEMLESS_MODULE 1
#define CONFIG_XZ_DEC_SPARC 1
#define CONFIG_VIDEO_OV64A40_MODULE 1
#define CONFIG_RXGK 1
#define CONFIG_B43_LEDS 1
#define CONFIG_NF_REJECT_IPV4_MODULE 1
#define CONFIG_MEMSTICK_MODULE 1
#define CONFIG_TOUCHSCREEN_TSC2007_IIO 1
#define CONFIG_MWIFIEX_PCIE_MODULE 1
#define CONFIG_LEDS_TRIGGER_INPUT_EVENTS_MODULE 1
#define CONFIG_TEST_BPF_MODULE 1
#define CONFIG_SERIAL_8250_EXTENDED 1
#define CONFIG_SYSTEM_BLACKLIST_AUTH_UPDATE 1
#define CONFIG_CC_HAS_NO_PROFILE_FN_ATTR 1
#define CONFIG_ATM_NICSTAR_MODULE 1
#define CONFIG_SENSORS_EMC1403_MODULE 1
#define CONFIG_MODULE_COMPRESS 1
#define CONFIG_CGROUP_MISC 1
#define CONFIG_NATSEMI_MODULE 1
#define CONFIG_NET_IPVTI_MODULE 1
#define CONFIG_GENERIC_IOMAP 1
#define CONFIG_TYPEC_WUSB3801_MODULE 1
#define CONFIG_SND_SERIAL_GENERIC_MODULE 1
#define CONFIG_OF_GPIO 1
#define CONFIG_MMA7660_MODULE 1
#define CONFIG_MPRLS0025PA_MODULE 1
#define CONFIG_MISC_RTSX_USB_MODULE 1
#define CONFIG_DVB_MB86A20S_MODULE 1
#define CONFIG_HOTPLUG_PCI_RPA_DLPAR_MODULE 1
#define CONFIG_OCTEON_EP_MODULE 1
#define CONFIG_RCU_CPU_STALL_TIMEOUT 60
#define CONFIG_NF_DUP_IPV6_MODULE 1
#define CONFIG_RCU_TRACE 1
#define CONFIG_IP_MROUTE_COMMON 1
#define CONFIG_TOUCHSCREEN_USB_EASYTOUCH 1
#define CONFIG_USB_EHCI_HCD 1
#define CONFIG_RTL8192DU_MODULE 1
#define CONFIG_STRICT_KERNEL_RWX 1
#define CONFIG_MMC_SDHCI_UHS2_MODULE 1
#define CONFIG_INFINIBAND_ON_DEMAND_PAGING 1
#define CONFIG_HAVE_ARCH_NVRAM_OPS 1
#define CONFIG_USB_SERIAL_QUALCOMM_MODULE 1
#define CONFIG_RTW88_SDIO_MODULE 1
#define CONFIG_LZO_COMPRESS 1
#define CONFIG_SPARSEMEM_VMEMMAP 1
#define CONFIG_MEDIA_TUNER_MT2266_MODULE 1
#define CONFIG_I2C_NFORCE2_MODULE 1
#define CONFIG_CRYPTO_STREEBOG_MODULE 1
#define CONFIG_HID_SUPPORT 1
#define CONFIG_CXL_PMEM_MODULE 1
#define CONFIG_CRYPTO_DEV_CHELSIO_MODULE 1
#define CONFIG_IP_NF_ARPFILTER_MODULE 1
#define CONFIG_MACB_MODULE 1
#define CONFIG_HID_SMARTJOYPLUS_MODULE 1
#define CONFIG_I2C_CCGX_UCSI_MODULE 1
#define CONFIG_ASYNC_PQ_MODULE 1
#define CONFIG_VFAT_FS_MODULE 1
#define CONFIG_LCD_CLASS_DEVICE_MODULE 1
#define CONFIG_NET_VENDOR_ROCKER 1
#define CONFIG_NFC_MICROREAD_I2C_MODULE 1
#define CONFIG_DMA_OPS_BYPASS 1
#define CONFIG_UBIFS_FS_XATTR 1
#define CONFIG_SECURITY_DMESG_RESTRICT 1
#define CONFIG_LEDS_GPIO_MODULE 1
#define CONFIG_CHELSIO_T1_1G 1
#define CONFIG_DVB_HOPPER_MODULE 1
#define CONFIG_CRYPTO_DEFLATE 1
#define CONFIG_LTR501_MODULE 1
#define CONFIG_E100_MODULE 1
#define CONFIG_MACSEC_MODULE 1
#define CONFIG_IP_ROUTE_MULTIPATH 1
#define CONFIG_MFD_RT4831_MODULE 1
#define CONFIG_KFENCE_STRESS_TEST_FAULTS 0
#define CONFIG_TIME_NS 1
#define CONFIG_QEDI_MODULE 1
#define CONFIG_DM_MULTIPATH_QL_MODULE 1
#define CONFIG_REGULATOR_PF9453_MODULE 1
#define CONFIG_MAGIC_SYSRQ_DEFAULT_ENABLE 0x0
#define CONFIG_VIDEO_MT9M001_MODULE 1
#define CONFIG_PACKET 1
#define CONFIG_LEDS_TRIGGER_TRANSIENT_MODULE 1
#define CONFIG_VGA_ARB_MAX_GPUS 16
#define CONFIG_NO_HZ_COMMON 1
#define CONFIG_ARCH_DISABLE_KASAN_INLINE 1
#define CONFIG_PPC_MSI_BITMAP 1
#define CONFIG_SCSI_ESAS2R_MODULE 1
#define CONFIG_MEMTEST 1
#define CONFIG_6LOWPAN_GHC_EXT_HDR_HOP_MODULE 1
#define CONFIG_DRM_VIRTIO_GPU_MODULE 1
#define CONFIG_USB_STORAGE_SDDR55_MODULE 1
#define CONFIG_CC_HAS_SANE_FUNCTION_ALIGNMENT 1
#define CONFIG_SENSORS_LM95234_MODULE 1
#define CONFIG_I8253_LOCK 1
#define CONFIG_STACKDEPOT_MAX_FRAMES 64
#define CONFIG_CHARLCD_BL_FLASH 1
#define CONFIG_BRIDGE_EBT_ARP_MODULE 1
#define CONFIG_KEXEC_ELF 1
#define CONFIG_SND_SERIAL_U16550_MODULE 1
#define CONFIG_FPGA_DFL_PCI_MODULE 1
#define CONFIG_FB_CFB_FILLRECT 1
#define CONFIG_BRIDGE_EBT_T_NAT_MODULE 1
#define CONFIG_NF_DEFRAG_IPV4_MODULE 1
#define CONFIG_HTMDUMP_MODULE 1
#define CONFIG_FAIR_GROUP_SCHED 1
#define CONFIG_ZSTD_DECOMPRESS 1
#define CONFIG_NETFILTER_ADVANCED 1
#define CONFIG_FAT_FS_MODULE 1
#define CONFIG_DRM_PANIC_FOREGROUND_COLOR 0xffffff
#define CONFIG_REGULATOR_MAX77503_MODULE 1
#define CONFIG_VDPA_USER_MODULE 1
#define CONFIG_RV_MON_SCHED 1
#define CONFIG_NVME_MULTIPATH 1
#define CONFIG_B43LEGACY_DMA_AND_PIO_MODE 1
#define CONFIG_PSTORE_DEFAULT_KMSG_BYTES 10240
#define CONFIG_VIDEO_CX231XX_RC 1
#define CONFIG_IP6_NF_TARGET_REJECT_MODULE 1
#define CONFIG_AIC79XX_CMDS_PER_DEVICE 4
#define CONFIG_MLX5_BRIDGE 1
#define CONFIG_RTC_DRV_R7301_MODULE 1
#define CONFIG_RTW89_8852BTE_MODULE 1
#define CONFIG_GENERIC_TIME_VSYSCALL 1
#define CONFIG_ARCH_SUPPORTS_PER_VMA_LOCK 1
#define CONFIG_MMU_GATHER_RCU_TABLE_FREE 1
#define CONFIG_SND_HDA_CODEC_ALC680_MODULE 1
#define CONFIG_SND_CS46XX_NEW_DSP 1
#define CONFIG_DVB_AS102_FE_MODULE 1
#define CONFIG_IR_IMON_DECODER_MODULE 1
#define CONFIG_DLN2_ADC_MODULE 1
#define CONFIG_NR_CPUS 512
#define CONFIG_V4L2_CCI_MODULE 1
#define CONFIG_PCI_PWRCTRL_SLOT_MODULE 1
#define CONFIG_ARCH_HIBERNATION_POSSIBLE 1
#define CONFIG_HID_ACCUTOUCH_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_E2I 1
#define CONFIG_HAVE_FUNCTION_TRACER 1
#define CONFIG_DW_I3C_MASTER_MODULE 1
#define CONFIG_ASYNC_MEMCPY_MODULE 1
#define CONFIG_MLXSW_PCI_MODULE 1
#define CONFIG_LINEAR_RANGES_TEST_MODULE 1
#define CONFIG_LOG_BUF_SHIFT 18
#define CONFIG_LEDS_TRIGGER_DEFAULT_ON_MODULE 1
#define CONFIG_IPV6_OPTIMISTIC_DAD 1
#define CONFIG_AUDIT 1
#define CONFIG_SQUASHFS_XATTR 1
#define CONFIG_BH1745_MODULE 1
#define CONFIG_NET_9P_MODULE 1
#define CONFIG_BT_HCIBTUSB_MODULE 1
#define CONFIG_USB_HUB_USB251XB_MODULE 1
#define CONFIG_TCP_CONG_SCALABLE_MODULE 1
#define CONFIG_SND_SEQ_DEVICE_MODULE 1
#define CONFIG_VEML3235_MODULE 1
#define CONFIG_DVB_DRX39XYJ_MODULE 1
#define CONFIG_RTW88_USB_MODULE 1
#define CONFIG_MOUSE_CYAPA_MODULE 1
#define CONFIG_VIDEO_TW9906_MODULE 1
#define CONFIG_BLK_DEV_ZONED_LOOP_MODULE 1
#define CONFIG_AD3552R_HS_MODULE 1
#define CONFIG_QED_MODULE 1
#define CONFIG_DAMON_RECLAIM 1
#define CONFIG_NVMEM_LAYOUT_U_BOOT_ENV_MODULE 1
#define CONFIG_SERIAL_ICOM_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_CONNTRACK_MODULE 1
#define CONFIG_NF_SOCKET_IPV4_MODULE 1
#define CONFIG_CAN_DEV 1
#define CONFIG_SND_SEQ_HRTIMER_DEFAULT 1
#define CONFIG_YELLOWFIN_MODULE 1
#define CONFIG_NVME_TCP_TLS 1
#define CONFIG_USB_ATM_MODULE 1
#define CONFIG_VSOCKETS_DIAG_MODULE 1
#define CONFIG_USB_LED_TRIG 1
#define CONFIG_NET_SCH_GRED_MODULE 1
#define CONFIG_DM_MULTIPATH_MODULE 1
#define CONFIG_CAN_J1939_MODULE 1
#define CONFIG_RDS_RDMA_MODULE 1
#define CONFIG_EVENT_TRACING 1
#define CONFIG_KRETPROBES 1
#define CONFIG_RV_MON_MAINTENANCE_EVENTS 1
#define CONFIG_BCM_NET_PHYLIB_MODULE 1
#define CONFIG_PAGE_POISONING 1
#define CONFIG_DRM_DISPLAY_HDMI_CEC_HELPER 1
#define CONFIG_SND_PPC 1
#define CONFIG_DVB_USB_CE6230_MODULE 1
#define CONFIG_PATA_CMD640_PCI_MODULE 1
#define CONFIG_SERIAL_8250_SHARE_IRQ 1
#define CONFIG_MLX5_HW_STEERING 1
#define CONFIG_LOCKD_MODULE 1
#define CONFIG_USB_SERIAL_CYPRESS_M8_MODULE 1
#define CONFIG_6LOWPAN_GHC_UDP_MODULE 1
#define CONFIG_HID_CREATIVE_SB0540_MODULE 1
#define CONFIG_BPF_LSM 1
#define CONFIG_BACKLIGHT_RT4831_MODULE 1
#define CONFIG_NVME_TARGET_RDMA_MODULE 1
#define CONFIG_IP_ROUTE_VERBOSE 1
#define CONFIG_COMPACTION 1
#define CONFIG_SND_HDA_CODEC_ALC861VD_MODULE 1
#define CONFIG_REGULATOR_RT4801_MODULE 1
#define CONFIG_FB_CFB_IMAGEBLIT 1
#define CONFIG_SCSI_BFA_FC_MODULE 1
#define CONFIG_HWPOISON_INJECT_MODULE 1
#define CONFIG_BRANCH_PROFILE_NONE 1
#define CONFIG_VIDEOBUF2_VMALLOC_MODULE 1
#define CONFIG_BRIDGE_EBT_IP_MODULE 1
#define CONFIG_NVMEM_U_BOOT_ENV_MODULE 1
#define CONFIG_INPUT 1
#define CONFIG_BIG_KEYS 1
#define CONFIG_WLAN_VENDOR_RSI 1
#define CONFIG_USB_NET_PLUSB_MODULE 1
#define CONFIG_OF_FPGA_REGION_MODULE 1
#define CONFIG_NFT_LOG_MODULE 1
#define CONFIG_GENWQE_PLATFORM_ERROR_RECOVERY 1
#define CONFIG_PWRSEQ_EMMC_MODULE 1
#define CONFIG_RADIO_ADAPTERS_MODULE 1
#define CONFIG_HAVE_OBJTOOL_MCOUNT 1
#define CONFIG_KPROBES_SANITY_TEST_MODULE 1
#define CONFIG_BRIDGE_EBT_REDIRECT_MODULE 1
#define CONFIG_FPGA_BRIDGE_MODULE 1
#define CONFIG_TCG_TPM2_HMAC 1
#define CONFIG_NET_TEST_MODULE 1
#define CONFIG_FUTEX_PRIVATE_HASH 1
#define CONFIG_FUEL_GAUGE_MM8013_MODULE 1
#define CONFIG_VXLAN_MODULE 1
#define CONFIG_RXKAD 1
#define CONFIG_SONY_FF 1
#define CONFIG_CAN_M_CAN_PCI_MODULE 1
#define CONFIG_RTW88_8723DE_MODULE 1
#define CONFIG_DEV_COREDUMP 1
#define CONFIG_HZ_300 1
#define CONFIG_CRYPTO_LIB_BLAKE2S_KUNIT_TEST_MODULE 1
#define CONFIG_DVB_USB_MODULE 1
#define CONFIG_OF_EARLY_FLATTREE 1
#define CONFIG_BATMAN_ADV_BATMAN_V 1
#define CONFIG_QAT_VFIO_PCI_MODULE 1
#define CONFIG_TYPEC_MUX_PTN36502_MODULE 1
#define CONFIG_DVB_B2C2_FLEXCOP_MODULE 1
#define CONFIG_GPIO_PCF857X_MODULE 1
#define CONFIG_I2C_OPAL_MODULE 1
#define CONFIG_TABLET_USB_HANWANG_MODULE 1
#define CONFIG_E1000_MODULE 1
#define CONFIG_FRAMEBUFFER_CONSOLE 1
#define CONFIG_SND_INDIGO_MODULE 1
#define CONFIG_CLK_FD_KUNIT_TEST_MODULE 1
#define CONFIG_ARCH_HAS_GENERIC_CRASHKERNEL_RESERVATION 1
#define CONFIG_JOYSTICK_XPAD_FF 1
#define CONFIG_CRYPTO_ENGINE_MODULE 1
#define CONFIG_POLYNOMIAL_MODULE 1
#define CONFIG_BLK_ICQ 1
#define CONFIG_MPL115_MODULE 1
#define CONFIG_NF_CONNTRACK_MARK 1
#define CONFIG_RTC_DRV_MSM6242_MODULE 1
#define CONFIG_HAVE_GCC_PLUGINS 1
#define CONFIG_USB_NET_SMSC95XX_MODULE 1
#define CONFIG_PATA_ARTOP_MODULE 1
#define CONFIG_FUSION_LAN_MODULE 1
#define CONFIG_MEMSTICK_TIFM_MS_MODULE 1
#define CONFIG_DVB_AF9033_MODULE 1
#define CONFIG_NO_HZ 1
#define CONFIG_VIDEO_GO7007_USB_MODULE 1
#define CONFIG_DP83869_PHY_MODULE 1
#define CONFIG_LIBIE_FWLOG_MODULE 1
#define CONFIG_MFD_ADP5585_MODULE 1
#define CONFIG_SCSI_LIB_KUNIT_TEST_MODULE 1
#define CONFIG_SENSORS_LT3074_MODULE 1
#define CONFIG_CEPH_FSCACHE 1
#define CONFIG_RING_BUFFER 1
#define CONFIG_NET_DSA_TAG_MTK_MODULE 1
#define CONFIG_D3323AA_MODULE 1
#define CONFIG_USB_HSIC_USB4604_MODULE 1
#define CONFIG_TOUCHSCREEN_TSC2007_MODULE 1
#define CONFIG_DM_SNAPSHOT 1
#define CONFIG_USB_SERIAL_IUU_MODULE 1
#define CONFIG_DRM_ITE_IT6263_MODULE 1
#define CONFIG_NET_PKTGEN_MODULE 1
#define CONFIG_LLC_MODULE 1
#define CONFIG_IMA_NG_TEMPLATE 1
#define CONFIG_VIDEO_AU0828_MODULE 1
#define CONFIG_DYNAMIC_FTRACE_WITH_REGS 1
#define CONFIG_MFD_SYSCON 1
#define CONFIG_VIDEO_STK1160_MODULE 1
#define CONFIG_GPIOLIB_FASTPATH_LIMIT 512
#define CONFIG_MAC80211_LEDS 1
#define CONFIG_USB_NET_AQC111_MODULE 1
#define CONFIG_SECURITY_SELINUX_BOOTPARAM 1
#define CONFIG_JFS_SECURITY 1
#define CONFIG_PCIPCWATCHDOG_MODULE 1
#define CONFIG_NFC_MRVL_USB_MODULE 1
#define CONFIG_DRM_LONTIUM_LT8912B_MODULE 1
#define CONFIG_GENERIC_IRQ_SHOW_LEVEL 1
#define CONFIG_RT2800USB_MODULE 1
#define CONFIG_BT_BNEP_PROTO_FILTER 1
#define CONFIG_RESET_CONTROLLER 1
#define CONFIG_ARCH_SUPPORTS_HUGETLBFS 1
#define CONFIG_RTC_DRV_M41T80_WDT 1
#define CONFIG_PID_NS 1
#define CONFIG_DRM_GPUVM_MODULE 1
#define CONFIG_VIDEO_TC358743_MODULE 1
#define CONFIG_NVME_HWMON 1
#define CONFIG_IP6_NF_TARGET_NPT_MODULE 1
#define CONFIG_MB1232_MODULE 1
#define CONFIG_HASH_KUNIT_TEST_MODULE 1
#define CONFIG_ROHM_BU27034_MODULE 1
#define CONFIG_CGROUP_PERF 1
#define CONFIG_GCD_KUNIT_TEST_MODULE 1
#define CONFIG_INET6_ESP_OFFLOAD_MODULE 1
#define CONFIG_L2TP_ETH_MODULE 1
#define CONFIG_IGC_LEDS 1
#define CONFIG_VIDEO_MSP3400_MODULE 1
#define CONFIG_USB_PWC_MODULE 1
#define CONFIG_HARDIRQS_SW_RESEND 1
#define CONFIG_IIO_CONSUMERS_PER_TRIGGER 2
#define CONFIG_FRAMEBUFFER_CONSOLE_ROTATION 1
#define CONFIG_BNX2X_MODULE 1
#define CONFIG_ARCH_HAVE_NMI_SAFE_CMPXCHG 1
#define CONFIG_SSIF_IPMI_BMC_MODULE 1
#define CONFIG_NETFILTER_NETLINK_GLUE_CT 1
#define CONFIG_ARCH_OPTIONAL_KERNEL_RWX_DEFAULT 1
#define CONFIG_FORCE_PCI 1
#define CONFIG_ENIC_MODULE 1
#define CONFIG_SCSI_MPT2SAS_MAX_SGE 128
#define CONFIG_KVM_BOOK3S_HV_PMU_MODULE 1
#define CONFIG_FREEZER 1
#define CONFIG_INFINIBAND_USER_MEM 1
#define CONFIG_PPC_P7_NAP 1
#define CONFIG_DVB_USB_V2_MODULE 1
#define CONFIG_MYRI10GE_MODULE 1
#define CONFIG_VM_EVENT_COUNTERS 1
#define CONFIG_VLAN_8021Q_MODULE 1
#define CONFIG_HID_HAPTIC 1
#define CONFIG_AQTION_MODULE 1
#define CONFIG_ASSOCIATIVE_ARRAY 1
#define CONFIG_PNFS_FILE_LAYOUT_MODULE 1
#define CONFIG_RTL8723_COMMON_MODULE 1
#define CONFIG_GPIOLIB_LEGACY 1
#define CONFIG_MINIX_SUBPARTITION 1
#define CONFIG_IPE_PROP_DM_VERITY_SIGNATURE 1
#define CONFIG_EXT4_FS_SECURITY 1
#define CONFIG_SENSORS_SHT3x_MODULE 1
#define CONFIG_BRCMFMAC_USB 1
#define CONFIG_SUNRPC_SWAP 1
#define CONFIG_XFS_SUPPORT_V4 1
#define CONFIG_PCI_MSI_ARCH_FALLBACKS 1
#define CONFIG_TOUCHSCREEN_USB_DMC_TSC10 1
#define CONFIG_BRIDGE_EBT_VLAN_MODULE 1
#define CONFIG_SENSORS_G760A_MODULE 1
#define CONFIG_UNICODE_NORMALIZATION_KUNIT_TEST_MODULE 1
#define CONFIG_CRASH_RESERVE 1
#define CONFIG_SND_VIRTUOSO_MODULE 1
#define CONFIG_ATH_COMMON_MODULE 1
#define CONFIG_UACCE_MODULE 1
#define CONFIG_BLK_DEV 1
#define CONFIG_CRYPTO_SIG 1
#define CONFIG_RT61PCI_MODULE 1
#define CONFIG_DRM_BUDDY_MODULE 1
#define CONFIG_POWER_SEQUENCING_MODULE 1
#define CONFIG_BT_HCIBTUSB_POLL_SYNC 1
#define CONFIG_GPIO_DS4520_MODULE 1
#define CONFIG_USB_SERIAL_PL2303_MODULE 1
#define CONFIG_DRM_AUX_BRIDGE_MODULE 1
#define CONFIG_TOUCHSCREEN_MSG2638_MODULE 1
#define CONFIG_9P_FSCACHE 1
#define CONFIG_CGROUP_PIDS 1
#define CONFIG_VIDEO_CX88_MODULE 1
#define CONFIG_RMI4_F30 1
#define CONFIG_SND_HDA_INTEL_HDMI_SILENT_STREAM 1
#define CONFIG_NVDIMM_KEYS 1
#define CONFIG_SENSORS_IBMAEM_MODULE 1
#define CONFIG_SENSORS_W83L786NG_MODULE 1
#define CONFIG_SOCK_VALIDATE_XMIT 1
#define CONFIG_LIBETH_XDP_MODULE 1
#define CONFIG_DVB_USB_AZ6007_MODULE 1
#define CONFIG_REGULATOR_RT8092_MODULE 1
#define CONFIG_INET6_ESP_MODULE 1
#define CONFIG_FPROBE 1
#define CONFIG_CAN_GW_MODULE 1
#define CONFIG_NF_CONNTRACK_SIP_MODULE 1
#define CONFIG_RTW89_8852AU_MODULE 1
#define CONFIG_NET_RX_BUSY_POLL 1
#define CONFIG_PHYSICAL_START 0x00000000
#define CONFIG_INPUT_ATI_REMOTE2_MODULE 1
#define CONFIG_EDAC 1
#define CONFIG_PAHOLE_VERSION 0
#define CONFIG_SND_MPU401_MODULE 1
#define CONFIG_PKCS7_MESSAGE_PARSER 1
#define CONFIG_VIDEO_TVP514X_MODULE 1
#define CONFIG_LRU_GEN_ENABLED 1
#define CONFIG_ADXL355_MODULE 1
#define CONFIG_COMMON_CLK_AXI_CLKGEN_MODULE 1
#define CONFIG_TOUCHSCREEN_SILEAD_MODULE 1
#define CONFIG_PTP_1588_CLOCK_IDT82P33_MODULE 1
#define CONFIG_B43LEGACY_PCICORE_AUTOSELECT 1
#define CONFIG_MEDIA_SUPPORT_MODULE 1
#define CONFIG_I2C_DESIGNWARE_PLATFORM 1
#define CONFIG_SENSORS_TSL2550_MODULE 1
#define CONFIG_BPQETHER_MODULE 1
#define CONFIG_NET_VENDOR_SMSC 1
#define CONFIG_TRACE_CLOCK 1
#define CONFIG_R6040_MODULE 1
#define CONFIG_TCG_TIS_I2C_INFINEON_MODULE 1
#define CONFIG_HAVE_KVM_IRQ_BYPASS 1
#define CONFIG_AUXDISPLAY 1
#define CONFIG_USB_SERIAL_TI_MODULE 1
#define CONFIG_FPGA_MGR_XILINX_CORE_MODULE 1
#define CONFIG_BATTERY_MAX1720X_MODULE 1
#define CONFIG_NOZOMI_MODULE 1
#define CONFIG_VIDEO_GC0308_MODULE 1
#define CONFIG_RTC_INTF_SYSFS 1
#define CONFIG_PATA_HPT3X3_MODULE 1
#define CONFIG_FB_NOTIFY 1
#define CONFIG_SUN_PARTITION 1
#define CONFIG_QUOTA 1
#define CONFIG_VIA_RHINE_MODULE 1
#define CONFIG_TCG_TIS_I2C_CR50_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_TCPMSS_MODULE 1
#define CONFIG_NFT_TUNNEL_MODULE 1
#define CONFIG_DVB_L64781_MODULE 1
#define CONFIG_MT7663U_MODULE 1
#define CONFIG_TASKS_RCU_GENERIC 1
#define CONFIG_CRYPTO_SHA1 1
#define CONFIG_IP_SET_HASH_IPPORT_MODULE 1
#define CONFIG_PREEMPT_NOTIFIERS 1
#define CONFIG_VIDEO_S5K3M5_MODULE 1
#define CONFIG_UBIFS_FS_SECURITY 1
#define CONFIG_VIDEO_VP27SMPX_MODULE 1
#define CONFIG_SND_VIRTIO_MODULE 1
#define CONFIG_FUSION_LOGGING 1
#define CONFIG_NFT_DUP_IPV6_MODULE 1
#define CONFIG_FS_VERITY 1
#define CONFIG_RC_ATI_REMOTE_MODULE 1
#define CONFIG_FB_BACKLIGHT 1
#define CONFIG_SMC_DIAG_MODULE 1
#define CONFIG_JOYSTICK_PXRC_MODULE 1
#define CONFIG_CONTEXT_TRACKING 1
#define CONFIG_USB_CHAOSKEY_MODULE 1
#define CONFIG_MMC_SDHCI_CADENCE_MODULE 1
#define CONFIG_ULI526X_MODULE 1
#define CONFIG_DEVTMPFS_SAFE 1
#define CONFIG_NET_EMATCH_U32_MODULE 1
#define CONFIG_MEDIA_CEC_RC 1
#define CONFIG_USB_SERIAL_CYBERJACK_MODULE 1
#define CONFIG_GPIB_CEC_PCI_MODULE 1
#define CONFIG_VIDEO_BT866_MODULE 1
#define CONFIG_TOUCHSCREEN_SIS_I2C_MODULE 1
#define CONFIG_STRING_HELPERS_KUNIT_TEST_MODULE 1
#define CONFIG_VIDEO_UPD64083_MODULE 1
#define CONFIG_DVB_LGDT3305_MODULE 1
#define CONFIG_IP6_NF_MANGLE_MODULE 1
#define CONFIG_PAGE_IDLE_FLAG 1
#define CONFIG_CRYPTO_LZ4HC_MODULE 1
#define CONFIG_TASK_IO_ACCOUNTING 1
#define CONFIG_WLAN 1
#define CONFIG_PTP_DFL_TOD_MODULE 1
#define CONFIG_INET6_AH_MODULE 1
#define CONFIG_IEEE802154_6LOWPAN_MODULE 1
#define CONFIG_DRM_PANEL_ORIENTATION_QUIRKS 1
#define CONFIG_ATH10K_USB_MODULE 1
#define CONFIG_EEPROM_M24LR_MODULE 1
#define CONFIG_PPC_HASH_MMU_NATIVE 1
#define CONFIG_SOFTLOCKUP_DETECTOR 1
#define CONFIG_BT_HCIUART_BCSP 1
#define CONFIG_USB4 1
#define CONFIG_IP_NF_TARGET_TTL_MODULE 1
#define CONFIG_ARCH_HAS_TICK_BROADCAST 1
#define CONFIG_NETFILTER_XT_TARGET_NFQUEUE_MODULE 1
#define CONFIG_SND_HDA_CODEC_ALC268_MODULE 1
#define CONFIG_CHARGER_MAX77705_MODULE 1
#define CONFIG_NETFILTER_XTABLES_LEGACY 1
#define CONFIG_DVB_SP2_MODULE 1
#define CONFIG_SENSORS_NCT7802_MODULE 1
#define CONFIG_PWRSEQ_SIMPLE_MODULE 1
#define CONFIG_DVB_AU8522_MODULE 1
#define CONFIG_RTL8192CE_MODULE 1
#define CONFIG_DVB_CXD2820R_MODULE 1
#define CONFIG_SENSORS_MP29502_MODULE 1
#define CONFIG_FSI_MASTER_HUB_MODULE 1
#define CONFIG_NF_SOCKET_IPV6_MODULE 1
#define CONFIG_IWLMLD_MODULE 1
#define CONFIG_USB_MON 1
#define CONFIG_TIPC_DIAG_MODULE 1
#define CONFIG_DAVICOM_PHY_MODULE 1
#define CONFIG_INET6_XFRM_TUNNEL_MODULE 1
#define CONFIG_TSNEP_MODULE 1
#define CONFIG_NETWORK_SECMARK 1
#define CONFIG_MFD_DLN2_MODULE 1
#define CONFIG_SENSORS_SY7636A_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_NEXIO 1
#define CONFIG_BLK_DEV_IO_TRACE 1
#define CONFIG_MLXSW_CORE_MODULE 1
#define CONFIG_IOMMU_HELPER 1
#define CONFIG_SND_CMIPCI_MODULE 1
#define CONFIG_CRYPTO_ECDSA 1
#define CONFIG_COMMON_CLK_SI544_MODULE 1
#define CONFIG_KVM 1
#define CONFIG_NFT_FLOW_OFFLOAD_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_CONNLIMIT_MODULE 1
#define CONFIG_DEFAULT_MMAP_MIN_ADDR 4096
#define CONFIG_SENSORS_LM95241_MODULE 1
#define CONFIG_NLS_CODEPAGE_936_MODULE 1
#define CONFIG_GPIO_PCA953X_MODULE 1
#define CONFIG_CRYPTO_CMAC 1
#define CONFIG_TRANSPARENT_HUGEPAGE 1
#define CONFIG_BT_MSFTEXT 1
#define CONFIG_REGULATOR_RT4803_MODULE 1
#define CONFIG_WLAN_VENDOR_BROADCOM 1
#define CONFIG_NET_SCH_MULTIQ_MODULE 1
#define CONFIG_TOUCHSCREEN_ZET6223_MODULE 1
#define CONFIG_TOUCHSCREEN_HYNITRON_CSTXXX_MODULE 1
#define CONFIG_BPF_JIT_DEFAULT_ON 1
#define CONFIG_LEDS_LT3593_MODULE 1
#define CONFIG_SERIAL_8250_EXAR_MODULE 1
#define CONFIG_QLCNIC_SRIOV 1
#define CONFIG_ZRAM_WRITEBACK 1
#define CONFIG_NFS_V4_MODULE 1
#define CONFIG_DVB_TUNER_ITD1000_MODULE 1
#define CONFIG_NET_NSH_MODULE 1
#define CONFIG_VIDEO_OV4689_MODULE 1
#define CONFIG_HID_GOOGLE_STADIA_FF_MODULE 1
#define CONFIG_HID_LCPOWER_MODULE 1
#define CONFIG_F2FS_FS_SECURITY 1
#define CONFIG_SND_AC97_CODEC_MODULE 1
#define CONFIG_GENEVE_MODULE 1
#define CONFIG_DVB_NXT6000_MODULE 1
#define CONFIG_BLK_DEV_NBD_MODULE 1
#define CONFIG_KFENCE_NUM_OBJECTS 255
#define CONFIG_IP_SET_HASH_NET_MODULE 1
#define CONFIG_OLD_SIGSUSPEND 1
#define CONFIG_KCMP 1
#define CONFIG_RTC_DRV_DS1742_MODULE 1
#define CONFIG_DRM_NOUVEAU_CH7006_MODULE 1
#define CONFIG_XFRM_ALGO 1
#define CONFIG_NET_VENDOR_STMICRO 1
#define CONFIG_AX88796B_PHY_MODULE 1
#define CONFIG_LEDS_TRIGGER_TTY_MODULE 1
#define CONFIG_BUILD_SALT ""
#define CONFIG_SND_BT87X_MODULE 1
#define CONFIG_HID_NINTENDO_MODULE 1
#define CONFIG_SND_HDA_CODEC_CIRRUS_MODULE 1
#define CONFIG_GPIO_EXAR_MODULE 1
#define CONFIG_DUMMY_CONSOLE_ROWS 25
#define CONFIG_NVME_FABRICS_MODULE 1
#define CONFIG_FUNCTION_TRACER 1
#define CONFIG_MEDIA_CONTROLLER_DVB 1
#define CONFIG_AD5272_MODULE 1
#define CONFIG_DM_DEBUG 1
#define CONFIG_MODPROBE_PATH "/sbin/modprobe"
#define CONFIG_MLXSW_CORE_HWMON 1
#define CONFIG_RTL8187_MODULE 1
#define CONFIG_HID_MEGAWORLD_FF_MODULE 1
#define CONFIG_POSIX_MQUEUE 1
#define CONFIG_LEDS_GROUP_MULTICOLOR_MODULE 1
#define CONFIG_SENSORS_POWR1220_MODULE 1
#define CONFIG_TEXTSEARCH_FSM_MODULE 1
#define CONFIG_VHOST_SCSI_MODULE 1
#define CONFIG_NFSD_SCSILAYOUT 1
#define CONFIG_RMI4_SMB_MODULE 1
#define CONFIG_MQ_IOSCHED_DEADLINE 1
#define CONFIG_HAVE_CLK 1
#define CONFIG_USB_SERIAL_SIERRAWIRELESS_MODULE 1
#define CONFIG_MAC_PARTITION 1
#define CONFIG_SECURITY_APPARMOR_HASH 1
#define CONFIG_DRM_AMD_SECURE_DISPLAY 1
#define CONFIG_DEBUG_FS 1
#define CONFIG_USB_ONBOARD_DEV_USB5744 1
#define CONFIG_MEDIA_CEC_SUPPORT 1
#define CONFIG_CRYPTO_LIB_MD5_ARCH 1
#define CONFIG_DRM_PANEL_MANTIX_MLAF057WE51_MODULE 1
#define CONFIG_RTC_DRV_DS1307_MODULE 1
#define CONFIG_PPTP_MODULE 1
#define CONFIG_PPC_RTAS 1
#define CONFIG_FUTEX 1
#define CONFIG_DM_VDO_MODULE 1
#define CONFIG_HID_GLORIOUS_MODULE 1
#define CONFIG_IP_SET_MAX 256
#define CONFIG_PPC_KUEP 1
#define CONFIG_ASYNC_CORE_MODULE 1
#define CONFIG_DRM_PANEL_FEIXIN_K101_IM2BA02_MODULE 1
#define CONFIG_HAVE_DYNAMIC_FTRACE_WITH_CALL_OPS 1
#define CONFIG_MODULE_SIG_KEY_TYPE_RSA 1
#define CONFIG_ATALK_MODULE 1
#define CONFIG_NF_TABLES_INET 1
#define CONFIG_TCM_USER2_MODULE 1
#define CONFIG_DRM_PANEL_SAMSUNG_S6E88A0_AMS452EF01_MODULE 1
#define CONFIG_VIDEO_TDA9840_MODULE 1
#define CONFIG_NEED_PER_CPU_EMBED_FIRST_CHUNK 1
#define CONFIG_JOYSTICK_XPAD_LEDS 1
#define CONFIG_RTC_DRV_DS1685 1
#define CONFIG_USB_LCD_MODULE 1
#define CONFIG_ARCH_HAS_CURRENT_STACK_POINTER 1
#define CONFIG_USB_SL811_HCD_ISO 1
#define CONFIG_SENSORS_CHIPCAP2_MODULE 1
#define CONFIG_FB_SYSMEM_HELPERS 1
#define CONFIG_USBIP_VHCI_NR_HCS 1
#define CONFIG_BE2NET_LANCER 1
#define CONFIG_HAVE_FUNCTION_ERROR_INJECTION 1
#define CONFIG_CAN_CTUCANFD_PCI_MODULE 1
#define CONFIG_RADIO_SHARK_MODULE 1
#define CONFIG_TOUCHSCREEN_PIXCIR_MODULE 1
#define CONFIG_SENSORS_MAX1668_MODULE 1
#define CONFIG_MEGARAID_MM_MODULE 1
#define CONFIG_ROHM_BD79124_MODULE 1
#define CONFIG_HAS_DMA 1
#define CONFIG_SOFTIRQ_ON_OWN_STACK 1
#define CONFIG_OPT3001_MODULE 1
#define CONFIG_SENSORS_TDA38640_MODULE 1
#define CONFIG_SND_MONA_MODULE 1
#define CONFIG_W1_SLAVE_DS2413_MODULE 1
#define CONFIG_MLX4_EN_MODULE 1
#define CONFIG_SX9324_MODULE 1
#define CONFIG_CRYPTO_USER_API 1
#define CONFIG_AUDITSYSCALL 1
#define CONFIG_STACKDEPOT 1
#define CONFIG_CHARGER_LTC4162L_MODULE 1
#define CONFIG_IP_SET_BITMAP_IPMAC_MODULE 1
#define CONFIG_SND_HDA_CODEC_ALC269_MODULE 1
#define CONFIG_INFINIBAND_IONIC_MODULE 1
#define CONFIG_GENERIC_BUG 1
#define CONFIG_XFS_MEMORY_BUFS 1
#define CONFIG_CRYPTO_LIB_BLAKE2B_KUNIT_TEST_MODULE 1
#define CONFIG_IP_VS_MODULE 1
#define CONFIG_SND_LOLA_MODULE 1
#define CONFIG_MOUSE_ELAN_I2C_I2C 1
#define CONFIG_USB_WDM_MODULE 1
#define CONFIG_TCP_CONG_ILLINOIS_MODULE 1
#define CONFIG_BT_RFCOMM_TTY 1
#define CONFIG_BT_ATH3K_MODULE 1
#define CONFIG_USBIP_CORE_MODULE 1
#define CONFIG_UNION_FIND 1
#define CONFIG_BRCMUTIL_MODULE 1
#define CONFIG_BTRFS_FS 1
#define CONFIG_KVFREE_RCU_BATCHED 1
#define CONFIG_SND_HDA_CODEC_REALTEK_MODULE 1
#define CONFIG_VIDEO_TW9910_MODULE 1
#define CONFIG_IP_VS_IPV6 1
#define CONFIG_CPU_FREQ_GOV_COMMON 1
#define CONFIG_PPP_MPPE_MODULE 1
#define CONFIG_SCSI_QLA_FC_MODULE 1
#define CONFIG_I2C_MUX_MULE_MODULE 1
#define CONFIG_CB710_CORE_MODULE 1
#define CONFIG_FUSE_IO_URING 1
#define CONFIG_SCHED_OMIT_FRAME_POINTER 1
#define CONFIG_CEPH_LIB_MODULE 1
#define CONFIG_LEGACY_DIRECT_IO 1
#define CONFIG_BATMAN_ADV_MCAST 1
#define CONFIG_LOCK_SPIN_ON_OWNER 1
#define CONFIG_HAVE_PERF_USER_STACK_DUMP 1
#define CONFIG_GPIO_IDIO_16_MODULE 1
#define CONFIG_VIDEO_ADV7842_MODULE 1
#define CONFIG_NVME_CORE 1
#define CONFIG_GPIB_MODULE 1
#define CONFIG_SVC_I3C_MASTER_MODULE 1
#define CONFIG_AMD_PHY_MODULE 1
#define CONFIG_USB_PCI_AMD 1
#define CONFIG_VIDEO_OV13B10_MODULE 1
#define CONFIG_SENSORS_AXI_FAN_CONTROL_MODULE 1
#define CONFIG_NF_CT_NETLINK_TIMEOUT_MODULE 1
#define CONFIG_OPTPROBES 1
#define CONFIG_B43LEGACY_DMA 1
#define CONFIG_SKB_EXTENSIONS 1
#define CONFIG_HUGETLB_PAGE 1
#define CONFIG_XFRM_INTERFACE_MODULE 1
#define CONFIG_HAVE_ARCH_RANDOMIZE_KSTACK_OFFSET 1
#define CONFIG_SERIAL_CORE 1
#define CONFIG_DVB_USB_GP8PSK_MODULE 1
#define CONFIG_ROSE_MODULE 1
#define CONFIG_NFS_USE_KERNEL_DNS 1
#define CONFIG_NO_HZ_FULL 1
#define CONFIG_SENSORS_MP5023_MODULE 1
#define CONFIG_LEDS_TRIGGERS 1
#define CONFIG_NET_SCH_TBF_MODULE 1
#define CONFIG_SECURITY_NETWORK_XFRM 1
#define CONFIG_BRCMSMAC_MODULE 1
#define CONFIG_UBIFS_FS_LZO 1
#define CONFIG_SENSORS_LTC2945_MODULE 1
#define CONFIG_EROFS_FS_ZIP_ACCEL 1
#define CONFIG_MLX5_CORE_MODULE 1
#define CONFIG_GENERIC_IRQ_STAT_SNAPSHOT 1
#define CONFIG_NET_KEY_MODULE 1
#define CONFIG_REMOTE_TARGET_MODULE 1
#define CONFIG_SERIAL_MCTRL_GPIO 1
#define CONFIG_NET_DSA_TAG_HELLCREEK_MODULE 1
#define CONFIG_USB_GSPCA_ZC3XX_MODULE 1
#define CONFIG_CRYPTO_CCM 1
#define CONFIG_SATA_PROMISE_MODULE 1
#define CONFIG_SCSI_INIA100_MODULE 1
#define CONFIG_QLCNIC_MODULE 1
#define CONFIG_EROFS_FS_ZIP_DEFLATE 1
#define CONFIG_IP_VS_NQ_MODULE 1
#define CONFIG_PRESTERA_PCI_MODULE 1
#define CONFIG_CODA_FS_MODULE 1
#define CONFIG_SPEAKUP_SYNTH_AUDPTR_MODULE 1
#define CONFIG_SND_RAWMIDI_MODULE 1
#define CONFIG_JOYSTICK_SIDEWINDER_MODULE 1
#define CONFIG_GENERIC_GETTIMEOFDAY 1
#define CONFIG_IP_SET_HASH_IPMARK_MODULE 1
#define CONFIG_FW_UPLOAD 1
#define CONFIG_ATM_SOLOS_MODULE 1
#define CONFIG_QEDE_MODULE 1
#define CONFIG_SND_USB 1
#define CONFIG_VIDEO_DS90UB960_MODULE 1
#define CONFIG_NET_ACT_CT_MODULE 1
#define CONFIG_VIDEO_CX18_ALSA_MODULE 1
#define CONFIG_USB_GSPCA_TV8532_MODULE 1
#define CONFIG_DVB_USB_RTL28XXU_MODULE 1
#define CONFIG_PCS_XPCS_MODULE 1
#define CONFIG_IMA_DEFAULT_TEMPLATE "ima-ng"
#define CONFIG_ND_CLAIM 1
#define CONFIG_BRCMFMAC_PROTO_MSGBUF 1
#define CONFIG_GPIO_CDEV_V1 1
#define CONFIG_SYN_COOKIES 1
#define CONFIG_INFINIBAND_IPOIB_CM 1
#define CONFIG_ARCH_HAS_FORTIFY_SOURCE 1
#define CONFIG_DRM_PANEL_RENESAS_R69328_MODULE 1
#define CONFIG_MT7663S_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_SCTP_MODULE 1
#define CONFIG_HID_GEMBIRD_MODULE 1
#define CONFIG_NF_FLOW_TABLE_MODULE 1
#define CONFIG_CRYPTO_SHA3 1
#define CONFIG_NFC_SIM_MODULE 1
#define CONFIG_SND_PCMTEST_MODULE 1
#define CONFIG_ZRAM_DEF_COMP_LZORLE 1
#define CONFIG_STACKTRACE_SUPPORT 1
#define CONFIG_JOYSTICK_ADC_MODULE 1
#define CONFIG_CRYPTO_LIB_AES_ARCH 1
#define CONFIG_NFT_DUP_IPV4_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_CONNMARK_MODULE 1
#define CONFIG_PCIEASPM_DEFAULT 1
#define CONFIG_PPC_DOORBELL 1
#define CONFIG_FPGA_MODULE 1
#define CONFIG_SND_MODULE 1
#define CONFIG_MEGARAID_LEGACY_MODULE 1
#define CONFIG_MD_LINEAR_MODULE 1
#define CONFIG_CRYPTO_AEAD 1
#define CONFIG_PPC_BOOK3S_64 1
#define CONFIG_BCMGENET_MODULE 1
#define CONFIG_CHARGER_RT9471_MODULE 1
#define CONFIG_BLK_DEV_THROTTLING 1
#define CONFIG_PDS_VFIO_PCI_MODULE 1
#define CONFIG_NET_DSA_TAG_LAN9303_MODULE 1
#define CONFIG_SECTION_MISMATCH_WARN_ONLY 1
#define CONFIG_ASYNC_XOR_MODULE 1
#define CONFIG_PROC_CHILDREN 1
#define CONFIG_INET_MPTCP_DIAG 1
#define CONFIG_FPGA_DFL_AFU_MODULE 1
#define CONFIG_DRM_DISPLAY_HDCP_HELPER 1
#define CONFIG_CC_IS_GCC 1
#define CONFIG_SQUASHFS_FILE_DIRECT 1
#define CONFIG_FDMA 1
#define CONFIG_AX25_MODULE 1
#define CONFIG_VSOCKETS_LOOPBACK_MODULE 1
#define CONFIG_VIDEO_MXB_MODULE 1
#define CONFIG_BLK_DEV_UBLK_MODULE 1
#define CONFIG_CMDLINE ""
#define CONFIG_JOYSTICK_COBRA_MODULE 1
#define CONFIG_BLK_DEV_3W_XXXX_RAID_MODULE 1
#define CONFIG_RPMB_MODULE 1
#define CONFIG_IPV6_ROUTER_PREF 1
#define CONFIG_CGROUP_SCHED 1
#define CONFIG_DMA_ENGINE 1
#define CONFIG_PRIME_NUMBERS_MODULE 1
#define CONFIG_FPGA_REGION_MODULE 1
#define CONFIG_DA280_MODULE 1
#define CONFIG_MAX44009_MODULE 1
#define CONFIG_DVB_TDA10048_MODULE 1
#define CONFIG_F2FS_FS_COMPRESSION 1
#define CONFIG_EARLY_PRINTK 1
#define CONFIG_IP_SET_HASH_MAC_MODULE 1
#define CONFIG_SQUASHFS_COMPILE_DECOMP_MULTI_PERCPU 1
#define CONFIG_SCSI_IBMVSCSI_MODULE 1
#define CONFIG_NET_DSA_LOOP_MODULE 1
#define CONFIG_ARCH_SELECT_MEMORY_MODEL 1
#define CONFIG_CRYPTO_CRC32C 1
#define CONFIG_L2TP_IP_MODULE 1
#define CONFIG_DM_ZONED_MODULE 1
#define CONFIG_SND_INDIGODJ_MODULE 1
#define CONFIG_RD_BZIP2 1
#define CONFIG_SCSI_3W_9XXX_MODULE 1
#define CONFIG_NLS_CODEPAGE_1250_MODULE 1
#define CONFIG_MEDIA_TUNER_MAX2165_MODULE 1
#define CONFIG_DVB_LNBP22_MODULE 1
#define CONFIG_ECRYPT_FS_MODULE 1
#define CONFIG_FRAMEBUFFER_CONSOLE_DEFERRED_TAKEOVER 1
#define CONFIG_PHYS_ADDR_T_64BIT 1
#define CONFIG_UVC_COMMON_MODULE 1
#define CONFIG_HAVE_FUNCTION_GRAPH_TRACER 1
#define CONFIG_BPF_UNPRIV_DEFAULT_OFF 1
#define CONFIG_USB_GSPCA_OV534_9_MODULE 1
#define CONFIG_MFD_TPS6594_I2C_MODULE 1
#define CONFIG_SCSI_IPR_TRACE 1
#define CONFIG_CGROUP_BPF 1
#define CONFIG_ARCH_WEAK_RELEASE_ACQUIRE 1
#define CONFIG_MT76_LEDS 1
#define CONFIG_BT_AOSPEXT 1
#define CONFIG_NET_SCH_CBS_MODULE 1
#define CONFIG_INFINIBAND_ERDMA_MODULE 1
#define CONFIG_HIDRAW 1
#define CONFIG_MSPRO_BLOCK_MODULE 1
#define CONFIG_DVB_USB_AF9005_REMOTE_MODULE 1
#define CONFIG_CRYPTO_DES_MODULE 1
#define CONFIG_IR_NEC_DECODER_MODULE 1
#define CONFIG_LOCK_DEBUGGING_SUPPORT 1
#define CONFIG_VIDEOBUF2_DMA_CONTIG_MODULE 1
#define CONFIG_ATH10K_PCI_MODULE 1
#define CONFIG_MCTP 1
#define CONFIG_SCSI_ISCSI_ATTRS_MODULE 1
#define CONFIG_SECURITY_SELINUX_SID2STR_CACHE_SIZE 256
#define CONFIG_GENTOO_LINUX_PORTAGE 1
#define CONFIG_BUG 1
#define CONFIG_JFFS2_RTIME 1
#define CONFIG_DVB_AU8522_V4L_MODULE 1
#define CONFIG_IIO_MULTIPLY_KUNIT_TEST_MODULE 1
#define CONFIG_IR_HIX5HD2_MODULE 1
#define CONFIG_IPVTAP_MODULE 1
#define CONFIG_VIDEO_MT9P031_MODULE 1
#define CONFIG_TTPCI_EEPROM_MODULE 1
#define CONFIG_ARCH_HAS_GCOV_PROFILE_ALL 1
#define CONFIG_AF_KCM_MODULE 1
#define CONFIG_NFC_SHDLC 1
#define CONFIG_VIDEO_KS0127_MODULE 1
#define CONFIG_XZ_DEC_BCJ 1
#define CONFIG_USB_SERIAL_OTI6858_MODULE 1
#define CONFIG_DRM_KMS_HELPER_MODULE 1
#define CONFIG_CHELSIO_T4_DCB 1
#define CONFIG_NETFILTER_XT_TARGET_TCPOPTSTRIP_MODULE 1
#define CONFIG_USB_OHCI_HCD_PPC_OF 1
#define CONFIG_FB_SYSMEM_FOPS 1
#define CONFIG_CRYPTO_DEV_NX_COMPRESS 1
#define CONFIG_PAGE_REPORTING 1
#define CONFIG_SCHED_HRTICK 1
#define CONFIG_MIN_HEAP_KUNIT_TEST_MODULE 1
#define CONFIG_DRM_WAVESHARE_BRIDGE_MODULE 1
#define CONFIG_IMA_APPRAISE_BOOTPARAM 1
#define CONFIG_TRACING_SUPPORT 1
#define CONFIG_MPLS 1
#define CONFIG_SERIAL_SC16IS7XX_I2C_MODULE 1
#define CONFIG_STACK_TRACER 1
#define CONFIG_SND_UMP_LEGACY_RAWMIDI 1
#define CONFIG_NETFILTER_XT_TARGET_HL_MODULE 1
#define CONFIG_ISA_DMA_API 1
#define CONFIG_NET_ACT_SKBMOD_MODULE 1
#define CONFIG_LEDS_LP50XX_MODULE 1
#define CONFIG_NET_HANDSHAKE_KUNIT_TEST_MODULE 1
#define CONFIG_USB_EHCI_PCI 1
#define CONFIG_REGULATOR_RT5120_MODULE 1
#define CONFIG_GNSS_SIRF_SERIAL_MODULE 1
#define CONFIG_IP_SET_HASH_IP_MODULE 1
#define CONFIG_VIRTIO_MENU 1
#define CONFIG_RTC_DRV_RX8581_MODULE 1
#define CONFIG_BRIDGE_MRP 1
#define CONFIG_NVME_TARGET_TCP_TLS 1
#define CONFIG_NET_VENDOR_CADENCE 1
#define CONFIG_NFT_BRIDGE_META_MODULE 1
#define CONFIG_SPLIT_PTE_PTLOCKS 1
#define CONFIG_BLK_DEV_WRITE_MOUNTED 1
#define CONFIG_CRYPTO_DEV_NITROX_CNN55XX_MODULE 1
#define CONFIG_NET_VENDOR_BROCADE 1
#define CONFIG_MT76x02_USB_MODULE 1
#define CONFIG_USB_NET_SMSC75XX_MODULE 1
#define CONFIG_ROMFS_BACKED_BY_BLOCK 1
#define CONFIG_SENSORS_BH1770_MODULE 1
#define CONFIG_ALTERA_PR_IP_CORE_MODULE 1
#define CONFIG_ADXL372_MODULE 1
#define CONFIG_DRM_ST7571_MODULE 1
#define CONFIG_HAVE_IOREMAP_PROT 1
#define CONFIG_INT_LOG_KUNIT_TEST_MODULE 1
#define CONFIG_CC_NO_STRINGOP_OVERFLOW 1
#define CONFIG_SECURITY_APPARMOR 1
#define CONFIG_DRM_UDL_MODULE 1
#define CONFIG_DRM_PANEL_SAMSUNG_ATNA33XC20_MODULE 1
#define CONFIG_LEDS_TRIGGER_TIMER_MODULE 1
#define CONFIG_MMC_REALTEK_USB_MODULE 1
#define CONFIG_INTEGRITY_AUDIT 1
#define CONFIG_VIRTIO_FS_MODULE 1
#define CONFIG_B53_MMAP_DRIVER_MODULE 1
#define CONFIG_NET_SELFTESTS 1
#define CONFIG_ARCH_SUPPORTS_MEMORY_FAILURE 1
#define CONFIG_CRASH_HOTPLUG 1
#define CONFIG_DW_DMAC_MODULE 1
#define CONFIG_SENSORS_GL520SM_MODULE 1
#define CONFIG_CRYPTO_DEV_ATMEL_I2C_MODULE 1
#define CONFIG_ARCH_WANT_PMD_MKWRITE 1
#define CONFIG_USB_SERIAL_QCAUX_MODULE 1
#define CONFIG_6LOWPAN_DEBUGFS 1
#define CONFIG_I2C_STUB_MODULE 1
#define CONFIG_USB_SERIAL_GARMIN_MODULE 1
#define CONFIG_NET_DSA_MXL862_MODULE 1
#define CONFIG_QLA3XXX_MODULE 1
#define CONFIG_AS_HAS_NON_CONST_ULEB128 1
#define CONFIG_RTL8187_LEDS 1
#define CONFIG_F2FS_UNFAIR_RWSEM 1
#define CONFIG_CHARGER_RT9467_MODULE 1
#define CONFIG_LLD_VERSION 0
#define CONFIG_USB_STORAGE_ONETOUCH_MODULE 1
#define CONFIG_REMOTEPROC 1
#define CONFIG_OCFS2_FS_O2CB_MODULE 1
#define CONFIG_6LOWPAN_NHC_DEST_MODULE 1
#define CONFIG_W1_MODULE 1
#define CONFIG_CRYPTO_HASH 1
#define CONFIG_SND_USB_UA101_MODULE 1
#define CONFIG_GROUP_SCHED_BANDWIDTH 1
#define CONFIG_MLXSW_CORE_THERMAL 1
#define CONFIG_CRYPTO_AUTHENC 1
#define CONFIG_DRM_CLIENT_LIB_MODULE 1
#define CONFIG_ARCH_ENABLE_MEMORY_HOTPLUG 1
#define CONFIG_NET_ACT_GACT_MODULE 1
#define CONFIG_MEDIA_ATTACH 1
#define CONFIG_VIDEO_VICODEC_MODULE 1
#define CONFIG_IP_NF_TARGET_NETMAP_MODULE 1
#define CONFIG_IP_NF_MANGLE_MODULE 1
#define CONFIG_IIO_ST_ACCEL_I2C_3AXIS_MODULE 1
#define CONFIG_NF_CONNTRACK_SANE_MODULE 1
#define CONFIG_MD_BITMAP 1
#define CONFIG_NETFILTER_XT_MATCH_DEVGROUP_MODULE 1
#define CONFIG_IP6_NF_MATCH_EUI64_MODULE 1
#define CONFIG_SERIAL_ARC_MODULE 1
#define CONFIG_ATL1_MODULE 1
#define CONFIG_DAX 1
#define CONFIG_MEMREGION 1
#define CONFIG_IS_SIGNED_TYPE_KUNIT_TEST_MODULE 1
#define CONFIG_IOSM_MODULE 1
#define CONFIG_TCP_CONG_WESTWOOD_MODULE 1
#define CONFIG_NETFILTER_EGRESS 1
#define CONFIG_OF_PARTITION 1
#define CONFIG_L2TP_V3 1
#define CONFIG_DVB_USB_AU6610_MODULE 1
#define CONFIG_SYSTEM_TRUSTED_KEYS ""
#define CONFIG_ARCH_CPU_PROBE_RELEASE 1
#define CONFIG_CHARGER_SMB347_MODULE 1
#define CONFIG_IP_MROUTE_MULTIPLE_TABLES 1
#define CONFIG_NET_DSA_TAG_BRCM_COMMON_MODULE 1
#define CONFIG_FPGA_MGR_ALTERA_CVP_MODULE 1
#define CONFIG_NLS_CODEPAGE_1251_MODULE 1
#define CONFIG_HID_PRODIKEYS_MODULE 1
#define CONFIG_LEDS_PCA9532_MODULE 1
#define CONFIG_GPIOLIB_IRQCHIP 1
#define CONFIG_XILLYBUS_CLASS_MODULE 1
#define CONFIG_SPEAKUP_SYNTH_ACNTSA_MODULE 1
#define CONFIG_NET_SCH_SFQ_MODULE 1
#define CONFIG_SPEAKUP_SYNTH_BNS_MODULE 1
#define CONFIG_INPUT_TPS6594_PWRBUTTON_MODULE 1
#define CONFIG_SCSI_SAS_ATTRS_MODULE 1
#define CONFIG_FORTIFY_KUNIT_TEST_MODULE 1
#define CONFIG_SUNRPC_MODULE 1
#define CONFIG_NLS_MAC_CELTIC_MODULE 1
#define CONFIG_THERMAL_EMERGENCY_POWEROFF_DELAY_MS 0
#define CONFIG_NF_CONNTRACK_NETBIOS_NS_MODULE 1
#define CONFIG_NF_NAT_OVS 1
#define CONFIG_TYPEC_RT1719_MODULE 1
#define CONFIG_DVB_STV0900_MODULE 1
#define CONFIG_RMI4_F34 1
#define CONFIG_CHELSIO_LIB_MODULE 1
#define CONFIG_FS_STACK 1
#define CONFIG_LEDS_BRIGHTNESS_HW_CHANGED 1
#define CONFIG_SENSORS_LTC2947_MODULE 1
#define CONFIG_MT76x2E_MODULE 1
#define CONFIG_SOFTLOCKUP_DETECTOR_INTR_STORM 1
#define CONFIG_VIDEO_MAX96717_MODULE 1
#define CONFIG_ARCH_SELECTS_KEXEC_FILE 1
#define CONFIG_CONNECTOR 1
#define CONFIG_SENSORS_TC654_MODULE 1
#define CONFIG_VIRTIO_PCI_LIB 1
#define CONFIG_DRM_VGEM_MODULE 1
#define CONFIG_HSR_MODULE 1
#define CONFIG_TCP_MD5SIG 1
#define CONFIG_MCTP_TRANSPORT_I3C_MODULE 1
#define CONFIG_INPUT_REGULATOR_HAPTIC_MODULE 1
#define CONFIG_SSB_BLOCKIO 1
#define CONFIG_REGULATOR_MP5416_MODULE 1
#define CONFIG_DM_UEVENT 1
#define CONFIG_UBIFS_FS_ZSTD 1
#define CONFIG_MEMORY_HOTREMOVE 1
#define CONFIG_CRYPTO_LIB_CHACHA 1
#define CONFIG_PAC1934_MODULE 1
#define CONFIG_BRIDGE_EBT_LOG_MODULE 1
#define CONFIG_SFP_MODULE 1
#define CONFIG_CPU_IDLE 1
#define CONFIG_CGROUP_FREEZER 1
#define CONFIG_EROFS_FS_ZIP_ZSTD 1
#define CONFIG_RT2X00_MODULE 1
#define CONFIG_PREEMPT_COUNT 1
#define CONFIG_GPIB_AGILENT_82350B_MODULE 1
#define CONFIG_SENSORS_TSC1641_MODULE 1
#define CONFIG_CAN 1
#define CONFIG_CRYPTO_LIB_BLAKE2B 1
#define CONFIG_TI_ADS1100_MODULE 1
#define CONFIG_GVE_MODULE 1
#define CONFIG_IIO_CONFIGFS_MODULE 1
#define CONFIG_IP6_NF_IPTABLES_LEGACY_MODULE 1
#define CONFIG_MLX5_CLS_ACT 1
#define CONFIG_HID_BATTERY_STRENGTH 1
#define CONFIG_DVB_NXT200X_MODULE 1
#define CONFIG_NET_SCH_HHF_MODULE 1
#define CONFIG_RTAS_PROC 1
#define CONFIG_LEDS_CLASS_MULTICOLOR_MODULE 1
#define CONFIG_DW_EDMA_PCIE_MODULE 1
#define CONFIG_HID_SENSOR_TEMP_MODULE 1
#define CONFIG_IOMMU_DEFAULT_DMA_LAZY 1
#define CONFIG_HID_SENSOR_HUB_MODULE 1
#define CONFIG_PM_CLK 1
#define CONFIG_NET_XGRESS 1
#define CONFIG_SND_VX222_MODULE 1
#define CONFIG_SATA_HOST 1
#define CONFIG_CRASH_DM_CRYPT_CONFIGS 1
#define CONFIG_SND_PCM_TIMER 1
#define CONFIG_DRM_ANALOGIX_ANX6345_MODULE 1
#define CONFIG_SCSI_PROC_FS 1
#define CONFIG_LITEX_LITEETH_MODULE 1
#define CONFIG_DVB_STB6000_MODULE 1
#define CONFIG_PPC_64S_HASH_MMU 1
#define CONFIG_QED_ISCSI 1
#define CONFIG_CRYPTO_HW 1
#define CONFIG_TIPC_MODULE 1
#define CONFIG_NFT_SOCKET_MODULE 1
#define CONFIG_NETFILTER_NETLINK_QUEUE_MODULE 1
#define CONFIG_CFG80211_REQUIRE_SIGNED_REGDB 1
#define CONFIG_MPU3050_MODULE 1
#define CONFIG_KALLSYMS_ALL 1
#define CONFIG_COREDUMP 1
#define CONFIG_RELOCATABLE 1
#define CONFIG_PM_GENERIC_DOMAINS_OF 1
#define CONFIG_LEDS_TPS6131X_MODULE 1
#define CONFIG_SERIAL_JSM_MODULE 1
#define CONFIG_SATA_ACARD_AHCI_MODULE 1
#define CONFIG_CC_HAS_ZERO_CALL_USED_REGS 1
#define CONFIG_BRIDGE_EBT_PKTTYPE_MODULE 1
#define CONFIG_RC_DEVICES 1
#define CONFIG_B43_BUSES_BCMA_AND_SSB 1
#define CONFIG_JOYSTICK_GUILLEMOT_MODULE 1
#define CONFIG_ARCH_KEEP_MEMBLOCK 1
#define CONFIG_CRYPTO_HASH2 1
#define CONFIG_SECCOMP 1
#define CONFIG_SNET_VDPA_MODULE 1
#define CONFIG_NET_DSA_MV88E6XXX_PTP 1
#define CONFIG_DAMON 1
#define CONFIG_TIFM_CORE_MODULE 1
#define CONFIG_CRYPTO_LIB_GF128MUL 1
#define CONFIG_SPLIT_PMD_PTLOCKS 1
#define CONFIG_HID_MICROSOFT_MODULE 1
#define CONFIG_SGETMASK_SYSCALL 1
#define CONFIG_VIDEO_CAMERA_LENS 1
#define CONFIG_NVMEM_LAYOUT_ONIE_TLV_MODULE 1
#define CONFIG_IP6_NF_IPTABLES_MODULE 1
#define CONFIG_SHMEM 1
#define CONFIG_A11Y_BRAILLE_CONSOLE 1
#define CONFIG_IWLWIFI_OPMODE_MODULAR 1
#define CONFIG_VIDEO_TC358746_MODULE 1
#define CONFIG_TRACEFS_AUTOMOUNT_DEPRECATED 1
#define CONFIG_WIZNET_W5300_MODULE 1
#define CONFIG_IIO_BUFFER_DMAENGINE_MODULE 1
#define CONFIG_SENSORS_MAX17616_MODULE 1
#define CONFIG_NFT_CT_MODULE 1
#define CONFIG_RTL8723BS_MODULE 1
#define CONFIG_SND_ALS4000_MODULE 1
#define CONFIG_IO_URING_ZCRX 1
#define CONFIG_NET_ACT_IFE_MODULE 1
#define CONFIG_VSOCKMON_MODULE 1
#define CONFIG_DVB_DIB3000MC_MODULE 1
#define CONFIG_I2C_BOARDINFO 1
#define CONFIG_VIDEO_OV7740_MODULE 1
#define CONFIG_SERIO_ALTERA_PS2_MODULE 1
#define CONFIG_ATL2_MODULE 1
#define CONFIG_GPIO_BD9571MWV_MODULE 1
#define CONFIG_UIO_MODULE 1
#define CONFIG_SSB_PCIHOST 1
#define CONFIG_NET_VENDOR_ALTEON 1
#define CONFIG_ATA_VERBOSE_ERROR 1
#define CONFIG_SECURITY_APPARMOR_HASH_DEFAULT 1
#define CONFIG_SND_HDA_CODEC_CONEXANT_MODULE 1
#define CONFIG_PLDMFW 1
#define CONFIG_VIDEO_MAX9271_LIB_MODULE 1
#define CONFIG_SENSORS_TPS40422_MODULE 1
#define CONFIG_THERMAL_NETLINK 1
#define CONFIG_VIRTIO_ANCHOR 1
#define CONFIG_SKGE_MODULE 1
#define CONFIG_MT76_SDIO_MODULE 1
#define CONFIG_CHELSIO_T3_MODULE 1
#define CONFIG_LIBIE_ADMINQ_MODULE 1
#define CONFIG_SATA_SIL_MODULE 1
#define CONFIG_SND_TIMER_MODULE 1
#define CONFIG_NETLINK_DIAG 1
#define CONFIG_NLS_MAC_ICELAND_MODULE 1
#define CONFIG_SMS_SIANO_RC 1
#define CONFIG_PPC_DISABLE_WERROR 1
#define CONFIG_RPCSEC_GSS_KRB5_ENCTYPES_AES_SHA2 1
#define CONFIG_NLS_CODEPAGE_949_MODULE 1
#define CONFIG_SENSORS_W83L785TS_MODULE 1
#define CONFIG_SENSORS_XDPE152_MODULE 1
#define CONFIG_VIRTIO_BLK 1
#define CONFIG_CC_HAS_PREFIXED 1
#define CONFIG_KUNIT_MODULE 1
#define CONFIG_RTLWIFI_PCI_MODULE 1
#define CONFIG_GENTOO_LINUX 1
#define CONFIG_BRIDGE_MODULE 1
#define CONFIG_IRQ_FORCED_THREADING 1
#define CONFIG_VIDEO_GO7007_LOADER_MODULE 1
#define CONFIG_HARDLOCKUP_DETECTOR 1
#define CONFIG_CRYPTO_MANAGER2 1
#define CONFIG_NET_VENDOR_MICROCHIP 1
#define CONFIG_MESSAGE_LOGLEVEL_DEFAULT 4
#define CONFIG_SCSI_CXGB4_ISCSI_MODULE 1
#define CONFIG_IP_NF_FILTER_MODULE 1
#define CONFIG_MAX77620_WATCHDOG_MODULE 1
#define CONFIG_SQUASHFS_MODULE 1
#define CONFIG_WLAN_VENDOR_ATH 1
#define CONFIG_DRM_TI_DLPC3433_MODULE 1
#define CONFIG_SND_DUMMY_MODULE 1
#define CONFIG_DVB_S921_MODULE 1
#define CONFIG_SENSORS_LM95245_MODULE 1
#define CONFIG_FPGA_DFL_FME_MODULE 1
#define CONFIG_NLS_CODEPAGE_932_MODULE 1
#define CONFIG_WLAN_VENDOR_MARVELL 1
#define CONFIG_NET_SCH_MQPRIO_LIB_MODULE 1
#define CONFIG_SOCK_RX_QUEUE_MAPPING 1
#define CONFIG_AUXILIARY_BUS 1
#define CONFIG_DECOMPRESS_LZ4 1
#define CONFIG_VFIO_CONTAINER 1
#define CONFIG_PPC64_BOOT_WRAPPER 1
#define CONFIG_DTPM_CPU 1
#define CONFIG_DVB_CX24117_MODULE 1
#define CONFIG_USB_GSPCA_ETOMS_MODULE 1
#define CONFIG_ARCH_WANT_DEFAULT_BPF_JIT 1
#define CONFIG_GPIO_SIM_MODULE 1
#define CONFIG_NLS_ISO8859_1_MODULE 1
#define CONFIG_ARCH_HAS_KCOV 1
#define CONFIG_HVC_CONSOLE 1
#define CONFIG_HID_TOPSEED_MODULE 1
#define CONFIG_NET_EMATCH_META_MODULE 1
#define CONFIG_HFSPLUS_KUNIT_TEST_MODULE 1
#define CONFIG_DYNAMIC_FTRACE_WITH_DIRECT_CALLS 1
#define CONFIG_MPLS_IPTUNNEL_MODULE 1
#define CONFIG_KVM_XIVE 1
#define CONFIG_PM_SLEEP_DEBUG 1
#define CONFIG_VIDEO_MAX96714_MODULE 1
#define CONFIG_AFS_FSCACHE 1
#define CONFIG_IMA_LSM_RULES 1
#define CONFIG_USB_TRANCEVIBRATOR_MODULE 1
#define CONFIG_WLAN_VENDOR_ZYDAS 1
#define CONFIG_AD3552R_LIB_MODULE 1
#define CONFIG_SENSORS_SHT21_MODULE 1
#define CONFIG_USB_GSPCA_MR97310A_MODULE 1
#define CONFIG_PCNET32_MODULE 1
#define CONFIG_DTC 1
#define CONFIG_PATA_TIMINGS 1
#define CONFIG_NETFILTER_XT_MARK_MODULE 1
#define CONFIG_VIRT_CPU_ACCOUNTING 1
#define CONFIG_UTIL_MACROS_KUNIT_MODULE 1
#define CONFIG_USB_SERIAL_DIGI_ACCELEPORT_MODULE 1
#define CONFIG_ARCH_ENABLE_HUGEPAGE_MIGRATION 1
#define CONFIG_PSTORE 1
#define CONFIG_NET_CLS_ROUTE4_MODULE 1
#define CONFIG_TMPFS_INODE64 1
#define CONFIG_IIO_TRIGGERED_EVENT_MODULE 1
#define CONFIG_DVB_USB_AF9035_MODULE 1
#define CONFIG_SND_BCD2000_MODULE 1
#define CONFIG_NET_SCH_CHOKE_MODULE 1
#define CONFIG_SCSI_3W_SAS_MODULE 1
#define CONFIG_SPAPR_TCE_IOMMU 1
#define CONFIG_INTEL_XWAY_PHY_MODULE 1
#define CONFIG_RMI4_F55 1
#define CONFIG_EXCLUSIVE_SYSTEM_RAM 1
#define CONFIG_FW_LOADER_COMPRESS 1
#define CONFIG_CRYPTO_LIB_POLY1305_RSIZE 1
#define CONFIG_INIT_ENV_ARG_LIMIT 32
#define CONFIG_CACHEFILES_MODULE 1
#define CONFIG_QEDF_MODULE 1
#define CONFIG_STANDALONE 1
#define CONFIG_USB_SERIAL_CONSOLE 1
#define CONFIG_TEHUTI_TN40_MODULE 1
#define CONFIG_XFRM_MIGRATE 1
#define CONFIG_SND_DYNAMIC_MINORS 1
#define CONFIG_IP_PIMSM_V1 1
#define CONFIG_CRYPTO_DEV_QAT_C62X_MODULE 1
#define CONFIG_CC_HAS_AUTO_VAR_INIT_ZERO 1
#define CONFIG_SCSI_VIRTIO_MODULE 1
#define CONFIG_FPGA_DFL_NIOS_INTEL_PAC_N3000_MODULE 1
#define CONFIG_DM_FLAKEY_MODULE 1
#define CONFIG_RTC_DRV_MAX77686_MODULE 1
#define CONFIG_ZD1211RW_MODULE 1
#define CONFIG_SECURITY_INFINIBAND 1
#define CONFIG_VIDEO_GO7007_USB_S2250_BOARD_MODULE 1
#define CONFIG_FUN_CORE_MODULE 1
#define CONFIG_GPIB_COMMON_MODULE 1
#define CONFIG_USB_GSPCA_XIRLINK_CIT_MODULE 1
#define CONFIG_DVB_DRXK_MODULE 1
#define CONFIG_JFFS2_FS_POSIX_ACL 1
#define CONFIG_I2C_PCA_PLATFORM_MODULE 1
#define CONFIG_SND_HDA_CODEC_ALC880_MODULE 1
#define CONFIG_EROFS_FS_SECURITY 1
#define CONFIG_I2C_HID 1
#define CONFIG_PREEMPT_BUILD 1
#define CONFIG_BT_HCIUART_MRVL 1
#define CONFIG_TPS6594_ESM_MODULE 1
#define CONFIG_DRM_DISPLAY_HDMI_CEC_NOTIFIER_HELPER 1
#define CONFIG_MEM_SOFT_DIRTY 1
#define CONFIG_LOGIWHEELS_FF 1
#define CONFIG_DRM_AMDGPU_MODULE 1
#define CONFIG_DRM_KUNIT_TEST_MODULE 1
#define CONFIG_AF_RXRPC_MODULE 1
#define CONFIG_842_COMPRESS 1
#define CONFIG_SERIAL_ARC_NR_PORTS 1
#define CONFIG_DVB_USB_ANYSEE_MODULE 1
#define CONFIG_DEFAULT_TCP_CONG "cubic"
#define CONFIG_VL53L0X_I2C_MODULE 1
#define CONFIG_I2C_TINY_USB_MODULE 1
#define CONFIG_IR_TTUSBIR_MODULE 1
#define CONFIG_MPTCP_IPV6 1
#define CONFIG_BATTERY_SAMSUNG_SDI 1
#define CONFIG_NETFILTER_NETLINK_ACCT_MODULE 1
#define CONFIG_PRINTK_EXECUTION_CTX 1
#define CONFIG_HW_RANDOM_PSERIES_MODULE 1
#define CONFIG_VORTEX_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_L2TP_MODULE 1
#define CONFIG_MIPI_I3C_HCI_MODULE 1
#define CONFIG_VIDEO_PVRUSB2_SYSFS 1
#define CONFIG_IP6_NF_SECURITY_MODULE 1
#define CONFIG_USB_STORAGE_REALTEK_MODULE 1
#define CONFIG_NET_VENDOR_OKI 1
#define CONFIG_RTW88_8821CS_MODULE 1
#define CONFIG_WIL6210_MODULE 1
#define CONFIG_SECURITY_IPE 1
#define CONFIG_CRYPTO_CRC32_MODULE 1
#define CONFIG_TPS6594_PFSM_MODULE 1
#define CONFIG_MMC_BLOCK_MODULE 1
#define CONFIG_RTC_DRV_RX8025_MODULE 1
#define CONFIG_GCC_NO_STRINGOP_OVERFLOW 1
#define CONFIG_CONTIG_ALLOC 1
#define CONFIG_VIDEO_MT9V111_MODULE 1
#define CONFIG_SMBFS_MODULE 1
#define CONFIG_AX25_DAMA_SLAVE 1
#define CONFIG_SND_HDA_CODEC_ALC662_MODULE 1
#define CONFIG_PSERIES_ENERGY_MODULE 1
#define CONFIG_CRYPTO_LIB_MD5_KUNIT_TEST_MODULE 1
#define CONFIG_CPU_FREQ_GOV_ATTR_SET 1
#define CONFIG_SENSORS_CRPS_MODULE 1
#define CONFIG_RWSEM_SPIN_ON_OWNER 1
#define CONFIG_HAVE_BOOTMEM_INFO_NODE 1
#define CONFIG_ARCH_HAS_STRICT_MODULE_RWX 1
#define CONFIG_PPC_EMULATED_STATS 1
#define CONFIG_SCSI_ACARD_MODULE 1
#define CONFIG_DVB_DIB3000MB_MODULE 1
#define CONFIG_DVB_TS2020_MODULE 1
#define CONFIG_DRM_DISPLAY_DP_AUX_BUS_MODULE 1
#define CONFIG_LRU_GEN 1
#define CONFIG_WLAN_VENDOR_INTEL 1
#define CONFIG_VIDEO_CX88_MPEG_MODULE 1
#define CONFIG_CRYPTO_ESSIV_MODULE 1
#define CONFIG_USB_SERIAL_SPCP8X5_MODULE 1
#define CONFIG_FPGA_MGR_XILINX_SELECTMAP_MODULE 1
#define CONFIG_ATH5K_DEBUG 1
#define CONFIG_ARCH_HAS_KERNEL_FPU_SUPPORT 1
#define CONFIG_KEYBOARD_QT1070_MODULE 1
#define CONFIG_OPEN_ALLIANCE_HELPERS 1
#define CONFIG_HID_CMEDIA_MODULE 1
#define CONFIG_DVB_LNBP21_MODULE 1
#define CONFIG_RPCSEC_GSS_KRB5_ENCTYPES_AES_SHA1 1
#define CONFIG_HAVE_RELIABLE_STACKTRACE 1
#define CONFIG_HID_ICADE_MODULE 1
#define CONFIG_WATCHDOG_HRTIMER_PRETIMEOUT 1
#define CONFIG_DVB_AU8522_DTV_MODULE 1
#define CONFIG_RTC_DRV_TPS6594_MODULE 1
#define CONFIG_NET_VENDOR_ATHEROS 1
#define CONFIG_DVB_SP887X_MODULE 1
#define CONFIG_LEDS_TRIGGER_CAMERA_MODULE 1
#define CONFIG_REGULATOR_RTMV20_MODULE 1
#define CONFIG_DVB_TDA10021_MODULE 1
#define CONFIG_VIRTIO_MMIO_MODULE 1
#define CONFIG_ELFCORE 1
#define CONFIG_CARL9170_WPC 1
#define CONFIG_BLK_PM 1
#define CONFIG_HDC2010_MODULE 1
#define CONFIG_SND_VMASTER 1
#define CONFIG_SERIAL_8250_RSA 1
#define CONFIG_MLX5_CORE_EN 1
#define CONFIG_VHOST_ENABLE_FORK_OWNER_CONTROL 1
#define CONFIG_CIFS_DEBUG 1
#define CONFIG_USBIP_VHCI_HCD_MODULE 1
#define CONFIG_DVB_CX24116_MODULE 1
#define CONFIG_BRIDGE_EBT_DNAT_MODULE 1
#define CONFIG_POSIX_AUX_CLOCKS 1
#define CONFIG_NET_VENDOR_SILAN 1
#define CONFIG_MEDIA_TUNER_MXL5005S_MODULE 1
#define CONFIG_NET_DSA_MV88E6XXX_LEDS 1
#define CONFIG_OF_PMEM_MODULE 1
#define CONFIG_HID_MULTITOUCH_MODULE 1
#define CONFIG_SND_HDA_CODEC_SI3054_MODULE 1
#define CONFIG_REGMAP_KUNIT_MODULE 1
#define CONFIG_SENSORS_UCD9000_MODULE 1
#define CONFIG_TOUCHSCREEN_ZINITIX_MODULE 1
#define CONFIG_GNSS_MODULE 1
#define CONFIG_SND_VERBOSE_PROCFS 1
#define CONFIG_NET_SCH_INGRESS_MODULE 1
#define CONFIG_BRIDGE_EBT_AMONG_MODULE 1
#define CONFIG_MEDIA_TUNER_TDA827X_MODULE 1
#define CONFIG_LEDS_KTD202X_MODULE 1
#define CONFIG_GENERIC_CPU_AUTOPROBE 1
#define CONFIG_NR_IRQS 512
#define CONFIG_WLAN_VENDOR_ST 1
#define CONFIG_SOLARIS_X86_PARTITION 1
#define CONFIG_NIU_MODULE 1
#define CONFIG_VMGENID 1
#define CONFIG_ADIN_PHY_MODULE 1
#define CONFIG_RETHOOK 1
#define CONFIG_JFFS2_ZLIB 1
#define CONFIG_NFC_MICROREAD_MODULE 1
#define CONFIG_INPUT_MOUSEDEV 1
#define CONFIG_NET_SCH_CAKE_MODULE 1
#define CONFIG_NETDEVICES 1
#define CONFIG_FW_CS_DSP_MODULE 1
#define CONFIG_DRM_CLIENT_DEFAULT "fbdev"
#define CONFIG_LAN743X_MODULE 1
#define CONFIG_IIO_BUFFER_DMA_MODULE 1
#define CONFIG_NLS_CODEPAGE_874_MODULE 1
#define CONFIG_VIDEO_TEA6415C_MODULE 1
#define CONFIG_RTL8180_MODULE 1
#define CONFIG_USB_GSPCA_BENQ_MODULE 1
#define CONFIG_BMP280_MODULE 1
#define CONFIG_DRM_PANEL_NOVATEK_NT37801_MODULE 1
#define CONFIG_BLK_RQ_ALLOC_TIME 1
#define CONFIG_VIDEO_ADV7393_MODULE 1
#define CONFIG_INET_XFRM_TUNNEL_MODULE 1
#define CONFIG_INPUT_MISC 1
#define CONFIG_UBIFS_FS_ZLIB 1
#define CONFIG_INV_ICM45600_I2C_MODULE 1
#define CONFIG_NF_NAT_H323_MODULE 1
#define CONFIG_IP_PIMSM_V2 1
#define CONFIG_ETHTOOL_NETLINK 1
#define CONFIG_NVME_TARGET_LOOP_MODULE 1
#define CONFIG_VIRTIO_PCI_LIB_LEGACY 1
#define CONFIG_COMMON_CLK 1
#define CONFIG_USBPCWATCHDOG_MODULE 1
#define CONFIG_SND_RME96_MODULE 1
#define CONFIG_VIDEO_CX2341X_MODULE 1
#define CONFIG_RMI4_F3A 1
#define CONFIG_RT2X00_LIB_LEDS 1
#define CONFIG_RTC_DRV_DS1374_WDT 1
#define CONFIG_VIDEO_CADENCE_CSI2TX_MODULE 1
#define CONFIG_FSI_OCC_MODULE 1
#define CONFIG_HID_PICOLCD_FB 1
#define CONFIG_SCSI_SCAN_ASYNC 1
#define CONFIG_CRYPTO_BLOWFISH_MODULE 1
#define CONFIG_BCM54140_PHY_MODULE 1
#define CONFIG_RTW89_PCI_MODULE 1
#define CONFIG_SENSORS_IBM_CFFPS_MODULE 1
#define CONFIG_USB_PULSE8_CEC_MODULE 1
#define CONFIG_PHYLINK_MODULE 1
#define CONFIG_SENSORS_MPQ7932_REGULATOR 1
#define CONFIG_SATA_PMP 1
#define CONFIG_DMA_SHARED_BUFFER 1
#define CONFIG_MODULES 1
#define CONFIG_MLX5_VDPA 1
#define CONFIG_RT2800PCI_RT35XX 1
#define CONFIG_IPV6_SEG6_HMAC 1
#define CONFIG_SND_SEQ_VIRMIDI_MODULE 1
#define CONFIG_NF_NAT_MODULE 1
#define CONFIG_CACHESTAT_SYSCALL 1
#define CONFIG_SND_HDA_INPUT_BEEP 1
#define CONFIG_ARCH_HAS_PHYS_TO_DMA 1
#define CONFIG_NFS_DISABLE_UDP_SUPPORT 1
#define CONFIG_MAX_SKB_FRAGS 17
#define CONFIG_USERFAULTFD 1
#define CONFIG_CRYPTO_ACOMP2 1
#define CONFIG_SND_CA0106_MODULE 1
#define CONFIG_VIDEO_AU0828_V4L2 1
#define CONFIG_DM_WRITECACHE_MODULE 1
#define CONFIG_MTD_POWERNV_FLASH_MODULE 1
#define CONFIG_VIDEO_APTINA_PLL_MODULE 1
#define CONFIG_SENSORS_W83773G_MODULE 1
#define CONFIG_V4L2_CCI_I2C_MODULE 1
#define CONFIG_NF_NAT_TFTP_MODULE 1
#define CONFIG_RTW88_8723D_MODULE 1
#define CONFIG_PPS_CLIENT_GPIO_MODULE 1
#define CONFIG_INET_TABLE_PERTURB_ORDER 16
#define CONFIG_LEDS_TRIGGER_MTD 1
#define CONFIG_VIRTIO_INPUT_MODULE 1
#define CONFIG_IIO_SW_TRIGGER_MODULE 1
#define CONFIG_HID_LOGITECH_MODULE 1
#define CONFIG_SND_NM256_MODULE 1
#define CONFIG_IXGBEVF_IPSEC 1
#define CONFIG_SECURITY_APPARMOR_KUNIT_TEST_MODULE 1
#define CONFIG_RT2X00_LIB_USB_MODULE 1
#define CONFIG_CORTINA_PHY_MODULE 1
#define CONFIG_IIO_BUFFER 1
#define CONFIG_DVB_B2C2_FLEXCOP_USB_MODULE 1
#define CONFIG_INET_ESPINTCP 1
#define CONFIG_ISCSI_BOOT_SYSFS_MODULE 1
#define CONFIG_TCM_IBLOCK_MODULE 1
#define CONFIG_DRM_PANEL_FEIYANG_FY07024DI26A30D_MODULE 1
#define CONFIG_ATOMIC64_SELFTEST 1
#define CONFIG_HAVE_PAGE_SIZE_64KB 1
#define CONFIG_HID_SENSOR_MAGNETOMETER_3D_MODULE 1
#define CONFIG_NFC_PN544_I2C_MODULE 1
#define CONFIG_SENSORS_NCT7904_MODULE 1
#define CONFIG_NF_CONNTRACK_FTP_MODULE 1
#define CONFIG_MAXLINEAR_86110_PHY_MODULE 1
#define CONFIG_FWCTL_MLX5_MODULE 1
#define CONFIG_MMU_GATHER_TABLE_FREE 1
#define CONFIG_PERSISTENT_KEYRINGS 1
#define CONFIG_CHELSIO_T1_MODULE 1
#define CONFIG_PMS7003_MODULE 1
#define CONFIG_USB_GSPCA_TOPRO_MODULE 1
#define CONFIG_GPIB_LPVO_MODULE 1
#define CONFIG_SENSORS_PWM_FAN_MODULE 1
#define CONFIG_BITREVERSE 1
#define CONFIG_CRASH_DUMP 1
#define CONFIG_PSI 1
#define CONFIG_REGULATOR_TPS6594_MODULE 1
#define CONFIG_HAVE_ARCH_KASAN_VMALLOC 1
#define CONFIG_IPV6_NDISC_NODETYPE 1
#define CONFIG_DRM_TOSHIBA_TC358764_MODULE 1
#define CONFIG_EDAC_ECS 1
#define CONFIG_DRM_GUD_MODULE 1
#define CONFIG_HID_MONTEREY_MODULE 1
#define CONFIG_NET_VENDOR_PACKET_ENGINES 1
#define CONFIG_GFS2_FS_LOCKING_DLM 1
#define CONFIG_IIO_BACKEND_MODULE 1
#define CONFIG_USB_NET_NET1080_MODULE 1
#define CONFIG_INPUT_IQS269A_MODULE 1
#define CONFIG_AD4062_MODULE 1
#define CONFIG_BLK_DEV_SD 1
#define CONFIG_IIO_ST_GYRO_I2C_3AXIS_MODULE 1
#define CONFIG_MT76x0U_MODULE 1
#define CONFIG_USB_XHCI_PCI 1
#define CONFIG_RT2800USB_RT3573 1
#define CONFIG_SENSORS_VT8231_MODULE 1
#define CONFIG_HAVE_FTRACE_GRAPH_FUNC 1
#define CONFIG_ATH9K_BTCOEX_SUPPORT 1
#define CONFIG_TOUCHSCREEN_HYCON_HY46XX_MODULE 1
#define CONFIG_SSB_DRIVER_PCICORE_POSSIBLE 1
#define CONFIG_NFC_PN544_MODULE 1
#define CONFIG_MSI_BITMAP_SELFTEST 1
#define CONFIG_SCHED_INFO 1
#define CONFIG_FUSION_CTL_MODULE 1
#define CONFIG_IP_VS_LC_MODULE 1
#define CONFIG_NLS_ISO8859_3_MODULE 1
#define CONFIG_CC_OPTIMIZE_FOR_PERFORMANCE 1
#define CONFIG_W1_SLAVE_DS2780_MODULE 1
#define CONFIG_UDF_FS_MODULE 1
#define CONFIG_HAVE_KPROBES 1
#define CONFIG_NET_SCH_ETF_MODULE 1
#define CONFIG_NVME_FC_MODULE 1
#define CONFIG_IIO_INV_SENSORS_TIMESTAMP_MODULE 1
#define CONFIG_IGB_HWMON 1
#define CONFIG_DEV_DAX_KMEM_MODULE 1
#define CONFIG_CRYPTO_KRB5_MODULE 1
#define CONFIG_DEBUG_MEMORY_INIT 1
#define CONFIG_USB_SERIAL_KEYSPAN_PDA_MODULE 1
#define CONFIG_VIDEO_CX231XX_DVB_MODULE 1
#define CONFIG_RC_MAP_MODULE 1
#define CONFIG_RTLWIFI_USB_MODULE 1
#define CONFIG_RTL8192D_COMMON_MODULE 1
#define CONFIG_CM3605_MODULE 1
#define CONFIG_HAVE_PCSPKR_PLATFORM 1
#define CONFIG_OF_KUNIT_TEST_MODULE 1
#define CONFIG_NLS_UCS2_UTILS_MODULE 1
#define CONFIG_VIDEO_OV5693_MODULE 1
#define CONFIG_SND_HDA_CODEC_CA0110_MODULE 1
#define CONFIG_MEDIA_PLATFORM_DRIVERS 1
#define CONFIG_RADIO_TEA5764_MODULE 1
#define CONFIG_MEDIA_TEST_SUPPORT 1
#define CONFIG_SND_VX_LIB_MODULE 1
#define CONFIG_HID_JABRA_MODULE 1
#define CONFIG_ASN1_ENCODER 1
#define CONFIG_DVB_MB86A16_MODULE 1
#define CONFIG_CHELSIO_INLINE_CRYPTO 1
#define CONFIG_ETHOC_MODULE 1
#define CONFIG_LWTUNNEL 1
#define CONFIG_CRYPTO_DEV_NX_COMPRESS_PSERIES 1
#define CONFIG_SND_ICE1724_MODULE 1
#define CONFIG_SND_RIPTIDE_MODULE 1
#define CONFIG_BCM_VK_TTY 1
#define CONFIG_HAVE_HARDLOCKUP_DETECTOR_ARCH 1
#define CONFIG_VIDEO_SAA7185_MODULE 1
#define CONFIG_SND_HDA_SCODEC_COMPONENT_MODULE 1
#define CONFIG_MT7601U_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_BPF_MODULE 1
#define CONFIG_IWLEGACY_MODULE 1
#define CONFIG_FORCEDETH_MODULE 1
#define CONFIG_USB_SERIAL_IPAQ_MODULE 1
#define CONFIG_RTW88_8723CS_MODULE 1
#define CONFIG_LAN966X_SWITCH_MODULE 1
#define CONFIG_FS_ENCRYPTION_INLINE_CRYPT 1
#define CONFIG_PSERIES_CPUIDLE 1
#define CONFIG_LEDS_SY7802_MODULE 1
#define CONFIG_SSB_SDIOHOST 1
#define CONFIG_SND_HDA_CODEC_ALC882_MODULE 1
#define CONFIG_8139TOO_MODULE 1
#define CONFIG_CRYPTO_XCTR_MODULE 1
#define CONFIG_VIDEO_CX88_VP3054_MODULE 1
#define CONFIG_RMI4_F1A 1
#define CONFIG_TYPEC_UCSI_MODULE 1
#define CONFIG_SYSVIPC_SYSCTL 1
#define CONFIG_USB_SERIAL_OMNINET_MODULE 1
#define CONFIG_I2C_MUX_MODULE 1
#define CONFIG_DRM_PANEL 1
#define CONFIG_PATA_CMD64X_MODULE 1
#define CONFIG_AIC7XXX_CMDS_PER_DEVICE 4
#define CONFIG_FANOTIFY 1
#define CONFIG_DMADEVICES 1
#define CONFIG_TCP_CONG_YEAH_MODULE 1
#define CONFIG_FS_MBCACHE 1
#define CONFIG_HAVE_ARCH_TLB_REMOVE_TABLE 1
#define CONFIG_LEDS_PCA995X_MODULE 1
#define CONFIG_RTW88_8821CU_MODULE 1
#define CONFIG_GPIO_REGMAP_MODULE 1
#define CONFIG_BCM87XX_PHY_MODULE 1
#define CONFIG_INIT_ON_ALLOC_DEFAULT_ON 1
#define CONFIG_SQUASHFS_FRAGMENT_CACHE_SIZE 3
#define CONFIG_FA_DUMP 1
#define CONFIG_HASHTABLE_KUNIT_TEST_MODULE 1
#define CONFIG_PTP_1588_CLOCK_FC3W_MODULE 1
#define CONFIG_KSM 1
#define CONFIG_USB_UAS_MODULE 1
#define CONFIG_CAN_RX_OFFLOAD 1
#define CONFIG_SND_HWDEP_MODULE 1
#define CONFIG_AHCI_DWC_MODULE 1
#define CONFIG_PM_QOS_CPU_SYSTEM_WAKEUP 1
#define CONFIG_RC_XBOX_DVD_MODULE 1
#define CONFIG_CRYPTO_LIB_SHA256 1
#define CONFIG_IIO_ST_LSM6DSX_MODULE 1
#define CONFIG_AF_RXRPC_IPV6 1
#define CONFIG_PPC_DENORMALISATION 1
#define CONFIG_LZ4HC_COMPRESS_MODULE 1
#define CONFIG_SERIAL_8250_DMA 1
#define CONFIG_BT_HCIBTUSB_BCM 1
#define CONFIG_NET_IP_TUNNEL_MODULE 1
#define CONFIG_FXAS21002C_I2C_MODULE 1
#define CONFIG_I2C_DESIGNWARE_PCI_MODULE 1
#define CONFIG_SCSI_BNX2X_FCOE_MODULE 1
#define CONFIG_HAVE_PCI 1
#define CONFIG_DVB_TDA665x_MODULE 1
#define CONFIG_USERCOPY_KUNIT_TEST_MODULE 1
#define CONFIG_MEDIA_TUNER_QM1D1C0042_MODULE 1
#define CONFIG_DM_CLONE_MODULE 1
#define CONFIG_MLXSW_I2C_MODULE 1
#define CONFIG_NEED_SG_DMA_FLAGS 1
#define CONFIG_IP_NF_TARGET_SYNPROXY_MODULE 1
#define CONFIG_USB_KEENE_MODULE 1
#define CONFIG_RC_DECODERS 1
#define CONFIG_XMON_DEFAULT_RO_MODE 1
#define CONFIG_MT76x2_COMMON_MODULE 1
#define CONFIG_W1_SLAVE_SMEM_MODULE 1
#define CONFIG_GP2AP002_MODULE 1
#define CONFIG_RTW89_8852B_COMMON_MODULE 1
#define CONFIG_ISL29020_MODULE 1
#define CONFIG_6PACK_MODULE 1
#define CONFIG_BRIDGE_EBT_802_3_MODULE 1
#define CONFIG_CEC_GPIO_MODULE 1
#define CONFIG_DVB_USB_M920X_MODULE 1
#define CONFIG_DVB_TDA10023_MODULE 1
#define CONFIG_CHR_DEV_SG 1
#define CONFIG_SMC_HS_CTRL_BPF 1
#define CONFIG_RTC_DRV_RV3032_MODULE 1
#define CONFIG_TOUCHSCREEN_COLIBRI_VF50_MODULE 1
#define CONFIG_MMU 1
#define CONFIG_QTNFMAC_PCIE_MODULE 1
#define CONFIG_NET_DSA_HIRSCHMANN_HELLCREEK_MODULE 1
#define CONFIG_SSB_DRIVER_PCICORE 1
#define CONFIG_MEDIA_TUNER_MT2131_MODULE 1
#define CONFIG_DIBS_MODULE 1
#define CONFIG_IEEE802154_SOCKET_MODULE 1
#define CONFIG_HID_NVIDIA_SHIELD_MODULE 1
#define CONFIG_IP6_NF_MATCH_MH_MODULE 1
#define CONFIG_SECURITY 1
#define CONFIG_DRM_MIPI_DSI 1
#define CONFIG_FS_ENCRYPTION 1
#define CONFIG_IR_XMP_DECODER_MODULE 1
#define CONFIG_MEDIA_COMMON_OPTIONS 1
#define CONFIG_CRYPTO_RNG2 1
#define CONFIG_SENSORS_MP5920_MODULE 1
#define CONFIG_NLS_ISO8859_2_MODULE 1
#define CONFIG_HID_TIVO_MODULE 1
#define CONFIG_W1_SLAVE_DS2781_MODULE 1
#define CONFIG_DEBUG_STACKOVERFLOW 1
#define CONFIG_VIDEO_OV7640_MODULE 1
#define CONFIG_IPV6_GRE_MODULE 1
#define CONFIG_NET_FOU_IP_TUNNELS 1
#define CONFIG_DTL 1
#define CONFIG_FSI_SCOM_MODULE 1
#define CONFIG_DRM_LOAD_EDID_FIRMWARE 1
#define CONFIG_HAVE_DYNAMIC_FTRACE_WITH_REGS 1
#define CONFIG_KFENCE_KUNIT_TEST_MODULE 1
#define CONFIG_MINIX_FS_MODULE 1
#define CONFIG_SND_USB_AUDIO_MODULE 1
#define CONFIG_RC_CORE 1
#define CONFIG_ADXL380_I2C_MODULE 1
#define CONFIG_HAMACHI_MODULE 1
#define CONFIG_CRYPTO_DRBG_CTR 1
#define CONFIG_REGULATOR_TPS6286X_MODULE 1
#define CONFIG_NET_SCH_QFQ_MODULE 1
#define CONFIG_MANTIS_CORE_MODULE 1
#define CONFIG_IPE_POLICY_SIG_SECONDARY_KEYRING 1
#define CONFIG_DRM_AMD_DC_FP 1
#define CONFIG_TOUCHSCREEN_USB_ZYTRONIC 1
#define CONFIG_NFT_OSF_MODULE 1
#define CONFIG_BME680_I2C_MODULE 1
#define CONFIG_MOUSE_SYNAPTICS_USB_MODULE 1
#define CONFIG_BNGE_MODULE 1
#define CONFIG_VCAP 1
#define CONFIG_CFG80211_DEFAULT_PS 1
#define CONFIG_XFS_ONLINE_REPAIR 1
#define CONFIG_BT_LE_L2CAP_ECRED 1
#define CONFIG_MT76_CONNAC_LIB_MODULE 1
#define CONFIG_SND_SEQUENCER_MODULE 1
#define CONFIG_OPENVSWITCH_GENEVE_MODULE 1
#define CONFIG_XILLYBUS_MODULE 1
#define CONFIG_NET_TC_SKB_EXT 1
#define CONFIG_WLAN_VENDOR_MEDIATEK 1
#define CONFIG_CAN_CALC_BITTIMING 1
#define CONFIG_IPC_NS 1
#define CONFIG_ARCH_HAS_DEBUG_VIRTUAL 1
#define CONFIG_HID_PID 1
#define CONFIG_IOMMU_SUPPORT 1
#define CONFIG_MUX_GPIO_MODULE 1
#define CONFIG_GENTOO_LINUX_UDEV 1
#define CONFIG_IPMI_WATCHDOG_MODULE 1
#define CONFIG_NET_ACT_NAT_MODULE 1
#define CONFIG_BROADCOM_PHY_MODULE 1
#define CONFIG_VDPA_SIM_MODULE 1
#define CONFIG_DPLL 1
#define CONFIG_HID_MACALLY_MODULE 1
#define CONFIG_JOYSTICK_ADI_MODULE 1
#define CONFIG_ARCH_SUPPORTS_KEXEC_PURGATORY 1
#define CONFIG_MEMBARRIER 1
#define CONFIG_ENERGY_MODEL 1
#define CONFIG_INPUT_KUNIT_TEST_MODULE 1
#define CONFIG_B43_BCMA 1
#define CONFIG_VIDEO_TLV320AIC23B_MODULE 1
#define CONFIG_CHARGER_BQ2515X_MODULE 1
#define CONFIG_PCI_IOV 1
#define CONFIG_TOUCHSCREEN_NOVATEK_NVT_TS_MODULE 1
#define CONFIG_USB_VL600_MODULE 1
#define CONFIG_CPU_IDLE_GOV_MENU 1
#define CONFIG_INV_ICM42600_MODULE 1
#define CONFIG_NETKIT 1
#define CONFIG_SENSORS_XDP710_MODULE 1
#define CONFIG_GPIO_PCA9570_MODULE 1
#define CONFIG_JFFS2_FS_DEBUG 0
#define CONFIG_NETFILTER_XT_MATCH_HELPER_MODULE 1
#define CONFIG_OF_ADDRESS 1
#define CONFIG_CRC32_ARCH 1
#define CONFIG_I2C_MPC_MODULE 1
#define CONFIG_INFINIBAND_IPOIB_MODULE 1
#define CONFIG_VIDEO_LM3560_MODULE 1
#define CONFIG_USB_SERIAL_MCT_U232_MODULE 1
#define CONFIG_VIDEO_OV2640_MODULE 1
#define CONFIG_SECURITY_TOMOYO_ACTIVATION_TRIGGER "/usr/lib/systemd/systemd"
#define CONFIG_MFD_RSMU_I2C_MODULE 1
#define CONFIG_VSOCKETS_MODULE 1
#define CONFIG_CHECK_SIGNATURE 1
#define CONFIG_KEYBOARD_CYPRESS_SF_MODULE 1
#define CONFIG_QED_OOO 1
#define CONFIG_MEDIA_DIGITAL_TV_SUPPORT 1
#define CONFIG_SENSORS_EMC2305_MODULE 1
#define CONFIG_MXC4005_MODULE 1
#define CONFIG_NET_VENDOR_NETRONOME 1
#define CONFIG_NET_CLS_CGROUP 1
#define CONFIG_CRYPTO_DRBG_MENU 1
#define CONFIG_CLS_U32_MARK 1
#define CONFIG_READ_ONLY_THP_FOR_FS 1
#define CONFIG_SCSI_SNIC_MODULE 1
#define CONFIG_DVB_LNBH25_MODULE 1
#define CONFIG_BATTERY_CW2015_MODULE 1
#define CONFIG_MT7996E_MODULE 1
#define CONFIG_W1_SLAVE_THERM_MODULE 1
#define CONFIG_MAX30100_MODULE 1
#define CONFIG_DRM_TI_TFP410_MODULE 1
#define CONFIG_VHOST_IOTLB_MODULE 1
#define CONFIG_XZ_DEC 1
#define CONFIG_HSA_AMD 1
#define CONFIG_SENSORS_ADS7828_MODULE 1
#define CONFIG_SND_HDA_CODEC_ALC260_MODULE 1
#define CONFIG_USB_DSBR_MODULE 1
#define CONFIG_CIFS_ALLOW_INSECURE_LEGACY 1
#define CONFIG_USB_OHCI_BIG_ENDIAN_DESC 1
#define CONFIG_THERMAL_HWMON 1
#define CONFIG_HAVE_OBJTOOL 1
#define CONFIG_MLX5_EN_ARFS 1
#define CONFIG_DRM_OFDRM_MODULE 1
#define CONFIG_PCI_QUIRKS 1
#define CONFIG_MFD_CS40L50_I2C_MODULE 1
#define CONFIG_NET_DSA_TAG_BRCM_MODULE 1
#define CONFIG_NET_SCH_FQ_CODEL 1
#define CONFIG_IPV6_ILA_MODULE 1
#define CONFIG_PPC_SMP_MUXED_IPI 1
#define CONFIG_NET_VENDOR_FUNGIBLE 1
#define CONFIG_PATA_MARVELL_MODULE 1
#define CONFIG_MTD_NAND_NANDSIM_MODULE 1
#define CONFIG_MEDIA_TUNER_SI2157_MODULE 1
#define CONFIG_DW_DMAC_PCI_MODULE 1
#define CONFIG_NFS_COMMON_LOCALIO_SUPPORT_MODULE 1
#define CONFIG_BLK_CGROUP_IOPRIO 1
#define CONFIG_MD 1
#define CONFIG_I2C_VIRTIO_MODULE 1
#define CONFIG_PCPU_DEV_REFCNT 1
#define CONFIG_PVPANIC 1
#define CONFIG_I40E_MODULE 1
#define CONFIG_EDAC_ATOMIC_SCRUB 1
#define CONFIG_NET_VENDOR_AMAZON 1
#define CONFIG_MODULE_UNLOAD 1
#define CONFIG_NET_VENDOR_INTEL 1
#define CONFIG_THERMAL_DEFAULT_GOV_STEP_WISE 1
#define CONFIG_IMA_READ_POLICY 1
#define CONFIG_DVB_TDA8261_MODULE 1
#define CONFIG_ATH9K_RFKILL 1
#define CONFIG_FPGA_DFL_FME_MGR_MODULE 1
#define CONFIG_CONSOLE_LOGLEVEL_DEFAULT 7
#define CONFIG_RT2800USB_RT55XX 1
#define CONFIG_SPEAKUP_SYNTH_LTLK_MODULE 1
#define CONFIG_DVB_DIB7000M_MODULE 1
#define CONFIG_USB_GSPCA_T613_MODULE 1
#define CONFIG_REGULATOR_ADP5055_MODULE 1
#define CONFIG_VITESSE_PHY_MODULE 1
#define CONFIG_TCP_CONG_BIC_MODULE 1
#define CONFIG_IGBVF_MODULE 1
#define CONFIG_PAGE_COUNTER 1
#define CONFIG_CONSOLE_TRANSLATIONS 1
#define CONFIG_IP_VS_RR_MODULE 1
#define CONFIG_CPU_FREQ_GOV_POWERSAVE 1
#define CONFIG_CRYPTO_TWOFISH_COMMON_MODULE 1
#define CONFIG_USB_ACM_MODULE 1
#define CONFIG_DVB_CX22700_MODULE 1
#define CONFIG_MHP_DEFAULT_ONLINE_TYPE_OFFLINE 1
#define CONFIG_WATCHDOG_OPEN_TIMEOUT 0
#define CONFIG_BUILDTIME_TABLE_SORT 1
#define CONFIG_NLS_ISO8859_5_MODULE 1
#define CONFIG_DVB_USB_VP702X_MODULE 1
#define CONFIG_E1000E_MODULE 1
#define CONFIG_HAVE_CONTEXT_TRACKING_USER 1
#define CONFIG_IEEE802154_DRIVERS_MODULE 1
#define CONFIG_BINFMT_SCRIPT 1
#define CONFIG_OPT4060_MODULE 1
#define CONFIG_SENSORS_CORSAIR_PSU_MODULE 1
#define CONFIG_ARCH_HAS_COPY_MC 1
#define CONFIG_MD_RAID1_MODULE 1
#define CONFIG_VIRTIO_NET_MODULE 1
#define CONFIG_CRYPTO_SIG2 1
#define CONFIG_XFS_DRAIN_INTENTS 1
#define CONFIG_MAC80211_MODULE 1
#define CONFIG_DVB_STV0910_MODULE 1
#define CONFIG_THRUSTMASTER_FF 1
#define CONFIG_RTW88_8822CU_MODULE 1
#define CONFIG_MEDIA_TUNER_TDA18271_MODULE 1
#define CONFIG_MULTIUSER 1
#define CONFIG_CRYPTO_842 1
#define CONFIG_USB_AN2720 1
#define CONFIG_FONT_8x16 1
#define CONFIG_HWLAT_TRACER 1
#define CONFIG_NLS_MAC_ROMAN_MODULE 1
#define CONFIG_RTW88_8822C_MODULE 1
#define CONFIG_PPP_MODULE 1
#define CONFIG_ZOPT2201_MODULE 1
#define CONFIG_HID_SENSOR_CUSTOM_INTEL_HINGE_MODULE 1
#define CONFIG_NETFILTER_SKIP_EGRESS 1
#define CONFIG_TOOLS_SUPPORT_RELR 1
#define CONFIG_BRIDGE_IGMP_SNOOPING 1
#define CONFIG_SECURITY_SELINUX_AVC_STATS 1
#define CONFIG_USB_GSPCA_JL2005BCD_MODULE 1
#define CONFIG_VCNL4035_MODULE 1
#define CONFIG_I2C_DLN2_MODULE 1
#define CONFIG_CMA 1
#define CONFIG_SND_USB_TONEPORT_MODULE 1
#define CONFIG_IP6_NF_FILTER_MODULE 1
#define CONFIG_CRYPTO_SIMD_MODULE 1
#define CONFIG_HID_SAITEK_MODULE 1
#define CONFIG_REGULATOR_MAX77675_MODULE 1
#define CONFIG_DRM_PANEL_NOVATEK_NT35560_MODULE 1
#define CONFIG_IP_NF_ARPTABLES_MODULE 1
#define CONFIG_FFS_KUNIT_TEST_MODULE 1
#define CONFIG_ARCH_FORCE_MAX_ORDER 8
#define CONFIG_HID_VIVALDI_MODULE 1
#define CONFIG_MHI_WWAN_CTRL_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_MARK_MODULE 1
#define CONFIG_LAN966X_DCB 1
#define CONFIG_SCSI_DH 1
#define CONFIG_HID_ACRUX_MODULE 1
#define CONFIG_NVMEM_MAX77759_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_LENGTH_MODULE 1
#define CONFIG_RTC_DRV_M41T80_MODULE 1
#define CONFIG_TMPFS_XATTR 1
#define CONFIG_TOUCHSCREEN_USB_GUNZE 1
#define CONFIG_NEED_PER_CPU_PAGE_FIRST_CHUNK 1
#define CONFIG_BRCMFMAC_PCIE 1
#define CONFIG_SND_HDA_CODEC_HDMI_GENERIC_MODULE 1
#define CONFIG_DVB_BCM3510_MODULE 1
#define CONFIG_BLOCK 1
#define CONFIG_CYPRESS_FIRMWARE_MODULE 1
#define CONFIG_SND_LX6464ES_MODULE 1
#define CONFIG_PTP_1588_CLOCK_OPTIONAL 1
#define CONFIG_CRYPTO_JITTERENTROPY_MEMORY_BLOCKS 64
#define CONFIG_DM_BUFIO 1
#define CONFIG_HAVE_IMA_KEXEC 1
#define CONFIG_PRINTK_INDEX 1
#define CONFIG_SPEAKUP_SYNTH_TXPRT_MODULE 1
#define CONFIG_NET_VENDOR_3COM 1
#define CONFIG_MPTCP 1
#define CONFIG_TYPEC_TCPCI_MAXIM_MODULE 1
#define CONFIG_CPU_FREQ 1
#define CONFIG_IWLWIFI_KUNIT_TESTS_MODULE 1
#define CONFIG_FAT_DEFAULT_IOCHARSET "ascii"
#define CONFIG_SENSORS_DS620_MODULE 1
#define CONFIG_NFSD_PNFS 1
#define CONFIG_TTY 1
#define CONFIG_DVB_USB_ZD1301_MODULE 1
#define CONFIG_DECOMPRESS_LZMA 1
#define CONFIG_DVB_MANTIS_MODULE 1
#define CONFIG_IR_IGUANA_MODULE 1
#define CONFIG_GPIO_MAX77759_MODULE 1
#define CONFIG_DRM_SYSFB_HELPER_MODULE 1
#define CONFIG_CEC_NXP_TDA9950_MODULE 1
#define CONFIG_842_DECOMPRESS 1
#define CONFIG_FEALNX_MODULE 1
#define CONFIG_I40EVF_MODULE 1
#define CONFIG_CFS_BANDWIDTH 1
#define CONFIG_LEDS_LM3530_MODULE 1
#define CONFIG_B43_BCMA_PIO 1
#define CONFIG_SYMBOLIC_ERRNAME 1
#define CONFIG_SENSORS_LINEAGE_MODULE 1
#define CONFIG_DRM_PANEL_LG_LD070WX3_MODULE 1
#define CONFIG_VIDEO_CX88_ENABLE_VP3054 1
#define CONFIG_CAN_PEAK_USB_MODULE 1
#define CONFIG_LEDS_AW200XX_MODULE 1
#define CONFIG_CRYPTO_LIB_SHA3_KUNIT_TEST_MODULE 1
#define CONFIG_SENSORS_LTC2978_MODULE 1
#define CONFIG_TCP_CONG_BBR_MODULE 1
#define CONFIG_RV_REACT_PANIC 1
#define CONFIG_CRYPTO_MD4_MODULE 1
#define CONFIG_DRM_PANEL_NEWVISION_NV3051D_MODULE 1
#define CONFIG_SERIAL_8250_CONSOLE 1
#define CONFIG_LOCK_MM_AND_FIND_VMA 1
#define CONFIG_SOC_BUS 1
#define CONFIG_MODULE_SIG_HASH "sha512"
#define CONFIG_NETWORK_PHY_TIMESTAMPING 1
#define CONFIG_PROC_KCORE 1
#define CONFIG_ARCH_USE_CMPXCHG_LOCKREF 1
#define CONFIG_FW_LOADER_DEBUG 1
#define CONFIG_RTW88_LEDS 1
#define CONFIG_VIDEO_CX25840_MODULE 1
#define CONFIG_XFS_POSIX_ACL 1
#define CONFIG_BLK_MQ_STACKING 1
#define CONFIG_PPC_FPU 1
#define CONFIG_DRM_KUNIT_TEST_HELPERS_MODULE 1
#define CONFIG_LEDS_BLINKM_MULTICOLOR 1
#define CONFIG_SENSORS_NZXT_KRAKEN2_MODULE 1
#define CONFIG_HAVE_TRUSTED_KEYS 1
#define CONFIG_CARL9170_LEDS 1
#define CONFIG_NF_DUP_NETDEV_MODULE 1
#define CONFIG_NET_ACT_CSUM_MODULE 1
#define CONFIG_PPC_SVM 1
#define CONFIG_MEDIA_TUNER_TDA9887_MODULE 1
#define CONFIG_POWERCAP 1
#define CONFIG_I2C_MUX_GPMUX_MODULE 1
#define CONFIG_IOMMUFD_MODULE 1
#define CONFIG_TIMERLAT_TRACER 1
#define CONFIG_INET6_IPCOMP_MODULE 1
#define CONFIG_FW_LOADER_SYSFS 1
#define CONFIG_NFT_COMPAT_MODULE 1
#define CONFIG_NET_DSA_TAG_XRS700X_MODULE 1
#define CONFIG_HAVE_KERNEL_XZ 1
#define CONFIG_RTC_DRV_CMOS 1
#define CONFIG_CRYPTO_ALGAPI2 1
#define CONFIG_SQUASHFS_ZSTD 1
#define CONFIG_SCSI_PMCRAID_MODULE 1
#define CONFIG_BCM7XXX_PHY_MODULE 1
#define CONFIG_SENSORS_SBTSI_MODULE 1
#define CONFIG_CPU_MITIGATIONS 1
#define CONFIG_MEDIA_TUNER_R820T_MODULE 1
#define CONFIG_NLS_ISO8859_4_MODULE 1
#define CONFIG_RAID6_PQ 1
#define CONFIG_HID_WACOM_MODULE 1
#define CONFIG_SENSORS_MP2975_MODULE 1
#define CONFIG_MD_RAID0_MODULE 1
#define CONFIG_LEDS_NCP5623_MODULE 1
#define CONFIG_CLZ_TAB 1
#define CONFIG_PROC_MEM_ALWAYS_FORCE 1
#define CONFIG_INTERCONNECT_KUNIT_TEST_MODULE 1
#define CONFIG_BLKDEV_UBLK_LEGACY_OPCODES 1
#define CONFIG_SND_CS4281_MODULE 1
#define CONFIG_NVME_TARGET_MODULE 1
#define CONFIG_SND_HDA_PATCH_LOADER 1
#define CONFIG_FUNCTION_ALIGNMENT 4
#define CONFIG_F2FS_STAT_FS 1
#define CONFIG_VIRTIO_RTC_PTP 1
#define CONFIG_ATH9K_AHB 1
#define CONFIG_CFG80211_USE_KERNEL_REGDB_KEYS 1
#define CONFIG_NCN26000_PHY_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_PKTTYPE_MODULE 1
#define CONFIG_CRYPTO_XTS 1
#define CONFIG_NVMEM_LAYOUT_SL28_VPD_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_MAC_MODULE 1
#define CONFIG_IIO_SW_DEVICE_MODULE 1
#define CONFIG_INET6_ESPINTCP 1
#define CONFIG_DVB_TC90522_MODULE 1
#define CONFIG_LEDS_TRIGGER_HEARTBEAT_MODULE 1
#define CONFIG_ARCH_SUPPORTS_CRASH_HOTPLUG 1
#define CONFIG_INET_SCTP_DIAG_MODULE 1
#define CONFIG_RTW88_8822B_MODULE 1
#define CONFIG_PPS 1
#define CONFIG_NFC_PORT100_MODULE 1
#define CONFIG_INTERCONNECT 1
#define CONFIG_MAC80211_HAS_RC 1
#define CONFIG_CRASH_MAX_MEMORY_RANGES 8192
#define CONFIG_NLS_MAC_GREEK_MODULE 1
#define CONFIG_VIDEO_EM28XX_MODULE 1
#define CONFIG_GLOB 1
#define CONFIG_RTW89_8851BU_MODULE 1
#define CONFIG_OCTEON_EP_VF_MODULE 1
#define CONFIG_CAN_BCM_MODULE 1
#define CONFIG_PAC1921_MODULE 1
#define CONFIG_SCSI_SYM53C8XX_DMA_ADDRESSING_MODE 1
#define CONFIG_SENSORS_TMP421_MODULE 1
#define CONFIG_USB_GSPCA_MARS_MODULE 1
#define CONFIG_IP_ROUTE_CLASSID 1
#define CONFIG_ST_UVIS25_I2C_MODULE 1
#define CONFIG_VIDEO_VIVID_OSD 1
#define CONFIG_INFINIBAND_BNG_RE_MODULE 1
#define CONFIG_FM10K_MODULE 1
#define CONFIG_DEVMEM 1
#define CONFIG_USBIP_HOST_MODULE 1
#define CONFIG_VIDEO_IMX214_MODULE 1
#define CONFIG_NETFILTER_XT_CONNMARK_MODULE 1
#define CONFIG_RANDSTRUCT_KUNIT_TEST_MODULE 1
#define CONFIG_NFS_V4_SECURITY_LABEL 1
#define CONFIG_TRANSPARENT_HUGEPAGE_TMPFS_HUGE_NEVER 1
#define CONFIG_KEXEC_CORE 1
#define CONFIG_MFD_BD9571MWV_MODULE 1
#define CONFIG_NET_9P_FD_MODULE 1
#define CONFIG_MEDIA_TUNER_TDA18218_MODULE 1
#define CONFIG_VIDEO_DS90UB913_MODULE 1
#define CONFIG_DVB_DRXD_MODULE 1
#define CONFIG_LSM "landlock,yama,bpf"
#define CONFIG_ARCH_HAS_PKEYS 1
#define CONFIG_BMC150_ACCEL_I2C_MODULE 1
#define CONFIG_THREAD_SHIFT 14
#define CONFIG_HID_RETRODE_MODULE 1
#define CONFIG_USB_NET_MCS7830_MODULE 1
#define CONFIG_GNSS_USB_MODULE 1
#define CONFIG_FB_CFB_COPYAREA 1
#define CONFIG_CRYPTO_ALGAPI 1
#define CONFIG_ICP10100_MODULE 1
#define CONFIG_DYNAMIC_EVENTS 1
#define CONFIG_SND_PCXHR_MODULE 1
#define CONFIG_CLONE_BACKWARDS 1
#define CONFIG_DRM_PANEL_ORISETECH_OTM8009A_MODULE 1
#define CONFIG_SERIAL_EARLYCON 1
#define CONFIG_NF_NAT_IRC_MODULE 1
#define CONFIG_GET_FREE_REGION 1
#define CONFIG_DEVPORT 1
#define CONFIG_CRYPTO_HKDF 1
#define CONFIG_MFD_INTEL_M10_BMC_CORE_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_CT_MODULE 1
#define CONFIG_USB 1
#define CONFIG_N_HDLC_MODULE 1
#define CONFIG_DRM_PANEL_JDI_LPM102A188A_MODULE 1
#define CONFIG_ELF_CORE 1
#define CONFIG_PCI_HOST_COMMON 1
#define CONFIG_TASK_DELAY_ACCT 1
#define CONFIG_F2FS_FS_XATTR 1
#define CONFIG_DVB_ATBM8830_MODULE 1
#define CONFIG_DM_DELAY_MODULE 1
#define CONFIG_SCSI_QLOGIC_1280_MODULE 1
#define CONFIG_DVB_PLL_MODULE 1
#define CONFIG_THERMAL 1
#define CONFIG_NFC_ST21NFCA_MODULE 1
#define CONFIG_SELECT_MEMORY_MODEL 1
#define CONFIG_NETFILTER_BPF_LINK 1
#define CONFIG_DVB_LG2160_MODULE 1
#define CONFIG_BEFS_FS_MODULE 1
#define CONFIG_NET_VENDOR_DLINK 1
#define CONFIG_IP_SET_HASH_IPPORTNET_MODULE 1
#define CONFIG_I2C_MUX_MLXCPLD_MODULE 1
#define CONFIG_PPPOE_HASH_BITS 4
#define CONFIG_CAN_EMS_USB_MODULE 1
#define CONFIG_FTRACE_SYSCALLS 1
#define CONFIG_SND_HDA_CODEC_ALC262_MODULE 1
#define CONFIG_NET_TEAM_MODE_ROUNDROBIN_MODULE 1
#define CONFIG_CAN_VXCAN_MODULE 1
#define CONFIG_DVB_USB_NOVA_T_USB2_MODULE 1
#define CONFIG_SENSORS_LTC4245_MODULE 1
#define CONFIG_UCSI_STM32G0_MODULE 1
#define CONFIG_CRYPTO_MD5 1
#define CONFIG_SND_DRIVERS 1
#define CONFIG_PACKING_KUNIT_TEST_MODULE 1
#define CONFIG_SND_AC97_POWER_SAVE_DEFAULT 0
#define CONFIG_INPUT_CMA3000_MODULE 1
#define CONFIG_DVB_USB_CXUSB_ANALOG 1
#define CONFIG_SMSC9420_MODULE 1
#define CONFIG_ATH6KL_MODULE 1
#define CONFIG_INPUT_KEYSPAN_REMOTE_MODULE 1
#define CONFIG_ATH9K_HTC_MODULE 1
#define CONFIG_DM_AUDIT 1
#define CONFIG_NETFILTER_XT_MATCH_POLICY_MODULE 1
#define CONFIG_XZ_DEC_X86 1
#define CONFIG_SND_HDA_HWDEP 1
#define CONFIG_VT_CONSOLE_SLEEP 1
#define CONFIG_NLS_KOI8_U_MODULE 1
#define CONFIG_SCSI_SPI_ATTRS_MODULE 1
#define CONFIG_RUSTC_VERSION 0
#define CONFIG_CRYPTO_SERPENT_MODULE 1
#define CONFIG_SENSORS_NZXT_KRAKEN3_MODULE 1
#define CONFIG_KERNEL_GZIP 1
#define CONFIG_MCP4018_MODULE 1
#define CONFIG_DEBUG_KERNEL 1
#define CONFIG_RCU_NOCB_CPU 1
#define CONFIG_ASYNC_TX_DMA 1
#define CONFIG_SCSI_QLA_ISCSI_MODULE 1
#define CONFIG_NVME_TARGET_PASSTHRU 1
#define CONFIG_CAN_M_CAN_MODULE 1
#define CONFIG_MOUSE_ELAN_I2C_SMBUS 1
#define CONFIG_CEC_CORE_MODULE 1
#define CONFIG_WLAN_VENDOR_TI 1
#define CONFIG_EEPROM_93CX6_MODULE 1
#define CONFIG_FORCE_SMP 1
#define CONFIG_VIDEOBUF2_CORE_MODULE 1
#define CONFIG_DRM_TOSHIBA_TC358775_MODULE 1
#define CONFIG_ATM_CLIP_MODULE 1
#define CONFIG_MISC_FILESYSTEMS 1
#define CONFIG_USB_SERIAL_DEBUG_MODULE 1
#define CONFIG_HID_VIEWSONIC_MODULE 1
#define CONFIG_QUOTA_TREE 1
#define CONFIG_NFT_FIB_MODULE 1
#define CONFIG_NET_DSA_REALTEK_MODULE 1
#define CONFIG_CRYPTO_GHASH 1
#define CONFIG_NET_ACT_TUNNEL_KEY_MODULE 1
#define CONFIG_MDIO_BITBANG_MODULE 1
#define CONFIG_CGROUP_RDMA 1
#define CONFIG_ARCH_OPTIONAL_KERNEL_RWX 1
#define CONFIG_INOTIFY_USER 1
#define CONFIG_DVB_CX22702_MODULE 1
#define CONFIG_USB_GSPCA_VICAM_MODULE 1
#define CONFIG_SCOM_DEBUGFS 1
#define CONFIG_NLS_ISO8859_7_MODULE 1
#define CONFIG_ALTERA_MSGDMA_MODULE 1
#define CONFIG_PHYLIB 1
#define CONFIG_FAT_KUNIT_TEST_MODULE 1
#define CONFIG_B43_HWRNG 1
#define CONFIG_SLUB_KUNIT_TEST_MODULE 1
#define CONFIG_DVB_GP8PSK_FE_MODULE 1
#define CONFIG_ARCH_HAS_GIGANTIC_PAGE 1
#define CONFIG_FPGA_DFL_FME_REGION_MODULE 1
#define CONFIG_NET_EMATCH_IPT_MODULE 1
#define CONFIG_USB_MICROTEK_MODULE 1
#define CONFIG_HDMI 1
#define CONFIG_NET_UDP_TUNNEL_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_COMPOSITE_MODULE 1
#define CONFIG_DVB_USB_DIB3000MC_MODULE 1
#define CONFIG_DRM_GM12U320_MODULE 1
#define CONFIG_RTW88_8822CS_MODULE 1
#define CONFIG_NET_DSA_TAG_RTL8_4_MODULE 1
#define CONFIG_INPUT_TABLET 1
#define CONFIG_NCSI_OEM_CMD_GET_MAC 1
#define CONFIG_PPPOE_MODULE 1
#define CONFIG_LIBERTAS_THINFIRM_MODULE 1
#define CONFIG_HAVE_ARCH_SOFT_DIRTY 1
#define CONFIG_DP83867_PHY_MODULE 1
#define CONFIG_XILINX_GMII2RGMII_MODULE 1
#define CONFIG_TRANSPARENT_HUGEPAGE_MADVISE 1
#define CONFIG_CAN_GS_USB_MODULE 1
#define CONFIG_HAVE_DYNAMIC_FTRACE_WITH_ARGS 1
#define CONFIG_TEST_HMM_MODULE 1
#define CONFIG_BITFIELD_KUNIT_MODULE 1
#define CONFIG_PATA_AMD_MODULE 1
#define CONFIG_ARCH_HAS_SYSCALL_WRAPPER 1
#define CONFIG_WLAN_VENDOR_QUANTENNA 1
#define CONFIG_REED_SOLOMON_MODULE 1
#define CONFIG_PPC64_SUPPORTS_MEMORY_FAILURE 1
#define CONFIG_FAT_DEFAULT_CODEPAGE 437
#define CONFIG_RDMA_SIW_MODULE 1
#define CONFIG_VIDEO_OV01A10_MODULE 1
#define CONFIG_RMI4_2D_SENSOR 1
#define CONFIG_SENSORS_LM25066_MODULE 1
#define CONFIG_MMC_REALTEK_PCI_MODULE 1
#define CONFIG_NLATTR 1
#define CONFIG_SIPHASH_KUNIT_TEST_MODULE 1
#define CONFIG_SUNRPC_XPRT_RDMA_MODULE 1
#define CONFIG_DVB_SI21XX_MODULE 1
#define CONFIG_SLAB_BUCKETS 1
#define CONFIG_PDC_ADMA_MODULE 1
#define CONFIG_RT2800PCI_RT3290 1
#define CONFIG_HAVE_GUP_FAST 1
#define CONFIG_SENSORS_Q54SJ108A2_MODULE 1
#define CONFIG_MLX4_EN_DCB 1
#define CONFIG_TEST_KMOD_MODULE 1
#define CONFIG_NFT_REJECT_INET_MODULE 1
#define CONFIG_SND_OXYGEN_LIB_MODULE 1
#define CONFIG_REGULATOR_MAX8893_MODULE 1
#define CONFIG_CORDIC_MODULE 1
#define CONFIG_RTC_DRV_M48T35_MODULE 1
#define CONFIG_NUMA_BALANCING_DEFAULT_ENABLED 1
#define CONFIG_SCHED_MC 1
#define CONFIG_RTC_DRV_PCF2127_MODULE 1
#define CONFIG_TCP_CONG_ADVANCED 1
#define CONFIG_CEPH_FS_SECURITY_LABEL 1
#define CONFIG_MLX5_TC_CT 1
#define CONFIG_ARCH_HAS_CC_PLATFORM 1
#define CONFIG_USB_KAWETH_MODULE 1
#define CONFIG_USB_NET_CDC_MBIM_MODULE 1
#define CONFIG_SENSORS_LM90_MODULE 1
#define CONFIG_JOYSTICK_QWIIC_MODULE 1
#define CONFIG_MCTP_TRANSPORT_USB_MODULE 1
#define CONFIG_DE2104X_DSL 0
#define CONFIG_MMU_NOTIFIER 1
#define CONFIG_DRM_PANEL_ELIDA_KD35T133_MODULE 1
#define CONFIG_NFT_REJECT_NETDEV_MODULE 1
#define CONFIG_FUSION 1
#define CONFIG_CRYPTO_CAMELLIA_MODULE 1
#define CONFIG_HID_GFRM_MODULE 1
#define CONFIG_MEDIA_USB_SUPPORT 1
#define CONFIG_MOUSE_ELAN_I2C_MODULE 1
#define CONFIG_RTLWIFI_MODULE 1
#define CONFIG_HAVE_EBPF_JIT 1
#define CONFIG_USB_SERIAL_EMPEG_MODULE 1
#define CONFIG_NET_ACT_MIRRED_MODULE 1
#define CONFIG_NET_VENDOR_ADAPTEC 1
#define CONFIG_NET_VENDOR_WANGXUN 1
#define CONFIG_INTEGRITY 1
#define CONFIG_RFS_ACCEL 1
#define CONFIG_HAVE_LD_DEAD_CODE_DATA_ELIMINATION 1
#define CONFIG_SCSI_MPT3SAS_MODULE 1
#define CONFIG_YAM_MODULE 1
#define CONFIG_KALLSYMS 1
#define CONFIG_FUSION_MAX_SGE 128
#define CONFIG_LEDS_TRIGGER_ACTIVITY_MODULE 1
#define CONFIG_HID_PLANTRONICS_MODULE 1
#define CONFIG_HID_SEMITEK_MODULE 1
#define CONFIG_REGULATOR_RT4831_MODULE 1
#define CONFIG_SMS_SIANO_MDTV_MODULE 1
#define CONFIG_INPUT_E3X0_BUTTON_MODULE 1
#define CONFIG_MDIO_BUS 1
#define CONFIG_NVMEM_LAYOUTS 1
#define CONFIG_LEDS_LM3532_MODULE 1
#define CONFIG_FSI_SBEFIFO_MODULE 1
#define CONFIG_SCSI_MOD 1
#define CONFIG_SND_SEQ_UMP_CLIENT_MODULE 1
#define CONFIG_ZSMALLOC_CHAIN_SIZE 8
#define CONFIG_CUSE_MODULE 1
#define CONFIG_IIO_MODULE 1
#define CONFIG_ATH9K_HW_MODULE 1
#define CONFIG_FUNCTION_GRAPH_TRACER 1
#define CONFIG_BCMA_DRIVER_GPIO 1
#define CONFIG_DVB_BUDGET_CI_MODULE 1
#define CONFIG_I2C_HELPER_AUTO 1
#define CONFIG_HZ 300
#define CONFIG_TRACING 1
#define CONFIG_HAMRADIO 1
#define CONFIG_HVC_OPAL 1
#define CONFIG_MLX5_TC_SAMPLE 1
#define CONFIG_IWLWIFI_LEDS 1
#define CONFIG_CHECKPOINT_RESTORE 1
#define CONFIG_BLK_DEV_DM 1
#define CONFIG_FONT_TER16x32 1
#define CONFIG_ICE_HWMON 1
#define CONFIG_NF_CONNTRACK_LABELS 1
#define CONFIG_NETFILTER_NETLINK_OSF_MODULE 1
#define CONFIG_KVM_COMMON 1
#define CONFIG_W1_SLAVE_DS2406_MODULE 1
#define CONFIG_RUNTIME_TESTING_MENU 1
#define CONFIG_RTW88_PCI_MODULE 1
#define CONFIG_VIRTUALIZATION 1
#define CONFIG_MFD_VX855_MODULE 1
#define CONFIG_USB_SERIAL_NAVMAN_MODULE 1
#define CONFIG_CHELSIO_T4_MODULE 1
#define CONFIG_DRM_RADEON_MODULE 1
#define CONFIG_DVB_USB_AZ6027_MODULE 1
#define CONFIG_NET_CLS_FLOWER_MODULE 1
#define CONFIG_DVB_MT312_MODULE 1
#define CONFIG_TOUCHSCREEN_IQS5XX_MODULE 1
#define CONFIG_TASKSTATS 1
#define CONFIG_TARGET_CORE_MODULE 1
#define CONFIG_NFT_FIB_NETDEV_MODULE 1
#define CONFIG_IP6_NF_TARGET_MASQUERADE_MODULE 1
#define CONFIG_IDPF_SINGLEQ 1
#define CONFIG_PTDUMP 1
#define CONFIG_IIO_RESCALE_KUNIT_TEST_MODULE 1
#define CONFIG_SND_MIA_MODULE 1
#define CONFIG_VIDEO_IMX258_MODULE 1
#define CONFIG_MDIO_REGMAP_MODULE 1
#define CONFIG_CXL_REGION 1
#define CONFIG_ZONEFS_FS_MODULE 1
#define CONFIG_MT76_CORE_MODULE 1
#define CONFIG_MTDRAM_ERASE_SIZE 128
#define CONFIG_DYNAMIC_FTRACE_WITH_ARGS 1
#define CONFIG_SENSORS_MR75203_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_AUDIT_MODULE 1
#define CONFIG_ATH9K_DEBUGFS 1
#define CONFIG_CXL_PORT_MODULE 1
#define CONFIG_OPENVSWITCH_MODULE 1
#define CONFIG_MFD_MAX5970_MODULE 1
#define CONFIG_QLCNIC_DCB 1
#define CONFIG_IPV6_SIT_MODULE 1
#define CONFIG_RT2800_LIB_MMIO_MODULE 1
#define CONFIG_DMABUF_HEAPS 1
#define CONFIG_STE10XP_MODULE 1
#define CONFIG_FS_POSIX_ACL 1
#define CONFIG_DVB_CX24110_MODULE 1
#define CONFIG_VIDEO_I2C_MODULE 1
#define CONFIG_BLK_DEV_RAM_COUNT 16
#define CONFIG_INTERVAL_TREE_SPAN_ITER 1
#define CONFIG_NLS_ISO8859_6_MODULE 1
#define CONFIG_PPC_UDBG_16550 1
#define CONFIG_USB_STORAGE_USBAT_MODULE 1
#define CONFIG_SENSORS_ADC128D818_MODULE 1
#define CONFIG_RTW88_8821AU_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_DSCP_MODULE 1
#define CONFIG_USB_GSPCA_SQ905C_MODULE 1
#define CONFIG_MT7921U_MODULE 1
#define CONFIG_INET_RAW_DIAG 1
#define CONFIG_ADL8113_MODULE 1
#define CONFIG_DVB_TDA10086_MODULE 1
#define CONFIG_SND_HDA_CODEC_HDMI_INTEL_MODULE 1
#define CONFIG_VALIDATE_FS_PARSER 1
#define CONFIG_IR_IMON_MODULE 1
#define CONFIG_SLIP_COMPRESSED 1
#define CONFIG_PREEMPTION 1
#define CONFIG_BINARY_PRINTF 1
#define CONFIG_TIGON3_HWMON 1
#define CONFIG_HAVE_ARCH_KCSAN 1
#define CONFIG_HAVE_PERF_EVENTS_NMI 1
#define CONFIG_KERNEL_START 0xc000000000000000
#define CONFIG_VIDEO_RDACM20_MODULE 1
#define CONFIG_MFD_CORE_MODULE 1
#define CONFIG_TRACE_IRQFLAGS_SUPPORT 1
#define CONFIG_IPVLAN_L3S 1
#define CONFIG_CRYPTO_XXHASH 1
#define CONFIG_SENSORS_NCT7363_MODULE 1
#define CONFIG_IR_SHARP_DECODER_MODULE 1
#define CONFIG_PRP_DUP_DISCARD_KUNIT_TEST_MODULE 1
#define CONFIG_TIMERFD 1
#define CONFIG_RTC_HCTOSYS_DEVICE "rtc0"
#define CONFIG_SND_USB_POD_MODULE 1
#define CONFIG_UFS_FS_MODULE 1
#define CONFIG_USB_STORAGE_FREECOM_MODULE 1
#define CONFIG_DRM_SSD130X_MODULE 1
#define CONFIG_IIO_ST_LSM6DSX_I3C_MODULE 1
#define CONFIG_KEYS_REQUEST_CACHE 1
#define CONFIG_MLX5_VFIO_PCI_MODULE 1
#define CONFIG_SND_JACK_INPUT_DEV 1
#define CONFIG_JFFS2_FS_MODULE 1
#define CONFIG_DRM_CLIENT_SETUP 1
#define CONFIG_VIDEO_OG0VE1B_MODULE 1
#define CONFIG_NET_VENDOR_MYRI 1
#define CONFIG_NET_VENDOR_META 1
#define CONFIG_IKCONFIG_PROC 1
#define CONFIG_NET_TEAM_MODE_LOADBALANCE_MODULE 1
#define CONFIG_USB_SERIAL_ARK3116_MODULE 1
#define CONFIG_SECURITY_PATH 1
#define CONFIG_HID_SUNPLUS_MODULE 1
#define CONFIG_NET_IFE_SKBMARK_MODULE 1
#define CONFIG_UCLAMP_BUCKETS_COUNT 5
#define CONFIG_VIDEO_ADV7183_MODULE 1
#define CONFIG_NF_NAT_SNMP_BASIC_MODULE 1
#define CONFIG_VMCORE_INFO 1
#define CONFIG_CAN_MCBA_USB_MODULE 1
#define CONFIG_SND_KORG1212_MODULE 1
#define CONFIG_GPIB_TMS9914_MODULE 1
#define CONFIG_DLM_DEBUG 1
#define CONFIG_USB_NET_CH9200_MODULE 1
#define CONFIG_USB_SI470X_MODULE 1
#define CONFIG_CHARGER_MAX8971_MODULE 1
#define CONFIG_NFT_FWD_NETDEV_MODULE 1
#define CONFIG_FB_CORE 1
#define CONFIG_CRYPTO_JITTERENTROPY_MEMSIZE_2 1
#define CONFIG_VIDEO_S5KJN1_MODULE 1
#define CONFIG_SENSORS_LM93_MODULE 1
#define CONFIG_IRQ_DOMAIN 1
#define CONFIG_HID_ELO_MODULE 1
#define CONFIG_HAVE_KERNEL_GZIP 1
#define CONFIG_PROBE_EVENTS 1
#define CONFIG_BPF_JIT_ALWAYS_ON 1
#define CONFIG_LDISC_AUTOLOAD 1
#define CONFIG_DYNAMIC_DEBUG_CORE 1
#define CONFIG_XDP_SOCKETS_DIAG_MODULE 1
#define CONFIG_PCI_PRI 1
#define CONFIG_SIS190_MODULE 1
#define CONFIG_DEVICE_PRIVATE 1
#define CONFIG_CRYPTO_AKCIPHER2 1
#define CONFIG_WATCHDOG_HANDLE_BOOT_ENABLED 1
#define CONFIG_NF_NAT_REDIRECT 1
#define CONFIG_SECURITY_SELINUX_SIDTAB_HASH_BITS 9
#define CONFIG_SWIOTLB 1
#define CONFIG_SCHED_AUTOGROUP 1
#define CONFIG_IGC_MODULE 1
#define CONFIG_IP_VS_SH_MODULE 1
#define CONFIG_USB_XHCI_DBGCAP 1
#define CONFIG_NETFILTER_XT_MATCH_CPU_MODULE 1
#define CONFIG_HAVE_ARCH_HUGE_VMALLOC 1
#define CONFIG_DRM_PANEL_VISIONOX_RM692E5_MODULE 1
#define CONFIG_KEYBOARD_BCM_MODULE 1
#define CONFIG_NFT_CONNLIMIT_MODULE 1
#define CONFIG_NLS_MAC_CYRILLIC_MODULE 1
#define CONFIG_BT_BCM_MODULE 1
#define CONFIG_HARDENED_USERCOPY 1
#define CONFIG_TOUCHSCREEN_EETI_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_MARK_MODULE 1
#define CONFIG_HAS_IOMEM 1
#define CONFIG_VIDEO_ADV7343_MODULE 1
#define CONFIG_PCIEASPM 1
#define CONFIG_CMA_AREAS 7
#define CONFIG_MVMDIO_MODULE 1
#define CONFIG_SECURITY_APPARMOR_PARANOID_LOAD 1
#define CONFIG_NETFILTER_XTABLES 1
#define CONFIG_MT7915E_MODULE 1
#define CONFIG_MEDIA_TUNER_MXL5007T_MODULE 1
#define CONFIG_BT_MRVL_SDIO_MODULE 1
#define CONFIG_IR_GPIO_TX_MODULE 1
#define CONFIG_RESET_TI_TPS380X_MODULE 1
#define CONFIG_B43LEGACY_MODULE 1
#define CONFIG_SCSI_FDOMAIN_PCI_MODULE 1
#define CONFIG_W1_SLAVE_DS2405_MODULE 1
#define CONFIG_TRACER_MAX_TRACE 1
#define CONFIG_KEY_NOTIFICATIONS 1
#define CONFIG_WWAN_HWSIM_MODULE 1
#define CONFIG_IPV6_MULTIPLE_TABLES 1
#define CONFIG_BRIDGE_EBT_T_FILTER_MODULE 1
#define CONFIG_USB_ADUTUX_MODULE 1
#define CONFIG_SENSORS_BPA_RS600_MODULE 1
#define CONFIG_CRYPTO_LIB_AESCFB 1
#define CONFIG_BT_HCIBTSDIO_MODULE 1
#define CONFIG_DVB_USB_A800_MODULE 1
#define CONFIG_IR_STREAMZAP_MODULE 1
#define CONFIG_BNXT_DCB 1
#define CONFIG_NF_CONNTRACK_PROCFS 1
#define CONFIG_IMA_APPRAISE 1
#define CONFIG_NF_CONNTRACK_H323_MODULE 1
#define CONFIG_MEDIA_TUNER_FC0011_MODULE 1
#define CONFIG_HID_UNIVERSAL_PIDFF_MODULE 1
#define CONFIG_VHOST_NET_MODULE 1
#define CONFIG_DRM_TOSHIBA_TC358762_MODULE 1
#define CONFIG_IP_VS_PROTO_UDP 1
#define CONFIG_ARCH_HAS_DEVMEM_IS_ALLOWED 1
#define CONFIG_XZ_DEC_POWERPC 1
#define CONFIG_BPF 1
#define CONFIG_FW_CS_DSP_KUNIT_TEST_UTILS_MODULE 1
#define CONFIG_XZ_DEC_ARM64 1
#define CONFIG_LOCALVERSION "-gentoo-dist-bin"
#define CONFIG_HID_GT683R_MODULE 1
#define CONFIG_BT_INTEL_MODULE 1
#define CONFIG_IOSCHED_BFQ 1
#define CONFIG_CRYPTO_DEV_NX_COMPRESS_POWERNV 1
#define CONFIG_LIBETH_MODULE 1
#define CONFIG_MICROCHIP_PHY_MODULE 1
#define CONFIG_HID_EVISION_MODULE 1
#define CONFIG_TYPEC_NVIDIA_ALTMODE_MODULE 1
#define CONFIG_DVB_RTL2830_MODULE 1
#define CONFIG_USB_STORAGE_DATAFAB_MODULE 1
#define CONFIG_SENSORS_MPQ8785_MODULE 1
#define CONFIG_DVB_USB_AF9015_MODULE 1
#define CONFIG_PATA_HPT366_MODULE 1
#define CONFIG_PPC_HAS_LBARX_LHARX 1
#define CONFIG_HID_TWINHAN_MODULE 1
#define CONFIG_REGMAP_MMIO 1
#define CONFIG_KEXEC 1
#define CONFIG_SND_ALOOP_MODULE 1
#define CONFIG_PPC_KUAP 1
#define CONFIG_SND_HRTIMER_MODULE 1
#define CONFIG_NF_NAT_SIP_MODULE 1
#define CONFIG_HVC_IRQ 1
#define CONFIG_B53_MDIO_DRIVER_MODULE 1
#define CONFIG_PRINTF_KUNIT_TEST_MODULE 1
#define CONFIG_RT2X00_LIB_MMIO_MODULE 1
#define CONFIG_CRYPTO_LIB_SHA512_KUNIT_TEST_MODULE 1
#define CONFIG_NLS_ISO8859_9_MODULE 1
#define CONFIG_CHARGER_BD99954_MODULE 1
#define CONFIG_IP_NF_TARGET_ECN_MODULE 1
#define CONFIG_PDS_CORE_MODULE 1
#define CONFIG_CRYPTO_ADIANTUM_MODULE 1
#define CONFIG_DRM_VIRTIO_GPU_KMS 1
#define CONFIG_CRYPTO_BLOWFISH_COMMON_MODULE 1
#define CONFIG_DVB_TTUSB_BUDGET_MODULE 1
#define CONFIG_RV_MON_SNROC 1
#define CONFIG_SLUB 1
#define CONFIG_CRYPTO_LIB_POLY1305_KUNIT_TEST_MODULE 1
#define CONFIG_NET_SCH_PRIO_MODULE 1
#define CONFIG_IFB_MODULE 1
#define CONFIG_STACKPROTECTOR 1
#define CONFIG_EXT4_FS_POSIX_ACL 1
#define CONFIG_NET_VENDOR_AMD 1
#define CONFIG_PPC_FTRACE_OUT_OF_LINE_NUM_RESERVE 32768
#define CONFIG_IR_MCEUSB_MODULE 1
#define CONFIG_USB_SERIAL_CP210X_MODULE 1
#define CONFIG_SLIP_MODULE 1
#define CONFIG_VIDEO_OV5695_MODULE 1
#define CONFIG_LONGEST_SYM_KUNIT_TEST_MODULE 1
#define CONFIG_DVB_USB_DW2102_MODULE 1
#define CONFIG_PRIME_NUMBERS_KUNIT_TEST_MODULE 1
#define CONFIG_IP_NF_IPTABLES_MODULE 1
#define CONFIG_USB_IOWARRIOR_MODULE 1
#define CONFIG_REGULATOR_MAX20411_MODULE 1
#define CONFIG_HID_KENSINGTON_MODULE 1
#define CONFIG_DRM_TI_TPD12S015_MODULE 1
#define CONFIG_USB_GSPCA_STV0680_MODULE 1
#define CONFIG_B43_PCICORE_AUTOSELECT 1
#define CONFIG_HAVE_REGS_AND_STACK_ACCESS_API 1
#define CONFIG_USB_GSPCA_SN9C20X_MODULE 1
#define CONFIG_DVB_ZL10353_MODULE 1
#define CONFIG_OCTEONEP_VDPA_MODULE 1
#define CONFIG_TABLET_USB_ACECAD_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_DSCP_MODULE 1
#define CONFIG_SCD30_CORE_MODULE 1
#define CONFIG_HAVE_SAMPLE_FTRACE_DIRECT 1
#define CONFIG_SND_USB_PODHD_MODULE 1
#define CONFIG_HAVE_GIGANTIC_FOLIOS 1
#define CONFIG_DVB_TDA8083_MODULE 1
#define CONFIG_B53_SERDES_MODULE 1
#define CONFIG_INET_PSP 1
#define CONFIG_FIRMWARE_MEMMAP 1
#define CONFIG_MMU_LAZY_TLB_SHOOTDOWN 1
#define CONFIG_ARCH_DMA_ADDR_T_64BIT 1
#define CONFIG_DVB_STV090x_MODULE 1
#define CONFIG_AIC7XXX_RESET_DELAY_MS 15000
#define CONFIG_VIRTIO_VSOCKETS_COMMON_MODULE 1
#define CONFIG_NFT_COMPAT_ARP_MODULE 1
#define CONFIG_HID_A4TECH_MODULE 1
#define CONFIG_VIDEO_IMX319_MODULE 1
#define CONFIG_VIDEO_ADV7180_MODULE 1
#define CONFIG_SENSORS_ADT7X10_MODULE 1
#define CONFIG_LEDS_CLASS_FLASH_MODULE 1
#define CONFIG_VIDEO_CX88_BLACKBIRD_MODULE 1
#define CONFIG_MEDIA_TUNER_TEA5767_MODULE 1
#define CONFIG_WATCH_QUEUE 1
#define CONFIG_NET_TULIP 1
#define CONFIG_LEDS_KUNIT_TEST_MODULE 1
#define CONFIG_CRYPTO_LIB_ARC4_MODULE 1
#define CONFIG_MFD_CS40L50_CORE_MODULE 1
#define CONFIG_TEST_KSTRTOX 1
#define CONFIG_SENSORS_LM92_MODULE 1
#define CONFIG_TEXTSEARCH_BM_MODULE 1
#define CONFIG_RTC_DRV_DS1553_MODULE 1
#define CONFIG_SND_PROC_FS 1
#define CONFIG_USB_GSPCA_SUNPLUS_MODULE 1
#define CONFIG_HID_SENSOR_INCLINOMETER_3D_MODULE 1
#define CONFIG_BATMAN_ADV_DAT 1
#define CONFIG_UTS_NS 1
#define CONFIG_MLX5_FPGA 1
#define CONFIG_HAVE_IRQ_TIME_ACCOUNTING 1
#define CONFIG_CRYPTO_CTR 1
#define CONFIG_DVB_M88DS3103_MODULE 1
#define CONFIG_SERIAL_8250_PCILIB 1
#define CONFIG_SHUFFLE_PAGE_ALLOCATOR 1
#define CONFIG_DVB_AF9013_MODULE 1
#define CONFIG_CAN_VCAN_MODULE 1
#define CONFIG_HID_ALPS_MODULE 1
#define CONFIG_IGB_MODULE 1
#define CONFIG_COMMON_CLK_SI521XX 1
#define CONFIG_NETFILTER_XT_TARGET_SECMARK_MODULE 1
#define CONFIG_MWIFIEX_MODULE 1
#define CONFIG_RTC_DRV_NCT3018Y_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_ITM 1
#define CONFIG_VHOST_MENU 1
#define CONFIG_LOG_CPU_MAX_BUF_SHIFT 12
#define CONFIG_TYPEC_STUSB160X_MODULE 1
#define CONFIG_LEDS_USER_MODULE 1
#define CONFIG_SENSORS_MP2925_MODULE 1
#define CONFIG_BLK_DEV_FD_MODULE 1
#define CONFIG_USB_EHCI_TT_NEWSCHED 1
#define CONFIG_SENSORS_NCT6775_CORE_MODULE 1
#define CONFIG_PAGE_OFFSET 0xc000000000000000
#define CONFIG_RMI4_F03 1
#define CONFIG_CRYPTO_CBC 1
#define CONFIG_SND_PCM_MODULE 1
#define CONFIG_SND_SEQ_MIDI_EVENT_MODULE 1
#define CONFIG_6LOWPAN_NHC_MOBILITY_MODULE 1
#define CONFIG_IBM_BSR_MODULE 1
#define CONFIG_NFS_ACL_SUPPORT_MODULE 1
#define CONFIG_CGROUP_WRITEBACK 1
#define CONFIG_NET_DEVLINK 1
#define CONFIG_FUTEX_MPOL 1
#define CONFIG_IIO_ST_LSM6DSX_I2C_MODULE 1
#define CONFIG_OF_RESOLVE 1
#define CONFIG_USB_CATC_MODULE 1
#define CONFIG_RMI4_F12 1
#define CONFIG_MARVELL_88X2222_PHY_MODULE 1
#define CONFIG_MAC80211_RC_DEFAULT_MINSTREL 1
#define CONFIG_NF_CONNTRACK_SNMP_MODULE 1
#define CONFIG_CRASH_DM_CRYPT 1
#define CONFIG_NLS_KOI8_R_MODULE 1
#define CONFIG_RTW88_8821A_MODULE 1
#define CONFIG_TEXTSEARCH_KMP_MODULE 1
#define CONFIG_DVB_EC100_MODULE 1
#define CONFIG_EXPERT 1
#define CONFIG_IIO_ST_SENSORS_I2C_MODULE 1
#define CONFIG_DRM_PANEL_SAMSUNG_LTL106HL02_MODULE 1
#define CONFIG_HIST_TRIGGERS 1
#define CONFIG_DM_MULTIPATH_ST_MODULE 1
#define CONFIG_REGMAP_SCCB_MODULE 1
#define CONFIG_NETFILTER_INGRESS 1
#define CONFIG_W1_SLAVE_DS2433_MODULE 1
#define CONFIG_MFD_CS42L43_I2C_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_CONNMARK_MODULE 1
#define CONFIG_RV_MON_NRP 1
#define CONFIG_RTW89_8852CU_MODULE 1
#define CONFIG_RT2800PCI_RT33XX 1
#define CONFIG_IIO_BUFFER_HW_CONSUMER_MODULE 1
#define CONFIG_DVB_DS3000_MODULE 1
#define CONFIG_VIDEO_RJ54N1_MODULE 1
#define CONFIG_IIO_FORMAT_KUNIT_TEST_MODULE 1
#define CONFIG_DRM_CHRONTEL_CH7033_MODULE 1
#define CONFIG_AMD8111_ETH_MODULE 1
#define CONFIG_NF_FLOW_TABLE_INET_MODULE 1
#define CONFIG_ACCESSIBILITY 1
#define CONFIG_KRETPROBE_ON_RETHOOK 1
#define CONFIG_NS83820_MODULE 1
#define CONFIG_USB_NET_HUAWEI_CDC_NCM_MODULE 1
#define CONFIG_DM_MULTIPATH_IOA_MODULE 1
#define CONFIG_USB_STORAGE_JUMPSHOT_MODULE 1
#define CONFIG_BCMA_DRIVER_PCI 1
#define CONFIG_BT_LEDS 1
#define CONFIG_VIDEO_MAX9286_MODULE 1
#define CONFIG_SENSORS_MP5926_MODULE 1
#define CONFIG_NLS_ISO8859_8_MODULE 1
#define CONFIG_TOUCHSCREEN_ATMEL_MXT_MODULE 1
#define CONFIG_LEDS_ST1202_MODULE 1
#define CONFIG_XZ_DEC_MICROLZMA 1
#define CONFIG_GPIO_ADP5585_MODULE 1
#define CONFIG_TIPC_MEDIA_UDP 1
#define CONFIG_SWAP 1
#define CONFIG_CB710_DEBUG_ASSUMPTIONS 1
#define CONFIG_MTD_UBI_NVMEM_MODULE 1
#define CONFIG_ASYNC_RAID6_TEST_MODULE 1
#define CONFIG_EPIC100_MODULE 1
#define CONFIG_MAGIC_SYSRQ_SERIAL 1
#define CONFIG_HAVE_STACKPROTECTOR 1
#define CONFIG_BNX2_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_LOG_MODULE 1
#define CONFIG_MT7615E_MODULE 1
#define CONFIG_RTC_DRV_ISL12026_MODULE 1
#define CONFIG_FXOS8700_I2C_MODULE 1
#define CONFIG_HMC425_MODULE 1
#define CONFIG_SENSORS_THMC50_MODULE 1
#define CONFIG_SENSORS_MP9945_MODULE 1
#define CONFIG_SND_SEQ_MIDI_EMUL_MODULE 1
#define CONFIG_HAVE_ARCH_JUMP_LABEL 1
#define CONFIG_CRYPTO_PCBC_MODULE 1
#define CONFIG_NETDEVSIM_MODULE 1
#define CONFIG_STEAM_FF 1
#define CONFIG_WIZNET_BUS_ANY 1
#define CONFIG_BCMA_MODULE 1
#define CONFIG_HID_HOLTEK_MODULE 1
#define CONFIG_NETCONSOLE_DYNAMIC 1
#define CONFIG_NF_CONNTRACK_BROADCAST_MODULE 1
#define CONFIG_PPC_I8259 1
#define CONFIG_GENWQE_MODULE 1
#define CONFIG_ARCH_HAS_NON_OVERLAPPING_ADDRESS_SPACE 1
#define CONFIG_QCA807X_PHY_MODULE 1
#define CONFIG_RT2800USB_RT33XX 1
#define CONFIG_MMC_SDHCI_PLTFM_MODULE 1
#define CONFIG_SENSORS_SPD5118_DETECT 1
#define CONFIG_MT76x0_COMMON_MODULE 1
#define CONFIG_CXL_MEM_MODULE 1
#define CONFIG_SND_OPL3_LIB_MODULE 1
#define CONFIG_XMON 1
#define CONFIG_NFT_SYNPROXY_MODULE 1
#define CONFIG_HID_ZYDACRON_MODULE 1
#define CONFIG_PREEMPT_RCU 1
#define CONFIG_USB_NET_ZAURUS_MODULE 1
#define CONFIG_VIDEO_SAA6588_MODULE 1
#define CONFIG_HAVE_SYSCALL_TRACEPOINTS 1
#define CONFIG_IOMMU_IOVA_MODULE 1
#define CONFIG_INV_ICM45600_MODULE 1
#define CONFIG_XOR_BLOCKS 1
#define CONFIG_DNOTIFY 1
#define CONFIG_LEDS_TRIGGER_BACKLIGHT_MODULE 1
#define CONFIG_BPF_STREAM_PARSER 1
#define CONFIG_MEDIA_SUBDRV_AUTOSELECT 1
#define CONFIG_RDS_MODULE 1
#define CONFIG_XILINX_VCU_MODULE 1
#define CONFIG_MACVLAN_MODULE 1
#define CONFIG_SENSORS_SHT15_MODULE 1
#define CONFIG_ARCH_SUPPORTS_KEXEC 1
#define CONFIG_IP_NF_RAW_MODULE 1
#define CONFIG_HAVE_HW_BREAKPOINT 1
#define CONFIG_BRIDGE_NF_EBTABLES_LEGACY_MODULE 1
#define CONFIG_PHY_COMMON_PROPS 1
#define CONFIG_DRM_PANEL_RAYDIUM_RM69380_MODULE 1
#define CONFIG_MOUSE_VSXXXAA_MODULE 1
#define CONFIG_DRM_PANEL_SAMSUNG_S6E88A0_AMS427AP24_MODULE 1
#define CONFIG_RANDOM_KMALLOC_CACHES 1
#define CONFIG_VIDEO_TEA6420_MODULE 1
#define CONFIG_SCSI_MPT3SAS_MAX_SGE 128
#define CONFIG_DRM_PANEL_BACKLIGHT_QUIRKS_MODULE 1
#define CONFIG_CRYPTO_CTS 1
#define CONFIG_DVB_USB_EC168_MODULE 1
#define CONFIG_RSI_91X_MODULE 1
#define CONFIG_USB_GSPCA_SPCA1528_MODULE 1
#define CONFIG_TOUCHSCREEN_ILI210X_MODULE 1
#define CONFIG_AD7625_MODULE 1
#define CONFIG_GPIO_AGGREGATOR_MODULE 1
#define CONFIG_HAVE_NMI 1
#define CONFIG_CC_HAS_INT128 1
#define CONFIG_VIDEO_THS8200_MODULE 1
#define CONFIG_SENSORS_F75375S_MODULE 1
#define CONFIG_SND_INDIGOIOX_MODULE 1
#define CONFIG_SCSI_NETLINK 1
#define CONFIG_CRYPTO_USER_API_AEAD 1
#define CONFIG_NETCONSOLE_MODULE 1
#define CONFIG_GPIB_FLUKE_MODULE 1
#define CONFIG_HAVE_VIRT_CPU_ACCOUNTING_GEN 1
#define CONFIG_VIRTIO_VFIO_PCI_MODULE 1
#define CONFIG_SND_HDSPM_MODULE 1
#define CONFIG_SEQ_BUF_KUNIT_TEST_MODULE 1
#define CONFIG_GENERIC_CLOCKEVENTS_BROADCAST 1
#define CONFIG_DVB_TTUSB_DEC_MODULE 1
#define CONFIG_RTC_NVMEM 1
#define CONFIG_DRM_PANEL_LINCOLNTECH_LCD197_MODULE 1
#define CONFIG_ADI_I3C_MASTER_MODULE 1
#define CONFIG_TOUCHSCREEN_IMAGIS_MODULE 1
#define CONFIG_SCSI_HPTIOP_MODULE 1
#define CONFIG_JFS_POSIX_ACL 1
#define CONFIG_VIDEO_VPX3220_MODULE 1
#define CONFIG_WATCHDOG_SYSFS 1
#define CONFIG_OPT4001_MODULE 1
#define CONFIG_REGULATOR_BD96801_MODULE 1
#define CONFIG_SENSORS_POWERZ_MODULE 1
#define CONFIG_MEDIA_ALTERA_CI_MODULE 1
#define CONFIG_SENSORS_TPS546D24_MODULE 1
#define CONFIG_NCSI_OEM_CMD_KEEP_PHY 1
#define CONFIG_USB_SERIAL_QT2_MODULE 1
#define CONFIG_DRM_PANEL_ILITEK_ILI9882T_MODULE 1
#define CONFIG_HID_LED_MODULE 1
#define CONFIG_FB_SYS_FILLRECT 1
#define CONFIG_I2C_DIOLAN_U2C_MODULE 1
#define CONFIG_IBMVMC_MODULE 1
#define CONFIG_XARRAY_MULTI 1
#define CONFIG_USB_SERIAL_IPW_MODULE 1
#define CONFIG_DRM_PANEL_SYNAPTICS_TDDI_MODULE 1
#define CONFIG_PPC_DT_CPU_FTRS 1
#define CONFIG_RMI4_F11 1
#define CONFIG_ARCH_SUPPORTS_KEXEC_FILE 1
#define CONFIG_DCB 1
#define CONFIG_ZSWAP 1
#define CONFIG_PPC64_PROC_SYSTEMCFG 1
#define CONFIG_VIDEO_IMX111_MODULE 1
#define CONFIG_SENSORS_TMP103_MODULE 1
#define CONFIG_MT7921_COMMON_MODULE 1
#define CONFIG_ZSWAP_COMPRESSOR_DEFAULT_LZO 1
#define CONFIG_MEDIA_TUNER_FC0013_MODULE 1
#define CONFIG_SENSORS_ADM1025_MODULE 1
#define CONFIG_CRYPTO_TWOFISH_MODULE 1
#define CONFIG_PACKING 1
#define CONFIG_FUSION_SPI_MODULE 1
#define CONFIG_IPV6_MIP6 1
#define CONFIG_SND_FM801_TEA575X_BOOL 1
#define CONFIG_NET_DSA_QCA8K_LEDS_SUPPORT 1
#define CONFIG_W1_SLAVE_DS2430_MODULE 1
#define CONFIG_BT_HCIUART_QCA 1
#define CONFIG_MHI_NET_MODULE 1
#define CONFIG_CRYPTO_RNG_DEFAULT 1
#define CONFIG_ARCH_MEMORY_PROBE 1
#define CONFIG_RV 1
#define CONFIG_STREAM_PARSER 1
#define CONFIG_WINBOND_840_MODULE 1
#define CONFIG_HID_RAPOO_MODULE 1
#define CONFIG_CRASH_DUMP_KUNIT_TEST_MODULE 1
#define CONFIG_SENSORS_CORSAIR_CPRO_MODULE 1
#define CONFIG_IP_VS_SED_MODULE 1
#define CONFIG_LIBFCOE_MODULE 1
#define CONFIG_XZ_DEC_ARM 1
#define CONFIG_DVB_RTL2832_MODULE 1
#define CONFIG_INET_ESP_OFFLOAD_MODULE 1
#define CONFIG_NET_DSA_MT7530_MODULE 1
#define CONFIG_NET_DSA_MODULE 1
#define CONFIG_SENSORS_INA2XX_MODULE 1
#define CONFIG_MTD_UBI_BEB_LIMIT 20
#define CONFIG_HOTPLUG_PCI 1
#define CONFIG_VIDEO_S5K5BAF_MODULE 1
#define CONFIG_USB_VIDEO_CLASS_MODULE 1
#define CONFIG_F2FS_FS_LZO 1
#define CONFIG_SCSI_PM8001_MODULE 1
#define CONFIG_SENSORS_AD7418_MODULE 1
#define CONFIG_NFC_NCI_MODULE 1
#define CONFIG_IEEE802154_MODULE 1
#define CONFIG_RTC_DRV_DS1374_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_IDLETIMER_MODULE 1
#define CONFIG_CIFS_POSIX 1
#define CONFIG_HID_COUGAR_MODULE 1
#define CONFIG_MMC_HSQ_MODULE 1
#define CONFIG_PWM 1
#define CONFIG_VSX 1
#define CONFIG_CONFIGFS_FS 1
#define CONFIG_NVIDIA_SHIELD_FF 1
#define CONFIG_JFFS2_FS_WRITEBUFFER 1
#define CONFIG_DEFAULT_INIT ""
#define CONFIG_RT2500PCI_MODULE 1
#define CONFIG_XILLYUSB_MODULE 1
#define CONFIG_KXCJK1013_MODULE 1
#define CONFIG_ARCH_MIGHT_HAVE_PC_SERIO 1
#define CONFIG_VIRT_DRIVERS 1
#define CONFIG_COMMON_CLK_VC3_MODULE 1
#define CONFIG_CRYPTO_DEV_NITROX_MODULE 1
#define CONFIG_RTW88_8723DU_MODULE 1
#define CONFIG_NET_CLS 1
#define CONFIG_XFS_LIVE_HOOKS 1
#define CONFIG_NET_VENDOR_AGERE 1
#define CONFIG_W1_CON 1
#define CONFIG_HID_ACRUX_FF 1
#define CONFIG_OF_RESERVED_MEM 1
#define CONFIG_NFT_REJECT_IPV4_MODULE 1
#define CONFIG_IP6_NF_MATCH_OPTS_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_EGALAX 1
#define CONFIG_ARCH_HAS_LAZY_MMU_MODE 1
#define CONFIG_SATA_MV_MODULE 1
#define CONFIG_SCHEDSTATS 1
#define CONFIG_ALTERA_STAPL_MODULE 1
#define CONFIG_DEBUGGER 1
#define CONFIG_INFINIBAND_IPOIB_DEBUG 1
#define CONFIG_VIA_VELOCITY_MODULE 1
#define CONFIG_REALTEK_PHY_MODULE 1
#define CONFIG_CRYPTO_KPP2 1
#define CONFIG_TCP_AO 1
#define CONFIG_NET_ACT_SKBEDIT_MODULE 1
#define CONFIG_DRM_DISPLAY_DP_AUX_CEC 1
#define CONFIG_MCP9600_MODULE 1
#define CONFIG_VHOST_VSOCK_MODULE 1
#define CONFIG_BLK_DEV_LOOP_MIN_COUNT 0
#define CONFIG_DRM_DISPLAY_CONNECTOR_MODULE 1
#define CONFIG_B43LEGACY_HWRNG 1
#define CONFIG_NFT_QUEUE_MODULE 1
#define CONFIG_CRYPTO_DRBG_HASH 1
#define CONFIG_SUNDANCE_MODULE 1
#define CONFIG_DCACHE_WORD_ACCESS 1
#define CONFIG_RANDOMIZE_KSTACK_OFFSET_DEFAULT 1
#define CONFIG_DRM_TTM_HELPER_MODULE 1
#define CONFIG_CDROM 1
#define CONFIG_RCU_TORTURE_TEST_MODULE 1
#define CONFIG_INTERRUPT_SANITIZE_REGISTERS 1
#define CONFIG_DRM_VKMS_MODULE 1
#define CONFIG_MEDIA_TUNER_TEA5761_MODULE 1
#define CONFIG_CONTEXT_SWITCH_TRACER 1
#define CONFIG_CRYPTO_BENCHMARK_MODULE 1
#define CONFIG_USB_GSPCA_VC032X_MODULE 1
#define CONFIG_USB_OHCI_HCD_PPC_OF_BE 1
#define CONFIG_RUSTC_LLVM_VERSION 0
#define CONFIG_EXTCON 1
#define CONFIG_CRYPTO_DEV_QAT_DH895xCCVF_MODULE 1
#define CONFIG_RESET_GPIO_MODULE 1
#define CONFIG_BAYCOM_SER_FDX_MODULE 1
#define CONFIG_ADXL367_MODULE 1
#define CONFIG_IIO_INTERRUPT_TRIGGER_MODULE 1
#define CONFIG_MT7663_USB_SDIO_COMMON_MODULE 1
#define CONFIG_ARCH_HAS_PTDUMP 1
#define CONFIG_USB_GSPCA_STK014_MODULE 1
#define CONFIG_INPUT_KEYBOARD 1
#define CONFIG_CRYPTO_DEV_QAT_MODULE 1
#define CONFIG_GENERIC_IDLE_POLL_SETUP 1
#define CONFIG_TRUSTED_KEYS 1
#define CONFIG_ATH9K_COMMON_MODULE 1
#define CONFIG_TCG_TIS_I2C_NUVOTON_MODULE 1
#define CONFIG_MQ_IOSCHED_KYBER 1
#define CONFIG_SND_SEQ_DUMMY_MODULE 1
#define CONFIG_XZ_DEC_RISCV 1
#define CONFIG_DEVICE_MIGRATION 1
#define CONFIG_SND_MPU401_UART_MODULE 1
#define CONFIG_DRAGONRISE_FF 1
#define CONFIG_REGULATOR_VCTRL_MODULE 1
#define CONFIG_THREAD_INFO_IN_TASK 1
#define CONFIG_SYSFS 1
#define CONFIG_XFS_RT 1
#define CONFIG_POWER8_CPU 1
#define CONFIG_RD_ZSTD 1
#define CONFIG_IRQ_TIME_ACCOUNTING 1
#define CONFIG_USB_MR800_MODULE 1
#define CONFIG_SECURITY_SELINUX_DEVELOP 1
#define CONFIG_MT7925E_MODULE 1
#define CONFIG_ADXL380_MODULE 1
#define CONFIG_DRM_SAMSUNG_DSIM_MODULE 1
#define CONFIG_BRCMFMAC_PROTO_BCDC 1
#define CONFIG_MFD_SM501_GPIO 1
#define CONFIG_CRYPTO_LIB_SHA512 1
#define CONFIG_INFINIBAND_SRPT_MODULE 1
#define CONFIG_NET_CLS_FLOW_MODULE 1
#define CONFIG_VIDEO_SAA7146_MODULE 1
#define CONFIG_TOUCHSCREEN_ST1232_MODULE 1
#define CONFIG_DM_THIN_PROVISIONING_MODULE 1
#define CONFIG_USB_OHCI_BIG_ENDIAN_MMIO 1
#define CONFIG_TCP_CONG_HYBLA_MODULE 1
#define CONFIG_SND_UMP_MODULE 1
#define CONFIG_MEDIA_TUNER_XC5000_MODULE 1
#define CONFIG_HAVE_RSEQ 1
#define CONFIG_HID_AUREAL_MODULE 1
#define CONFIG_LEDS_PCA9532_GPIO 1
#define CONFIG_DVB_CORE_MODULE 1
#define CONFIG_IP_NF_TARGET_REJECT_MODULE 1
#define CONFIG_REGMAP_IRQ 1
#define CONFIG_NET_EMATCH_TEXT_MODULE 1
#define CONFIG_SCSI_IBMVSCSIS_MODULE 1
#define CONFIG_HAVE_KCSAN_COMPILER 1
#define CONFIG_TCG_VTPM_PROXY_MODULE 1
#define CONFIG_SND_ENS1370_MODULE 1
#define CONFIG_ATA_OVER_ETH_MODULE 1
#define CONFIG_IPVLAN_MODULE 1
#define CONFIG_SENSORS_IBMPOWERNV 1
#define CONFIG_MMC_ALCOR_MODULE 1
#define CONFIG_SERIO_RAW_MODULE 1
#define CONFIG_STRICT_DEVMEM 1
#define CONFIG_GCC10_NO_ARRAY_BOUNDS 1
#define CONFIG_RTC_DRV_DS1672_MODULE 1
#define CONFIG_SENSORS_TMP102_MODULE 1
#define CONFIG_RTW88_8821C_MODULE 1
#define CONFIG_SND_INDIGOIO_MODULE 1
#define CONFIG_MEDIA_TUNER_FC0012_MODULE 1
#define CONFIG_SENSORS_ADM1026_MODULE 1
#define CONFIG_VIDEO_S5K6A3_MODULE 1
#define CONFIG_INV_ICM45600_I3C_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_STATE_MODULE 1
#define CONFIG_HAVE_MEMORYLESS_NODES 1
#define CONFIG_DRM_DRAW 1
#define CONFIG_SENSORS_NCT6775_I2C_MODULE 1
#define CONFIG_PPP_DEFLATE_MODULE 1
#define CONFIG_W1_SLAVE_DS2431_MODULE 1
#define CONFIG_ARCH_DMA_DEFAULT_COHERENT 1
#define CONFIG_CPU_FREQ_GOV_ONDEMAND 1
#define CONFIG_SENSORS_MAX8688_MODULE 1
#define CONFIG_BCM_VK_MODULE 1
#define CONFIG_VT_HW_CONSOLE_BINDING 1
#define CONFIG_I2C_ALGOBIT_MODULE 1
#define CONFIG_INPUT_RT5120_PWRKEY_MODULE 1
#define CONFIG_SENSORS_PM6764TR_MODULE 1
#define CONFIG_EXTCON_USBC_TUSB320_MODULE 1
#define CONFIG_HAVE_RETHOOK 1
#define CONFIG_CRYPTO_CAST_COMMON_MODULE 1
#define CONFIG_NF_NAT_MASQUERADE 1
#define CONFIG_GENERIC_CLOCKEVENTS 1
#define CONFIG_NF_CT_NETLINK_HELPER_MODULE 1
#define CONFIG_MOUSE_SYNAPTICS_I2C_MODULE 1
#define CONFIG_NLS_CODEPAGE_852_MODULE 1
#define CONFIG_DIBS_LO 1
#define CONFIG_ADC_BATTERY_HELPER_MODULE 1
#define CONFIG_USB_SERIAL_WHITEHEAT_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_ECN_MODULE 1
#define CONFIG_BMC150_MAGN_I2C_MODULE 1
#define CONFIG_BT_HCIUART_MODULE 1
#define CONFIG_IPMI_DEVICE_INTERFACE_MODULE 1
#define CONFIG_DVB_USB_AF9005_MODULE 1
#define CONFIG_VIDEO_USBTV_MODULE 1
#define CONFIG_SND_HDSP_MODULE 1
#define CONFIG_RTC_CLASS 1
#define CONFIG_POWER_SEQUENCING_QCOM_WCN_MODULE 1
#define CONFIG_REGULATOR_RAA215300_MODULE 1
#define CONFIG_SENSORS_ASUS_ROG_RYUJIN_MODULE 1
#define CONFIG_INPUT_PWM_BEEPER_MODULE 1
#define CONFIG_CIFS_SWN_UPCALL 1
#define CONFIG_CRYPTO_LZO 1
#define CONFIG_LEDS_IS31FL32XX_MODULE 1
#define CONFIG_NLS_CODEPAGE_869_MODULE 1
#define CONFIG_FWNODE_MDIO 1
#define CONFIG_PPC_XIVE_SPAPR 1
#define CONFIG_PERF_EVENTS 1
#define CONFIG_DRM_PANIC_BACKGROUND_COLOR 0x000000
#define CONFIG_IP6_NF_MATCH_AH_MODULE 1
#define CONFIG_HAVE_HARDLOCKUP_DETECTOR_PERF 1
#define CONFIG_DRM_PANEL_BOE_TV101WUM_NL6_MODULE 1
#define CONFIG_RV_REACT_PRINTK 1
#define CONFIG_MHI_WWAN_MBIM_MODULE 1
#define CONFIG_SENSORS_ADT7470_MODULE 1
#define CONFIG_NET_ACT_VLAN_MODULE 1
#define CONFIG_TCG_TIS 1
#define CONFIG_REGULATOR_RTQ2134_MODULE 1
#define CONFIG_DRM_PANEL_JADARD_JD9365DA_H3_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_LED_MODULE 1
#define CONFIG_SERIAL_8250_NR_UARTS 32
#define CONFIG_CRC8_MODULE 1
#define CONFIG_RAMDAX_MODULE 1
#define CONFIG_USB_STORAGE_MODULE 1
#define CONFIG_PPC_OF_BOOT_TRAMPOLINE 1
#define CONFIG_DVB_USB_LME2510_MODULE 1
#define CONFIG_BACKLIGHT_AW99706_MODULE 1
#define CONFIG_SENSORS_TPS53679_MODULE 1
#define CONFIG_PATA_IT821X_MODULE 1
#define CONFIG_IOMMU_API 1
#define CONFIG_ZISOFS 1
#define CONFIG_CRYPTO_RSA 1
#define CONFIG_HID_LOGITECH_HIDPP_MODULE 1
#define CONFIG_SERIAL_8250_MANY_PORTS 1
#define CONFIG_IR_JVC_DECODER_MODULE 1
#define CONFIG_SCSI_AIC79XX_MODULE 1
#define CONFIG_NVME_TCP_MODULE 1
#define CONFIG_NET_IPGRE_BROADCAST 1
#define CONFIG_SENSORS_SHTC1_MODULE 1
#define CONFIG_LOCKD_V4 1
#define CONFIG_DM_DUST_MODULE 1
#define CONFIG_CIFS_DFS_UPCALL 1
#define CONFIG_TYPEC_TCPCI_MODULE 1
#define CONFIG_NMI_IPI 1
#define CONFIG_DRM_PANEL_SONY_TULIP_TRULY_NT35521_MODULE 1
#define CONFIG_LEDS_OSRAM_AMS_AS3668_MODULE 1
#define CONFIG_AQUANTIA_PHY_MODULE 1
#define CONFIG_TYPEC_TBT_ALTMODE_MODULE 1
#define CONFIG_WATCHDOG_CORE 1
#define CONFIG_IIO_ST_ACCEL_3AXIS_MODULE 1
#define CONFIG_UCSI_CCG_MODULE 1
#define CONFIG_SND_HDA_CODEC_HDMI_MODULE 1
#define CONFIG_SCSI_DH_EMC_MODULE 1
#define CONFIG_RTC_LIB_KUNIT_TEST_MODULE 1
#define CONFIG_REGULATOR_RT6160_MODULE 1
#define CONFIG_CHARGER_UCS1002_MODULE 1
#define CONFIG_BQL 1
#define CONFIG_IO_URING 1
#define CONFIG_HID_SENSOR_ACCEL_3D_MODULE 1
#define CONFIG_DAMON_PADDR 1
#define CONFIG_ASYMMETRIC_KEY_TYPE 1
#define CONFIG_RTC_DRV_PCF8523_MODULE 1
#define CONFIG_SENSORS_MAX20751_MODULE 1
#define CONFIG_INPUT_MOUSE 1
#define CONFIG_PROC_VMCORE 1
#define CONFIG_SENSORS_MP2891_MODULE 1
#define CONFIG_DEV_DAX_PMEM_MODULE 1
#define CONFIG_RTC_DRV_X1205_MODULE 1
#define CONFIG_MMC_USHC_MODULE 1
#define CONFIG_SATA_SX4_MODULE 1
#define CONFIG_DRM_PANEL_BOE_TV101WUM_LL2_MODULE 1
#define CONFIG_DECOMPRESS_XZ 1
#define CONFIG_MICROCHIP_T1S_PHY_MODULE 1
#define CONFIG_NVME_HOST_AUTH 1
#define CONFIG_SND_HDA_CODEC_SIGMATEL_MODULE 1
#define CONFIG_MLX5_ESWITCH 1
#define CONFIG_VIDEO_SAA7134_MODULE 1
#define CONFIG_USB_LEGOTOWER_MODULE 1
#define CONFIG_RTC_DRV_MAX6900_MODULE 1
#define CONFIG_RATIONAL_KUNIT_TEST_MODULE 1
#define CONFIG_CC_HAS_MULTIDIMENSIONAL_NONSTRING 1
#define CONFIG_SND_RME9652_MODULE 1
#define CONFIG_NET_DSA_BCM_SF2_MODULE 1
#define CONFIG_TIPC_CRYPTO 1
#define CONFIG_INFINIBAND_USER_ACCESS_MODULE 1
#define CONFIG_CRYPTO_LIB_POLYVAL_MODULE 1
#define CONFIG_NLS_CODEPAGE_737_MODULE 1
#define CONFIG_SENSORS_SMSC47M192_MODULE 1
#define CONFIG_MODULES_USE_ELF_RELA 1
#define CONFIG_NFS_V4_2 1
#define CONFIG_NVME_KEYRING 1
#define CONFIG_MEMCG_NMI_SAFETY_REQUIRES_ATOMIC 1
#define CONFIG_CPU_FREQ_DEFAULT_GOV_SCHEDUTIL 1
#define CONFIG_RTC_DRV_RV3029C2_MODULE 1
#define CONFIG_RT2400PCI_MODULE 1
#define CONFIG_KUNIT_DEFAULT_FILTER_GLOB ""
#define CONFIG_CLOSURES 1
#define CONFIG_BOOT_CONFIG 1
#define CONFIG_TULIP_MODULE 1
#define CONFIG_MUX_ADG792A_MODULE 1
#define CONFIG_PWM_PROVIDE_GPIO 1
#define CONFIG_HID_KYE_MODULE 1
#define CONFIG_USB_XHCI_HCD 1
#define CONFIG_ETHERNET 1
#define CONFIG_DRM_PANIC 1
#define CONFIG_INFINIBAND_RTRS_CLIENT_MODULE 1
#define CONFIG_I3C_OR_I2C_MODULE 1
#define CONFIG_DRM_TTM_MODULE 1
#define CONFIG_VIDEO_OV772X_MODULE 1
#define CONFIG_TARGET_CPU "power8"
#define CONFIG_NFSD_V3_ACL 1
#define CONFIG_NETFILTER_CONNCOUNT_MODULE 1
#define CONFIG_LAZY_MMU_MODE_KUNIT_TEST_MODULE 1
#define CONFIG_MTD_CFI_I2 1
#define CONFIG_DRM_NOUVEAU_SIL164_MODULE 1
#define CONFIG_SND_ENS1371_MODULE 1
#define CONFIG_CRYPTO_DEV_QAT_4XXX_MODULE 1
#define CONFIG_GTP_MODULE 1
#define CONFIG_TREE_SRCU 1
#define CONFIG_NETFS_STATS 1
#define CONFIG_RADIO_SI470X_MODULE 1
#define CONFIG_IWLWIFI_MODULE 1
#define CONFIG_BT_MTK_MODULE 1
#define CONFIG_NF_CONNTRACK_AMANDA_MODULE 1
#define CONFIG_TI_ADS7924_MODULE 1
#define CONFIG_PM_RUNTIME_KUNIT_TEST_MODULE 1
#define CONFIG_EEPROM_AT24_MODULE 1
#define CONFIG_BT_HIDP_MODULE 1
#define CONFIG_ARCH_HAS_MMIOWB 1
#define CONFIG_BACKLIGHT_MP3309C_MODULE 1
#define CONFIG_ARCH_HAS_STRICT_KERNEL_RWX 1
#define CONFIG_SUNRPC_BACKCHANNEL 1
#define CONFIG_I2C_SLAVE 1
#define CONFIG_RTL8192SE_MODULE 1
#define CONFIG_WL1251_MODULE 1
#define CONFIG_HAVE_LIVEPATCH 1
#define CONFIG_KUNIT_DEFAULT_FILTER_ACTION ""
#define CONFIG_FSI_MASTER_GPIO_MODULE 1
#define CONFIG_REGULATOR_RTQ6752_MODULE 1
#define CONFIG_NF_CONNTRACK_TIMEOUT 1
#define CONFIG_WWAN_MODULE 1
#define CONFIG_NET_ACT_POLICE_MODULE 1
#define CONFIG_HID_UCLOGIC_MODULE 1
#define CONFIG_BLOCK_HOLDER_DEPRECATED 1
#define CONFIG_HID_WALTOP_MODULE 1
#define CONFIG_BALLOON_MIGRATION 1
#define CONFIG_USB_STORAGE_ISD200_MODULE 1
#define CONFIG_SENSORS_AQUACOMPUTER_D5NEXT_MODULE 1
#define CONFIG_SCSI_LPFC_MODULE 1
#define CONFIG_REALTEK_AUTOPM 1
#define CONFIG_VIDEO_IR_I2C_MODULE 1
#define CONFIG_LEDS_TRIGGER_GPIO_MODULE 1
#define CONFIG_LV0104CS_MODULE 1
#define CONFIG_NFT_REJECT_MODULE 1
#define CONFIG_SND_RME32_MODULE 1
#define CONFIG_INET_DIAG_DESTROY 1
#define CONFIG_KVM_BOOK3S_64_MODULE 1
#define CONFIG_8139CP_MODULE 1
#define CONFIG_NET_NS 1
#define CONFIG_PPC_VAS 1
#define CONFIG_NET_TEAM_MODE_BROADCAST_MODULE 1
#define CONFIG_CRC16 1
#define CONFIG_SOCK_CGROUP_DATA 1
#define CONFIG_TABLET_USB_PEGASUS_MODULE 1
#define CONFIG_ISL29003_MODULE 1
#define CONFIG_USB_AUTOSUSPEND_DELAY 2
#define CONFIG_CRYPTO_SEQIV 1
#define CONFIG_DP83640_PHY_MODULE 1
#define CONFIG_ARCH_MMAP_RND_COMPAT_BITS_MAX 13
#define CONFIG_NLS_CODEPAGE_866_MODULE 1
#define CONFIG_PPC_RADIX_MMU_DEFAULT 1
#define CONFIG_SENSORS_FTSTEUTATES_MODULE 1
#define CONFIG_BRCMSMAC_LEDS 1
#define CONFIG_IPMI_HANDLER_MODULE 1
#define CONFIG_MUTEX_SPIN_ON_OWNER 1
#define CONFIG_FSNOTIFY 1
#define CONFIG_NET_FLOW_LIMIT 1
#define CONFIG_NET_DSA_TAG_KSZ_MODULE 1
#define CONFIG_FS_DAX_PMD 1
#define CONFIG_OF_OVERLAY 1
#define CONFIG_NFT_REJECT_IPV6_MODULE 1
#define CONFIG_PPC_MEM_KEYS 1
#define CONFIG_CHELSIO_TLS_DEVICE_MODULE 1
#define CONFIG_GNSS_UBX_SERIAL_MODULE 1
#define CONFIG_CIFS_MODULE 1
#define CONFIG_DW_EDMA_MODULE 1
#define CONFIG_OCFS2_FS_USERSPACE_CLUSTER_MODULE 1
#define CONFIG_DRM_ANALOGIX_ANX78XX_MODULE 1
#define CONFIG_ATM_TCP_MODULE 1
#define CONFIG_ALX_MODULE 1
#define CONFIG_REGULATOR_ROHM_MODULE 1
#define CONFIG_EXT4_KUNIT_TESTS_MODULE 1
#define CONFIG_INPUT_TOUCHSCREEN 1
#define CONFIG_VIDEO_OV5670_MODULE 1
#define CONFIG_BFQ_GROUP_IOSCHED 1
#define CONFIG_CICADA_PHY_MODULE 1
#define CONFIG_VIRTIO 1
#define CONFIG_ALTIVEC 1
#define CONFIG_NET_DROP_MONITOR 1
#define CONFIG_IP_SET_HASH_NETIFACE_MODULE 1
#define CONFIG_SCANF_KUNIT_TEST_MODULE 1
#define CONFIG_I2C_SI470X_MODULE 1
#define CONFIG_SND_INTEL_DSP_CONFIG_MODULE 1
#define CONFIG_VIDEO_UPD64031A_MODULE 1
#define CONFIG_NET_DSA_TAG_BRCM_LEGACY_MODULE 1
#define CONFIG_BOSCH_BNO055_I2C_MODULE 1
#define CONFIG_TOUCHSCREEN_HIMAX_HX852X_MODULE 1
#define CONFIG_CLS_U32_PERF 1
#define CONFIG_MMC_BLOCK_MINORS 8
#define CONFIG_CRYPTO_USER_API_HASH 1
#define CONFIG_QED_FCOE 1
#define CONFIG_PARTITION_ADVANCED 1
#define CONFIG_JOYSTICK_JOYDUMP_MODULE 1
#define CONFIG_NFT_FIB_INET_MODULE 1
#define CONFIG_DVB_MT352_MODULE 1
#define CONFIG_EEPROM_IDT_89HPESX_MODULE 1
#define CONFIG_ALTERA_TSE_MODULE 1
#define CONFIG_SENSORS_LTC4151_MODULE 1
#define CONFIG_ZRAM_DEF_COMP "lzo-rle"
#define CONFIG_ATH5K_PCI 1
#define CONFIG_TCG_ATMEL_MODULE 1
#define CONFIG_NFSD_MODULE 1
#define CONFIG_PM_GENERIC_DOMAINS_SLEEP 1
#define CONFIG_HID_CHICONY_MODULE 1
#define CONFIG_USB_MA901_MODULE 1
#define CONFIG_SENSORS_MAX197_MODULE 1
#define CONFIG_PCI_ECAM 1
#define CONFIG_SYSTEM_TRUSTED_KEYRING 1
#define CONFIG_CRYPTO_ZSTD_MODULE 1
#define CONFIG_ARCH_MMAP_RND_BITS_MAX 29
#define CONFIG_BMP280_I2C_MODULE 1
#define CONFIG_VIDEO_TUNER_MODULE 1
#define CONFIG_SENSORS_HAC300S_MODULE 1
#define CONFIG_NET_EMATCH_IPSET_MODULE 1
#define CONFIG_IP_VS_MH_MODULE 1
#define CONFIG_MEDIA_TUNER_XC2028_MODULE 1
#define CONFIG_BATMAN_ADV_BLA 1
#define CONFIG_REGULATOR_MAX77857_MODULE 1
#define CONFIG_CGROUP_DEVICE 1
#define CONFIG_USB_NET_CDC_SUBSET_MODULE 1
#define CONFIG_SENSORS_NTC_THERMISTOR_MODULE 1
#define CONFIG_PPC_FPU_REGS 1
#define CONFIG_SCSI_FC_ATTRS_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_QUOTA_MODULE 1
#define CONFIG_INPUT_CM109_MODULE 1
#define CONFIG_VIDEO_SAA7134_DVB_MODULE 1
#define CONFIG_VIDEO_TVP7002_MODULE 1
#define CONFIG_HT16K33_MODULE 1
#define CONFIG_DVB_USB_OPERA1_MODULE 1
#define CONFIG_RTC_DRV_PCF8583_MODULE 1
#define CONFIG_MEDIA_TUNER_MODULE 1
#define CONFIG_SYNC_FILE 1
#define CONFIG_BPF_SYSCALL 1
#define CONFIG_DA_MON_EVENTS_IMPLICIT 1
#define CONFIG_VIDEO_GO7007_MODULE 1
#define CONFIG_ARCH_WANTS_PRE_LINK_VMLINUX 1
#define CONFIG_SND_PCI 1
#define CONFIG_USB_NET_INT51X1_MODULE 1
#define CONFIG_HID_PENMOUNT_MODULE 1
#define CONFIG_IP_SET_LIST_SET_MODULE 1
#define CONFIG_LIBFC_MODULE 1
#define CONFIG_ICPLUS_PHY_MODULE 1
#define CONFIG_CMDLINE_LOG_WRAP_IDEAL_LEN 1021
#define CONFIG_VIDEO_OV08X40_MODULE 1
#define CONFIG_BRIDGE_EBT_ARPREPLY_MODULE 1
#define CONFIG_NFS_V4_2_SSC_HELPER 1
#define CONFIG_IIO_KFIFO_BUF_MODULE 1
#define CONFIG_JOYSTICK_INTERACT_MODULE 1
#define CONFIG_MTD_CFI_I1 1
#define CONFIG_LEDS_LP3944_MODULE 1
#define CONFIG_LEDS_AN30259A_MODULE 1
#define CONFIG_VIRTIO_PCI 1
#define CONFIG_RD_LZMA 1
#define CONFIG_DA311_MODULE 1
#define CONFIG_MFD_INTEL_M10_BMC_PMCI_MODULE 1
#define CONFIG_W1_SLAVE_DS2408_MODULE 1
#define CONFIG_NFP_APP_FLOWER 1
#define CONFIG_VIRTIO_PCI_LEGACY 1
#define CONFIG_NTFS3_LZX_XPRESS 1
#define CONFIG_RTC_DRV_DS1685_FAMILY_MODULE 1
#define CONFIG_SENSORS_BEL_PFE_MODULE 1
#define CONFIG_MEDIA_TUNER_MXL301RF_MODULE 1
#define CONFIG_ARCH_SUPPORTS_UPROBES 1
#define CONFIG_MEMORY_ISOLATION 1
#define CONFIG_IWLEGACY_DEBUGFS 1
#define CONFIG_IP_MROUTE 1
#define CONFIG_JOYSTICK_TMDC_MODULE 1
#define CONFIG_ARCH_HAS_DEBUG_VM_PGTABLE 1
#define CONFIG_VIDEO_CS3308_MODULE 1
#define CONFIG_FB_SYS_IMAGEBLIT 1
#define CONFIG_NET_DSA_TAG_NONE_MODULE 1
#define CONFIG_CRYPTO_FIPS_NAME "Linux Kernel Cryptographic API"
#define CONFIG_MFD_MAX77714_MODULE 1
#define CONFIG_PREEMPT_DYNAMIC 1
#define CONFIG_RTL8XXXU_MODULE 1
#define CONFIG_USB_NET_KALMIA_MODULE 1
#define CONFIG_USB_SERIAL_BELKIN_MODULE 1
#define CONFIG_PPC_THP 1
#define CONFIG_DRM_ANALOGIX_ANX7625_MODULE 1
#define CONFIG_CRYPTO_JITTERENTROPY_OSR 1
#define CONFIG_HAVE_KRETPROBES 1
#define CONFIG_TOUCHSCREEN_USB_GOTOP 1
#define CONFIG_ZSWAP_SHRINKER_DEFAULT_ON 1
#define CONFIG_RTW88_8814A_MODULE 1
#define CONFIG_USB_GSPCA_PAC207_MODULE 1
#define CONFIG_NET_VENDOR_EMULEX 1
#define CONFIG_MARVELL_88Q2XXX_PHY_MODULE 1
#define CONFIG_NLS_CODEPAGE_850_MODULE 1
#define CONFIG_IIO_RESCALE_MODULE 1
#define CONFIG_IPE_BOOT_POLICY ""
#define CONFIG_SKB_DECRYPTED 1
#define CONFIG_NINTENDO_FF 1
#define CONFIG_OPENVSWITCH_GRE_MODULE 1
#define CONFIG_NLS_MAC_INUIT_MODULE 1
#define CONFIG_NF_CONNTRACK_BRIDGE_MODULE 1
#define CONFIG_HID_SENSOR_DEVICE_ROTATION_MODULE 1
#define CONFIG_KEYS 1
#define CONFIG_ADUX1020_MODULE 1
#define CONFIG_VIDEO_TVAUDIO_MODULE 1
#define CONFIG_INPUT_YEALINK_MODULE 1
#define CONFIG_MT7921S_MODULE 1
#define CONFIG_IFCVF_MODULE 1
#define CONFIG_RTC_DRV_ISL1208_MODULE 1
#define CONFIG_DVB_OR51132_MODULE 1
#define CONFIG_IIO_ST_MAGN_I2C_3AXIS_MODULE 1
#define CONFIG_OPAL_PRD_MODULE 1
#define CONFIG_CASSINI_MODULE 1
#define CONFIG_HID_GREENASIA_MODULE 1
#define CONFIG_CW1200_WLAN_SDIO_MODULE 1
#define CONFIG_TOUCHSCREEN_MMS114_MODULE 1
#define CONFIG_SATA_NV_MODULE 1
#define CONFIG_HID_SENSOR_GYRO_3D_MODULE 1
#define CONFIG_IPMI_POWEROFF_MODULE 1
#define CONFIG_TI_ADS7138_MODULE 1
#define CONFIG_RTC_DRV_ISL12022_MODULE 1
#define CONFIG_IMA_MEASURE_PCR_IDX 10
#define CONFIG_NET_SWITCHDEV 1
#define CONFIG_SENSORS_MP9941_MODULE 1
#define CONFIG_SENSORS_LT3074_REGULATOR_MODULE 1
#define CONFIG_MACB_USE_HWSTAMP 1
#define CONFIG_ADXL372_I2C_MODULE 1
#define CONFIG_DRM_ST7571_I2C_MODULE 1
#define CONFIG_SCSI_SYM53C8XX_2_MODULE 1
#define CONFIG_PNFS_BLOCK_MODULE 1
#define CONFIG_V4L2_ASYNC_MODULE 1
#define CONFIG_INT_POW_KUNIT_TEST_MODULE 1
#define CONFIG_QLCNIC_HWMON 1
#define CONFIG_DRM_PANEL_VISIONOX_RM69299_MODULE 1
#define CONFIG_PSAMPLE_MODULE 1
#define CONFIG_RTW88_8822BS_MODULE 1
#define CONFIG_I2C_SLAVE_EEPROM_MODULE 1
#define CONFIG_IO_WQ 1
#define CONFIG_VHOST_MODULE 1
#define CONFIG_RADIO_SI4713_MODULE 1
#define CONFIG_HAVE_ARCH_HUGE_VMAP 1
#define CONFIG_VDSO_GETRANDOM 1
#define CONFIG_RTLBTCOEXIST_MODULE 1
#define CONFIG_NET_DSA_MV88E6XXX_MODULE 1
#define CONFIG_MMU_GATHER_PAGE_SIZE 1
#define CONFIG_VIA_RHINE_MMIO 1
#define CONFIG_AS_VERSION 24600
#define CONFIG_SENSORS_LT7182S_MODULE 1
#define CONFIG_TEST_SORT_MODULE 1
#define CONFIG_RTW89_8852BE_MODULE 1
#define CONFIG_AMT_MODULE 1
#define CONFIG_BLK_CGROUP_PUNT_BIO 1
#define CONFIG_ARCH_USING_PATCHABLE_FUNCTION_ENTRY 1
#define CONFIG_INET_UDP_DIAG 1
#define CONFIG_USB_STORAGE_ALAUDA_MODULE 1
#define CONFIG_NET_SCH_HTB_MODULE 1
#define CONFIG_IP_VS_FO_MODULE 1
#define CONFIG_CHELSIO_IPSEC_INLINE_MODULE 1
#define CONFIG_USB_NET_DRIVERS 1
#define CONFIG_ARCH_USE_QUEUED_RWLOCKS 1
#define CONFIG_SCSI_SAS_LIBSAS_MODULE 1
#define CONFIG_DVB_TUA6100_MODULE 1
#define CONFIG_CRYPTO_LIB_SHA1_ARCH 1
#define CONFIG_PPC_BARRIER_NOSPEC 1
#define CONFIG_NFT_HASH_MODULE 1
#define CONFIG_IEEE802154_FAKELB_MODULE 1
#define CONFIG_AIR_EN8811H_PHY_MODULE 1
#define CONFIG_PATA_NINJA32_MODULE 1
#define CONFIG_MFD_MAX77759_MODULE 1
#define CONFIG_UBIFS_FS_AUTHENTICATION 1
#define CONFIG_BRCMFMAC_SDIO 1
#define CONFIG_DRM_PANEL_SAMSUNG_S6E3FC2X01_MODULE 1
#define CONFIG_LEDS_TRIGGER_PATTERN_MODULE 1
#define CONFIG_REGULATOR_RTQ2208_MODULE 1
#define CONFIG_RTC_DRV_ABX80X_MODULE 1
#define CONFIG_DM_CRYPT_MODULE 1
#define CONFIG_IPV6_FOU_TUNNEL_MODULE 1
#define CONFIG_IRQ_SIM 1
#define CONFIG_HID_THRUSTMASTER_MODULE 1
#define CONFIG_RV_REACTORS 1
#define CONFIG_I2C_HID_OF_ELAN_MODULE 1
#define CONFIG_NFS_FS_MODULE 1
#define CONFIG_DVB_S5H1420_MODULE 1
#define CONFIG_BE2NET_BE3 1
#define CONFIG_NET_VENDOR_NATSEMI 1
#define CONFIG_RTC_DRV_BQ32K_MODULE 1
#define CONFIG_MD_BITMAP_FILE 1
#define CONFIG_FUTEX_PI 1
#define CONFIG_HAS_IOPORT_MAP 1
#define CONFIG_XFRM_STATISTICS 1
#define CONFIG_ZRAM_BACKEND_LZO 1
#define CONFIG_IP_VS_PROTO_AH 1
#define CONFIG_RV_MON_SCO 1
#define CONFIG_RTC_INTF_DEV 1
#define CONFIG_NETFILTER_SYNPROXY_MODULE 1
#define CONFIG_DVB_M88RS2000_MODULE 1
#define CONFIG_IP6_NF_MATCH_IPV6HEADER_MODULE 1
#define CONFIG_NFS_V4_0 1
#define CONFIG_CRYPTO_DRBG_HMAC 1
#define CONFIG_BRIDGE_EBT_STP_MODULE 1
#define CONFIG_NFS_LOCALIO 1
#define CONFIG_DRM_SSD130X_I2C_MODULE 1
#define CONFIG_USB_SERIAL_KEYSPAN_MODULE 1
#define CONFIG_DVB_SI2165_MODULE 1
#define CONFIG_RTL8723BE_MODULE 1
#define CONFIG_PCIE_PLDA_HOST 1
#define CONFIG_DRM_PANEL_VISIONOX_G2647FB105_MODULE 1
#define CONFIG_IP6_NF_MATCH_FRAG_MODULE 1
#define CONFIG_REALTEK_PHY_HWMON 1
#define CONFIG_PATA_ALI_MODULE 1
#define CONFIG_RTC_DRV_PCF85063_MODULE 1
#define CONFIG_MLX5_EN_IPSEC 1
#define CONFIG_SENSORS_EMC6W201_MODULE 1
#define CONFIG_LINEDISP_MODULE 1
#define CONFIG_SENSORS_AMC6821_MODULE 1
#define CONFIG_MTD_NAND_ECC_SW_HAMMING 1
#define CONFIG_BNA_MODULE 1
#define CONFIG_MT7615_COMMON_MODULE 1
#define CONFIG_DRM_DISPLAY_HELPER_MODULE 1
#define CONFIG_BRIDGE_EBT_MARK_T_MODULE 1
#define CONFIG_TRACE_EVAL_MAP_FILE 1
#define CONFIG_DRM_PANEL_RONBO_RB070D30_MODULE 1
#define CONFIG_NF_CONNTRACK_PPTP_MODULE 1
#define CONFIG_ARCH_MMAP_RND_BITS 14
#define CONFIG_TYPEC_FUSB302_MODULE 1
#define CONFIG_SENSORS_W83792D_MODULE 1
#define CONFIG_LIST_KUNIT_TEST_MODULE 1
#define CONFIG_EROFS_FS_ZIP_LZMA 1
#define CONFIG_DRM_TOSHIBA_TC358768_MODULE 1
#define CONFIG_SENSORS_TMP464_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_IPCOMP_MODULE 1
#define CONFIG_KPROBES_ON_FTRACE 1
#define CONFIG_MLX5_MPFS 1
#define CONFIG_XFRM_SUB_POLICY 1
#define CONFIG_ARCH_DEFAULT_CRASH_DUMP 1
#define CONFIG_RT2800PCI_MODULE 1
#define CONFIG_IP6_NF_MATCH_RPFILTER_MODULE 1
#define CONFIG_DRM_BOCHS_MODULE 1
#define CONFIG_MM_ID 1
#define CONFIG_NET_SCH_SFB_MODULE 1
#define CONFIG_IBMVIO 1
#define CONFIG_HID_SENSOR_HUMIDITY_MODULE 1
#define CONFIG_RTW89_8851B_MODULE 1
#define CONFIG_MDIO_MODULE 1
#define CONFIG_NF_CT_PROTO_UDPLITE 1
#define CONFIG_XFS_FS 1
#define CONFIG_DVB_USB_DVBSKY_MODULE 1
#define CONFIG_SX9310_MODULE 1
#define CONFIG_SENSORS_AD7414_MODULE 1
#define CONFIG_BMI160_MODULE 1
#define CONFIG_NLS_CODEPAGE_857_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_TEE_MODULE 1
#define CONFIG_PROFILING 1
#define CONFIG_VIDEO_SAA7134_GO7007_MODULE 1
#define CONFIG_RTW88_8812A_MODULE 1
#define CONFIG_HW_RANDOM_TIMERIOMEM_MODULE 1
#define CONFIG_RT2X00_LIB_FIRMWARE 1
#define CONFIG_BT_HCIBCM203X_MODULE 1
#define CONFIG_TEST_LOCKUP_MODULE 1
#define CONFIG_SND_HDA_CODEC_HDMI_ATI_MODULE 1
#define CONFIG_HID_SAMSUNG_MODULE 1
#define CONFIG_MEDIA_TUNER_IT913X_MODULE 1
#define CONFIG_USB4_NET_MODULE 1
#define CONFIG_INT_SQRT_KUNIT_TEST_MODULE 1
#define CONFIG_NLS_CODEPAGE_864_MODULE 1
#define CONFIG_EPAPR_BOOT 1
#define CONFIG_CAN_8DEV_USB_MODULE 1
#define CONFIG_USB_CXACRU_MODULE 1
#define CONFIG_KVM_VFIO 1
#define CONFIG_F2FS_IOSTAT 1
#define CONFIG_HID_BPF 1
#define CONFIG_COMMON_CLK_VC7_MODULE 1
#define CONFIG_6LOWPAN_NHC_ROUTING_MODULE 1
#define CONFIG_USB_SERIAL_FTDI_SIO_MODULE 1
#define CONFIG_SCSI_CONSTANTS 1
#define CONFIG_HAVE_ARCH_TRANSPARENT_HUGEPAGE 1
#define CONFIG_NET_SCH_FIFO 1
#define CONFIG_BRIDGE_VLAN_FILTERING 1
#define CONFIG_BT_HCIUART_ATH3K 1
#define CONFIG_GPIO_VIRTUSER_MODULE 1
#define CONFIG_TYPEC_TCPM_MODULE 1
#define CONFIG_IIO_ST_MAGN_3AXIS_MODULE 1
#define CONFIG_GPIO_MPSSE_MODULE 1
#define CONFIG_INFINIBAND_EFA_MODULE 1
#define CONFIG_NLS_DEFAULT "utf8"
#define CONFIG_SND_HDA_CODEC_CS420X_MODULE 1
#define CONFIG_MEDIA_TUNER_XC4000_MODULE 1
#define CONFIG_SENSORS_MP2975_REGULATOR 1
#define CONFIG_KFIFO_KUNIT_TEST_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_ELO 1
#define CONFIG_EROFS_FS_ONDEMAND 1
#define CONFIG_RTW88_8723X_MODULE 1
#define CONFIG_CRYPTO_DH_RFC7919_GROUPS 1
#define CONFIG_SENSORS_ADT7462_MODULE 1
#define CONFIG_RTW88_88XXA_MODULE 1
#define CONFIG_CC_HAS_MIN_FUNCTION_ALIGNMENT 1
#define CONFIG_SENSORS_HS3001_MODULE 1
#define CONFIG_JOYSTICK_XPAD_MODULE 1
#define CONFIG_BT_MRVL_MODULE 1
#define CONFIG_SCSI_FDOMAIN_MODULE 1
#define CONFIG_SENSORS_TC74_MODULE 1
#define CONFIG_DMA_NEED_SYNC 1
#define CONFIG_VEML6030_MODULE 1
#define CONFIG_AF_RXRPC_DEBUG 1
#define CONFIG_NLS_ISO8859_13_MODULE 1
#define CONFIG_MMC_SDHCI_PCI_MODULE 1
#define CONFIG_TOUCHSCREEN_EGALAX_MODULE 1
#define CONFIG_MMC_VIA_SDMMC_MODULE 1
#define CONFIG_DRM_AMD_DC 1
#define CONFIG_MTDRAM_TOTAL_SIZE 4096
#define CONFIG_REGULATOR 1
#define CONFIG_TYPEC_MUX_PI3USB30532_MODULE 1
#define CONFIG_HID_PLAYSTATION_MODULE 1
#define CONFIG_SENSORS_INA3221_MODULE 1
#define CONFIG_NFP_MODULE 1
#define CONFIG_6LOWPAN_GHC_EXT_HDR_DEST_MODULE 1
#define CONFIG_VIDEO_LM3646_MODULE 1
#define CONFIG_INPUT_SPARSEKMAP_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_CONNLABEL_MODULE 1
#define CONFIG_TORTURE_TEST_MODULE 1
#define CONFIG_DVB_LGDT3306A_MODULE 1
#define CONFIG_GRO_CELLS 1
#define CONFIG_PPC_ICP_HV 1
#define CONFIG_CRYPTO_JITTERENTROPY_MEMORY_BLOCKSIZE 32
#define CONFIG_PACKET_DIAG 1
#define CONFIG_GENERIC_STRNCPY_FROM_USER 1
#define CONFIG_ISCSI_TARGET_CXGB4_MODULE 1
#define CONFIG_HID_RMI_MODULE 1
#define CONFIG_BE2NET_BE2 1
#define CONFIG_TEXTSEARCH 1
#define CONFIG_XFS_BTREE_IN_MEM 1
#define CONFIG_HAVE_MOVE_PMD 1
#define CONFIG_NETXEN_NIC_MODULE 1
#define CONFIG_VHOST_VDPA_MODULE 1
#define CONFIG_IWLWIFI_DEBUGFS 1
#define CONFIG_NETFILTER_XT_MATCH_RECENT_MODULE 1
#define CONFIG_PROC_EVENTS 1
#define CONFIG_INFINIBAND_SRP_MODULE 1
#define CONFIG_FAILOVER_MODULE 1
#define CONFIG_TYPEC_MUX_NB7VPQ904M_MODULE 1
#define CONFIG_PAGE_MAPCOUNT 1
#define CONFIG_LINEAR_RANGES 1
#define CONFIG_SATA_VIA_MODULE 1
#define CONFIG_PATA_NS87415_MODULE 1
#define CONFIG_INPUT_MAX77650_ONKEY_MODULE 1
#define CONFIG_BOSCH_BNO055_SERIAL_MODULE 1
#define CONFIG_NF_TPROXY_IPV4_MODULE 1
#define CONFIG_NF_CONNTRACK_IRC_MODULE 1
#define CONFIG_SCHED_STACK_END_CHECK 1
#define CONFIG_DM_LOG_USERSPACE_MODULE 1
#define CONFIG_BASE64_KUNIT_MODULE 1
#define CONFIG_LEDS_LP5812_MODULE 1
#define CONFIG_BRIDGE_EBT_BROUTE_MODULE 1
#define CONFIG_CHARGER_RT9756_MODULE 1
#define CONFIG_VIRTIO_DMA_SHARED_BUFFER_MODULE 1
#define CONFIG_SND_HDA_CORE_MODULE 1
#define CONFIG_HID 1
#define CONFIG_CRYPTO_LZ4_MODULE 1
#define CONFIG_XILINX_XDMA_MODULE 1
#define CONFIG_DM_VERITY_VERIFY_ROOTHASH_SIG_SECONDARY_KEYRING 1
#define CONFIG_KEYBOARD_ADP5585_MODULE 1
#define CONFIG_SCSI_MYRS_MODULE 1
#define CONFIG_NFC_PN533_MODULE 1
#define CONFIG_MTK_T7XX_MODULE 1
#define CONFIG_NE2K_PCI_MODULE 1
#define CONFIG_NET_TEAM_MODE_RANDOM_MODULE 1
#define CONFIG_SATA_VITESSE_MODULE 1
#define CONFIG_VIDEO_BT856_MODULE 1
#define CONFIG_NFT_MASQ_MODULE 1
#define CONFIG_NET_IFE_SKBPRIO_MODULE 1
#define CONFIG_HID_CP2112_MODULE 1
#define CONFIG_NET_SCH_TEQL_MODULE 1
#define CONFIG_CRYPTO_LIB_UTILS 1
#define CONFIG_SG_POOL 1
#define CONFIG_SND_HDA_CODEC_CM9825_MODULE 1
#define CONFIG_CRYPTO_DH 1
#define CONFIG_KFENCE 1
#define CONFIG_MTD_BLOCK2MTD_MODULE 1
#define CONFIG_CC_HAS_WORKING_NOSANITIZE_ADDRESS 1
#define CONFIG_NETFILTER_XT_TARGET_HMARK_MODULE 1
#define CONFIG_BLK_CGROUP_RWSTAT 1
#define CONFIG_NTFS3_FS_POSIX_ACL 1
#define CONFIG_DRM_PANEL_BOE_BF060Y8M_AJ0_MODULE 1
#define CONFIG_SMS_SDIO_DRV_MODULE 1
#define CONFIG_DRM_PANEL_VISIONOX_R66451_MODULE 1
#define CONFIG_CORE_DUMP_DEFAULT_ELF_HEADERS 1
#define CONFIG_NET_EMATCH 1
#define CONFIG_SND_HDA_POWER_SAVE_DEFAULT 1
#define CONFIG_B44_PCI_AUTOSELECT 1
#define CONFIG_TYPEC_MODULE 1
#define CONFIG_MODULE_SIG_SHA512 1
#define CONFIG_IP6_NF_MATCH_HL_MODULE 1
#define CONFIG_DRM_PANEL_HYDIS_HV101HD1_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_OWNER_MODULE 1
#define CONFIG_BINFMT_MISC_MODULE 1
#define CONFIG_CAN_PEAK_PCIEFD_MODULE 1
#define CONFIG_BLK_DEV_RNBD_CLIENT_MODULE 1
#define CONFIG_MT792x_LIB_MODULE 1
#define CONFIG_DVB_STV6110x_MODULE 1
#define CONFIG_PPC_QUEUED_SPINLOCKS 1
#define CONFIG_CRYPTO_DEV_VMX_ENCRYPT_MODULE 1
#define CONFIG_USB_PCI 1
#define CONFIG_USB_GSPCA_FINEPIX_MODULE 1
#define CONFIG_MRP_MODULE 1
#define CONFIG_CRYPTO_LIB_POLYVAL_KUNIT_TEST_MODULE 1
#define CONFIG_SCHED_MM_CID 1
#define CONFIG_NFSD_BLOCKLAYOUT 1
#define CONFIG_KVM_BOOK3S_HV_NESTED_PMU_WORKAROUND 1
#define CONFIG_PWM_DWC_CORE_MODULE 1
#define CONFIG_ADVISE_SYSCALLS 1
#define CONFIG_MT76x2U_MODULE 1
#define CONFIG_THERMAL_GOV_STEP_WISE 1
#define CONFIG_NET 1
#define CONFIG_MOUSE_APPLETOUCH_MODULE 1
#define CONFIG_ARCH_ENABLE_SPLIT_PMD_PTLOCK 1
#define CONFIG_HAVE_OPTPROBES 1
#define CONFIG_WLAN_VENDOR_MICROCHIP 1
#define CONFIG_MPL115_I2C_MODULE 1
#define CONFIG_SCSI_MVSAS_TASKLET 1
#define CONFIG_NET_DSA_TAG_GSWIP_MODULE 1
#define CONFIG_XFRM_IPCOMP_MODULE 1
#define CONFIG_DVB_USB_PCTV452E_MODULE 1
#define CONFIG_AD5110_MODULE 1
#define CONFIG_DEFAULT_CUBIC 1
#define CONFIG_XFRM_USER 1
#define CONFIG_HOTPLUG_PCI_OCTEONEP 1
#define CONFIG_NLS_CODEPAGE_865_MODULE 1
#define CONFIG_USB_DEFAULT_AUTHORIZATION_MODE 1
#define CONFIG_NET_REDIRECT 1
#define CONFIG_USB_LAN78XX_MODULE 1
#define CONFIG_VIRTIO_RTC_CLASS 1
#define CONFIG_SMS_USB_DRV_MODULE 1
#define CONFIG_GACT_PROB 1
#define CONFIG_NET_VENDOR_ENGLEDER 1
#define CONFIG_ATL1E_MODULE 1
#define CONFIG_VIDEO_CAMERA_SENSOR 1
#define CONFIG_TRACEPOINTS 1
#define CONFIG_SCHED_SMT 1
#define CONFIG_LIRC 1
#define CONFIG_UNINLINE_SPIN_UNLOCK 1
#define CONFIG_PCSPKR_PLATFORM 1
#define CONFIG_INITRAMFS_SOURCE ""
#define CONFIG_EXFAT_FS_MODULE 1
#define CONFIG_VCNL3020_MODULE 1
#define CONFIG_B43_PHY_N 1
#define CONFIG_TIME_KUNIT_TEST_MODULE 1
#define CONFIG_SIGNED_PE_FILE_VERIFICATION 1
#define CONFIG_UNIX98_PTYS 1
#define CONFIG_RANDOMIZE_KSTACK_OFFSET 1
#define CONFIG_CLKBLD_I8253 1
#define CONFIG_USB_APPLEDISPLAY_MODULE 1
#define CONFIG_PPP_SYNC_TTY_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_CONNBYTES_MODULE 1
#define CONFIG_JOYSTICK_GRIP_MP_MODULE 1
#define CONFIG_RTW88_8822BU_MODULE 1
#define CONFIG_SATA_ULI_MODULE 1
#define CONFIG_DMA_DECLARE_COHERENT 1
#define CONFIG_PCI_PF_STUB_MODULE 1
#define CONFIG_DRM_CIRRUS_QEMU_MODULE 1
#define CONFIG_DRM_SUBALLOC_HELPER_MODULE 1
#define CONFIG_PPP_BSDCOMP_MODULE 1
#define CONFIG_BRIDGE_EBT_LIMIT_MODULE 1
#define CONFIG_SENSORS_MC34VR500_MODULE 1
#define CONFIG_CMDLINE_KUNIT_TEST_MODULE 1
#define CONFIG_HID_WIIMOTE_MODULE 1
#define CONFIG_DRM_SCHED_KUNIT_TEST_MODULE 1
#define CONFIG_ARCH_SPARSEMEM_DEFAULT 1
#define CONFIG_BACKLIGHT_KTZ8866_MODULE 1
#define CONFIG_DRM_PANEL_HIMAX_HX83112B_MODULE 1
#define CONFIG_VFIO_MODULE 1
#define CONFIG_NFC_HCI_MODULE 1
#define CONFIG_NLS_ISO8859_14_MODULE 1
#define CONFIG_EXECMEM 1
#define CONFIG_DVB_ZD1301_DEMOD_MODULE 1
#define CONFIG_NET_VENDOR_REALTEK 1
#define CONFIG_XFRM_OFFLOAD 1
#define CONFIG_ZRAM_MULTI_COMP 1
#define CONFIG_IPV6_SEG6_LWTUNNEL 1
#define CONFIG_MEDIA_TUNER_TDA18212_MODULE 1
#define CONFIG_NET_DSA_TAG_RTL4_A_MODULE 1
#define CONFIG_SENSORS_LTC2947_I2C_MODULE 1
#define CONFIG_OVERLAY_FS_MODULE 1
#define CONFIG_SECONDARY_TRUSTED_KEYRING 1
#define CONFIG_BT_MODULE 1
#define CONFIG_ZRAM_BACKEND_DEFLATE 1
#define CONFIG_TOUCHSCREEN_USB_IDEALTEK 1
#define CONFIG_VT 1
#define CONFIG_USB_GSPCA_JEILINJ_MODULE 1
#define CONFIG_NET_CLS_ACT 1
#define CONFIG_DVB_USB_MXL111SF_MODULE 1
#define CONFIG_ET131X_MODULE 1
#define CONFIG_EEPROM_MAX6875_MODULE 1
#define CONFIG_VIDEO_VIMC_MODULE 1
#define CONFIG_LIBNVDIMM_MODULE 1
#define CONFIG_IP_SET_HASH_IPPORTIP_MODULE 1
#define CONFIG_SMP 1
#define CONFIG_VLAN_8021Q_MVRP 1
#define CONFIG_SCSI_BNX2_ISCSI_MODULE 1
#define CONFIG_R8169_LEDS 1
#define CONFIG_BT_HCIBTUSB_RTL 1
#define CONFIG_CFG80211_MODULE 1
#define CONFIG_NFT_QUOTA_MODULE 1
#define CONFIG_KVM_XICS 1
#define CONFIG_CEPH_FS_POSIX_ACL 1
#define CONFIG_BT_RTL_MODULE 1
#define CONFIG_FSI_MASTER_ASPEED_MODULE 1
#define CONFIG_SECURITYFS 1
#define CONFIG_VIDEO_CX88_DVB_MODULE 1
#define CONFIG_VIDEO_SOLO6X10_MODULE 1
#define CONFIG_QUEUED_RWLOCKS 1
#define CONFIG_DRM_PANEL_ILITEK_ILI9806E_MODULE 1
#define CONFIG_BT_HCIVHCI_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_DCCP_MODULE 1
#define CONFIG_LSM_MMAP_MIN_ADDR 65535
#define CONFIG_MAC80211_RC_MINSTREL 1
#define CONFIG_ARCH_HAS_VDSO_ARCH_DATA 1
#define CONFIG_VIDEO_CX231XX_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_HASHLIMIT_MODULE 1
#define CONFIG_HAVE_ARCH_AUDITSYSCALL 1
#define CONFIG_IMA_QUEUE_EARLY_BOOT_KEYS 1
#define CONFIG_BLK_DEV_LOOP_MODULE 1
#define CONFIG_IPV6_SUBTREES 1
#define CONFIG_FTRACE_MCOUNT_USE_PATCHABLE_FUNCTION_ENTRY 1
#define CONFIG_HID_EMS_FF_MODULE 1
#define CONFIG_SPEAKUP_MODULE 1
#define CONFIG_PSERIES_WDT_MODULE 1
#define CONFIG_MICROSEMI_PHY_MODULE 1
#define CONFIG_HID_SENSOR_IIO_TRIGGER_MODULE 1
#define CONFIG_VIDEO_OV2659_MODULE 1
#define CONFIG_BTRFS_FS_POSIX_ACL 1
#define CONFIG_HID_UDRAW_PS3_MODULE 1
#define CONFIG_HID_APPLE_MODULE 1
#define CONFIG_DVB_USB_DIGITV_MODULE 1
#define CONFIG_AC97_BUS_MODULE 1
#define CONFIG_STK3310_MODULE 1
#define CONFIG_VIDEO_SAA7134_ALSA_MODULE 1
#define CONFIG_USB_BELKIN 1
#define CONFIG_PCI_PWRCTRL_TC9563_MODULE 1
#define CONFIG_DRM_RADEON_USERPTR 1
#define CONFIG_RCU_STALL_COMMON 1
#define CONFIG_USB_UEAGLEATM_MODULE 1
#define CONFIG_BLK_DEV_ZONED 1
#define CONFIG_RT2X00_LIB_MODULE 1
#define CONFIG_HAVE_ASM_MODVERSIONS 1
#define CONFIG_DVB_STB6100_MODULE 1
#define CONFIG_HID_SENSOR_IIO_COMMON_MODULE 1
#define CONFIG_TCP_CONG_CDG_MODULE 1
#define CONFIG_AD7405_MODULE 1
#define CONFIG_INFINIBAND_ADDR_TRANS 1
#define CONFIG_NFS_DEBUG 1
#define CONFIG_NET_FC 1
#define CONFIG_BT_HCIRSI_MODULE 1
#define CONFIG_DA_MON_EVENTS_ID 1
#define CONFIG_PSERIES_PLPKS_SED 1
#define CONFIG_SCSI_LOGGING 1
#define CONFIG_VIDEO_ISL7998X_MODULE 1
#define CONFIG_PPC_RTAS_DAEMON 1
#define CONFIG_HAVE_PERF_EVENTS 1
#define CONFIG_PHY_CADENCE_SALVO_MODULE 1
#define CONFIG_RFKILL_LEDS 1
#define CONFIG_DRM_FBDEV_OVERALLOC 100
#define CONFIG_CRYPTO_SELFTESTS 1
#define CONFIG_MSA311_MODULE 1
#define CONFIG_SCSI_IBMVFC_MODULE 1
#define CONFIG_IO_STRICT_DEVMEM 1
#define CONFIG_VIDEO_OV2685_MODULE 1
#define CONFIG_DM_VERITY_MODULE 1
#define CONFIG_NFT_NUMGEN_MODULE 1
#define CONFIG_ATM_MODULE 1
#define CONFIG_REGULATOR_TPS65185_MODULE 1
#define CONFIG_NET_SCH_MQPRIO_MODULE 1
#define CONFIG_SENSORS_APDS990X_MODULE 1
#define CONFIG_AIC7XXX_DEBUG_MASK 0
#define CONFIG_W1_SLAVE_DS2423_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_U32_MODULE 1
#define CONFIG_DVB_DM1105_MODULE 1
#define CONFIG_SCSI_SYM53C8XX_MMIO 1
#define CONFIG_NLS_CODEPAGE_855_MODULE 1
#define CONFIG_RTC_DRV_DS1511_MODULE 1
#define CONFIG_NGBE_MODULE 1
#define CONFIG_IP6_NF_TARGET_HL_MODULE 1
#define CONFIG_BMI270_I2C_MODULE 1
#define CONFIG_NET_KEY_MIGRATE 1
#define CONFIG_APDS9160_MODULE 1
#define CONFIG_SND_INDIGODJX_MODULE 1
#define CONFIG_USB_STORAGE_KARMA_MODULE 1
#define CONFIG_VIDEO_AK7375_MODULE 1
#define CONFIG_HID_MAYFLASH_MODULE 1
#define CONFIG_B43LEGACY_PCI_AUTOSELECT 1
#define CONFIG_MEMSTICK_JMICRON_38X_MODULE 1
#define CONFIG_MEDIA_TUNER_QT1010_MODULE 1
#define CONFIG_BONDING_MODULE 1
#define CONFIG_PCIE_TPH 1
#define CONFIG_LTO_NONE 1
#define CONFIG_LEDS_QNAP_MCU_MODULE 1
#define CONFIG_SND_HDA_CODEC_HDMI_NVIDIA_MODULE 1
#define CONFIG_NLS_CODEPAGE_862_MODULE 1
#define CONFIG_TYPEC_MUX_IT5205_MODULE 1
#define CONFIG_SERIAL_8250_PCI1XXXX 1
#define CONFIG_HID_KUNIT_TEST_MODULE 1
#define CONFIG_DVB_ISL6405_MODULE 1
#define CONFIG_RTW88_8723DS_MODULE 1
#define CONFIG_SENSORS_ADT7475_MODULE 1
#define CONFIG_GLOB_KUNIT_TEST_MODULE 1
#define CONFIG_SENSORS_MP2993_MODULE 1
#define CONFIG_CAN_NETLINK 1
#define CONFIG_JFFS2_FS_XATTR 1
#define CONFIG_BACKLIGHT_CLASS_DEVICE 1
#define CONFIG_CFG80211_DEBUGFS 1
#define CONFIG_CRYPTO_DEV_VMX 1
#define CONFIG_NF_TABLES_MODULE 1
#define CONFIG_CRYPTO_MANAGER 1
#define CONFIG_EPROBE_EVENTS 1
#define CONFIG_LOGO_LINUX_CLUT224 1
#define CONFIG_SENSORS_PCF8591_MODULE 1
#define CONFIG_SUSPEND_FREEZER 1
#define CONFIG_NET_VENDOR_MARVELL 1
#define CONFIG_CAN_ISOTP_MODULE 1
#define CONFIG_RTW89_CORE_MODULE 1
#define CONFIG_SCSI_DC395x_MODULE 1
#define CONFIG_DVB_NET 1
#define CONFIG_SSB_MODULE 1
#define CONFIG_CRYPTO_LIB_POLY1305_GENERIC 1
#define CONFIG_PHY_COMMON_PROPS_TEST_MODULE 1
#define CONFIG_DECOMPRESS_LZO 1
#define CONFIG_OVERLAY_FS_REDIRECT_ALWAYS_FOLLOW 1
#define CONFIG_NLS_ISO8859_15_MODULE 1
#define CONFIG_LEDS_PWM_MULTICOLOR_MODULE 1
#define CONFIG_MEDIA_CAMERA_SUPPORT 1
#define CONFIG_CHARLCD_MODULE 1
#define CONFIG_RFKILL_GPIO_MODULE 1
#define CONFIG_NET_DSA_SMSC_LAN9303_I2C_MODULE 1
#define CONFIG_TYPEC_MUX_TUSB1046_MODULE 1
#define CONFIG_SND_SEQ_UMP 1
#define CONFIG_VIDEO_VIVID_CEC 1
#define CONFIG_NET_SCH_PLUG_MODULE 1
#define CONFIG_NET_DSA_TAG_SJA1105_MODULE 1
#define CONFIG_DEV_DAX_CXL_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_CLUSTER_MODULE 1
#define CONFIG_VIDEO_SAA6752HS_MODULE 1
#define CONFIG_PPP_MULTILINK 1
#define CONFIG_LZ4_DECOMPRESS 1
#define CONFIG_USB_SERIAL_IR_MODULE 1
#define CONFIG_SND_HDA_CODEC_CA0132_MODULE 1
#define CONFIG_IP_VS_PROTO_ESP 1
#define CONFIG_SCSI_CHELSIO_FCOE_MODULE 1
#define CONFIG_VIDEO_VD55G1_MODULE 1
#define CONFIG_ARCH_SUPPORTS_SCHED_SMT 1
#define CONFIG_QUOTA_NETLINK_INTERFACE 1
#define CONFIG_USB_EMI62_MODULE 1
#define CONFIG_NFT_DUP_NETDEV_MODULE 1
#define CONFIG_MPRLS0025PA_I2C_MODULE 1
#define CONFIG_SCSI_SAS_HOST_SMP 1
#define CONFIG_IIO_ST_SENSORS_CORE_MODULE 1
#define CONFIG_RTC_MC146818_LIB 1
#define CONFIG_IP6_NF_NAT_MODULE 1
#define CONFIG_RD_LZO 1
#define CONFIG_RTW89_8852C_MODULE 1
#define CONFIG_NET_VENDOR_AQUANTIA 1
#define CONFIG_HID_LENOVO_MODULE 1
#define CONFIG_CEPH_FS_MODULE 1
#define CONFIG_DRM_DISPLAY_HDMI_STATE_HELPER 1
#define CONFIG_OVPN_MODULE 1
#define CONFIG_ZL3073X_I2C_MODULE 1
#define CONFIG_ATA_SFF 1
#define CONFIG_INPUT_MOUSEDEV_SCREEN_X 1024
#define CONFIG_ALTERA_PR_IP_CORE_PLAT_MODULE 1
#define CONFIG_SENSORS_PIM4328_MODULE 1
#define CONFIG_SCSI_PROTO_TEST_MODULE 1
#define CONFIG_STP_MODULE 1
#define CONFIG_ATH9K_PCI_NO_EEPROM_MODULE 1
#define CONFIG_GPIO_MAX77650_MODULE 1
#define CONFIG_NFC_NXP_NCI_MODULE 1
#define CONFIG_TCM_QLA2XXX_MODULE 1
#define CONFIG_PPC_UV 1
#define CONFIG_SENSORS_HTU31_MODULE 1
#define CONFIG_JME_MODULE 1
#define CONFIG_USB_SERIAL_VISOR_MODULE 1
#define CONFIG_BE2NET_MODULE 1
#define CONFIG_NET_SCH_RED_MODULE 1
#define CONFIG_DAMON_VADDR 1
#define CONFIG_DVB_VES1X93_MODULE 1
#define CONFIG_VIDEO_MT9V011_MODULE 1
#define CONFIG_VIDEO_TVEEPROM_MODULE 1
#define CONFIG_NF_TPROXY_IPV6_MODULE 1
#define CONFIG_STAGING 1
#define CONFIG_SCSI_DH_HP_SW_MODULE 1
#define CONFIG_DRM_PANEL_BOE_TD4320_MODULE 1
#define CONFIG_MISC_RTSX_MODULE 1
#define CONFIG_OCXL_BASE 1
#define CONFIG_MEDIA_CONTROLLER 1
#define CONFIG_RTC_DRV_EM3027_MODULE 1
#define CONFIG_WLCORE_SDIO_MODULE 1
#define CONFIG_USB_SUPPORT 1
#define CONFIG_DVB_MXL5XX_MODULE 1
#define CONFIG_TCG_TIS_CORE 1
#define CONFIG_TOUCHSCREEN_HYNITRON_CST816X_MODULE 1
#define CONFIG_IP_VS_LBLC_MODULE 1
#define CONFIG_CRYPTO_JITTERENTROPY 1
#define CONFIG_PREVENT_FIRMWARE_BUILD 1
#define CONFIG_W1_MASTER_UART_MODULE 1
#define CONFIG_PATA_VIA_MODULE 1
#define CONFIG_SENSORS_ATXP1_MODULE 1
#define CONFIG_HW_RANDOM_VIRTIO 1
#define CONFIG_MEGARAID_SAS_MODULE 1
#define CONFIG_IP_VS_TAB_BITS 12
#define CONFIG_MFD_MAX77705_MODULE 1
#define CONFIG_SENSORS_FSP_3Y_MODULE 1
#define CONFIG_PSTORE_COMPRESS 1
#define CONFIG_DRM_VKMS_KUNIT_TEST_MODULE 1
#define CONFIG_DM_ZERO 1
#define CONFIG_RTW88_8814AU_MODULE 1
#define CONFIG_TCG_IBMVTPM 1
#define CONFIG_IP_SET_HASH_NETNET_MODULE 1
#define CONFIG_FW_LOADER_COMPRESS_ZSTD 1
#define CONFIG_IR_REDRAT3_MODULE 1
#define CONFIG_NFT_LIMIT_MODULE 1
#define CONFIG_IPV6_TUNNEL_MODULE 1
#define CONFIG_XZ_DEC_ARMTHUMB 1
#define CONFIG_VIDEO_ADV7604_MODULE 1
#define CONFIG_BT_HCIUART_SERDEV 1
#define CONFIG_RMI4_F21 1
#define CONFIG_CC_HAS_KASAN_GENERIC 1
#define CONFIG_NET_FAILOVER_MODULE 1
#define CONFIG_MLXSW_SPECTRUM_DCB 1
#define CONFIG_KEXEC_FILE 1
#define CONFIG_B43_PIO 1
#define CONFIG_SENSORS_ADM1031_MODULE 1
#define CONFIG_PRINTK_TIME 1
#define CONFIG_RATELIMIT_KUNIT_TEST_MODULE 1
#define CONFIG_DAMON_SYSFS 1
#define CONFIG_TSL2772_MODULE 1
#define CONFIG_TIFM_7XX1_MODULE 1
#define CONFIG_HOLTEK_FF 1
#define CONFIG_KUNIT_EXAMPLE_TEST_MODULE 1
#define CONFIG_BOSCH_BNO055_MODULE 1
#define CONFIG_GREENASIA_FF 1
#define CONFIG_PREEMPT_LAZY 1
#define CONFIG_FSCACHE_STATS 1
#define CONFIG_HFS_KUNIT_TEST_MODULE 1
#define CONFIG_APPLE_MFI_FASTCHARGE_MODULE 1
#define CONFIG_ZLIB_INFLATE 1
#define CONFIG_VIDEO_ML86V7667_MODULE 1
#define CONFIG_FPROBE_EVENTS 1
#define CONFIG_HID_BIGBEN_FF_MODULE 1
#define CONFIG_HID_BETOP_FF_MODULE 1
#define CONFIG_W1_MASTER_DS2482_MODULE 1
#define CONFIG_MEMCPY_KUNIT_TEST_MODULE 1
#define CONFIG_MTD_UBI_MODULE 1
#define CONFIG_NLS_CODEPAGE_863_MODULE 1
#define CONFIG_RTC_DRV_RP5C01_MODULE 1
#define CONFIG_BLK_DEV_RAM_MODULE 1
#define CONFIG_VGA_ARB 1
#define CONFIG_CRYPTO_LIB_CURVE25519_MODULE 1
#define CONFIG_LOGIRUMBLEPAD2_FF 1
#define CONFIG_GENERIC_IOREMAP 1
#define CONFIG_DP83TG720_PHY_MODULE 1
#define CONFIG_ATL1C_MODULE 1
#define CONFIG_APDS9802ALS_MODULE 1
#define CONFIG_PWM_ADP5585_MODULE 1
#define CONFIG_DE2104X_MODULE 1
#define CONFIG_USB_EPSON2888 1
#define CONFIG_ARCH_USE_MEMTEST 1
#define CONFIG_HID_GENERIC 1
#define CONFIG_USB_PHY 1
#define CONFIG_TABLET_USB_AIPTEK_MODULE 1
#define CONFIG_CAN_RAW_MODULE 1
#define CONFIG_VIDEO_OV5675_MODULE 1
#define CONFIG_QCA808X_PHY_MODULE 1
#define CONFIG_CRC_OPTIMIZATIONS 1
#define CONFIG_NET_DSA_XRS700X_MODULE 1
#define CONFIG_VIDEO_ADV748X_MODULE 1
#define CONFIG_APERTURE_HELPERS 1
#define CONFIG_UNICODE 1
#define CONFIG_GAMEPORT_MODULE 1
#define CONFIG_RMI4_I2C_MODULE 1
#define CONFIG_DM_LOG_WRITES_MODULE 1
#define CONFIG_B44_PCI 1
#define CONFIG_USB_SIERRA_NET_MODULE 1
#define CONFIG_SWPHY 1
#define CONFIG_TRUSTED_KEYS_PKWM 1
#define CONFIG_CGROUP_CPUACCT 1
#define CONFIG_SENSORS_ADM1275_MODULE 1
#define CONFIG_VIDEO_SAA717X_MODULE 1
#define CONFIG_MEMSTICK_REALTEK_USB_MODULE 1
#define CONFIG_SND_HDA_COMPONENT 1
#define CONFIG_NETFILTER_NETLINK_HOOK_MODULE 1
#define CONFIG_PA12203001_MODULE 1
#define CONFIG_USB_STORAGE_CYPRESS_ATACB_MODULE 1
#define CONFIG_SIS900_MODULE 1
#define CONFIG_PATA_WINBOND_MODULE 1
#define CONFIG_DRM_SCHED_MODULE 1
#define CONFIG_HSA_AMD_SVM 1
#define CONFIG_REGULATOR_RT6190_MODULE 1
#define CONFIG_DRM_PANEL_BRIDGE 1
#define CONFIG_DVB_USB_GL861_MODULE 1
#define CONFIG_IONIC_MODULE 1
#define CONFIG_IP_VS_WLC_MODULE 1
#define CONFIG_NET_POLL_CONTROLLER 1
#define CONFIG_DW_DMAC_CORE_MODULE 1
#define CONFIG_NET_VENDOR_IBM 1
#define CONFIG_JOYSTICK_ANALOG_MODULE 1
#define CONFIG_INFINIBAND_MTHCA_DEBUG 1
#define CONFIG_HID_XIAOMI_MODULE 1
#define CONFIG_TOUCHSCREEN_EDT_FT5X06_MODULE 1
#define CONFIG_FS_DAX 1
#define CONFIG_RTW89_8852B_MODULE 1
#define CONFIG_BT_HCIUART_3WIRE 1
#define CONFIG_DIMLIB 1
#define CONFIG_6LOWPAN_GHC_ICMPV6_MODULE 1
#define CONFIG_SERIAL_OF_PLATFORM_MODULE 1
#define CONFIG_INPUT_MOUSEDEV_SCREEN_Y 768
#define CONFIG_MEMCG 1
#define CONFIG_DRM_DISPLAY_DP_AUX_CHARDEV 1
#define CONFIG_SENSORS_SPD5118_MODULE 1
#define CONFIG_BH1750_MODULE 1
#define CONFIG_CRYPTO_LIB_CHACHA20POLY1305 1
#define CONFIG_ZSTD_COMPRESS 1
#define CONFIG_SPARSE_IRQ 1
#define CONFIG_NEED_SG_DMA_LENGTH 1
#define CONFIG_VHOST_TASK 1
#define CONFIG_IPV6_SEG6_BPF 1
#define CONFIG_DVB_TUNER_DIB0070_MODULE 1
#define CONFIG_PPC64 1
#define CONFIG_SUSPEND 1
#define CONFIG_SYSCTL_KUNIT_TEST_MODULE 1
#define CONFIG_USB_SERIAL_F8153X_MODULE 1
#define CONFIG_SND_USB_AUDIO_MIDI_V2 1
#define CONFIG_SENSORS_GIGABYTE_WATERFORCE_MODULE 1
#define CONFIG_GRACE_PERIOD_MODULE 1
#define CONFIG_ATH6KL_DEBUG 1
#define CONFIG_NF_FLOW_TABLE_PROCFS 1
#define CONFIG_RTL8192DE_MODULE 1
#define CONFIG_NFC_PN532_UART_MODULE 1
#define CONFIG_CRC_T10DIF_ARCH 1
#define CONFIG_HAVE_FUNCTION_ARG_ACCESS_API 1
#define CONFIG_SUNGEM_PHY_MODULE 1
#define CONFIG_MFD_CS42L43_MODULE 1
#define CONFIG_NFSD_V4_POSIX_ACLS 1
#define CONFIG_BD96801_WATCHDOG_MODULE 1
#define CONFIG_ENVELOPE_DETECTOR_MODULE 1
#define CONFIG_RT2800_LIB_MODULE 1
#define CONFIG_CC_IMPLICIT_FALLTHROUGH "-Wimplicit-fallthrough=5"
#define CONFIG_IP_MULTIPLE_TABLES 1
#define CONFIG_RT2X00_LIB_CRYPTO 1
#define CONFIG_TMP117_MODULE 1
#define CONFIG_PPP_FILTER 1
#define CONFIG_VDPA_SIM_BLOCK_MODULE 1
#define CONFIG_NET_DSA_TAG_OCELOT_8021Q_MODULE 1
#define CONFIG_WLAN_VENDOR_RALINK 1
#define CONFIG_IR_MCE_KBD_DECODER_MODULE 1
#define CONFIG_EDAC_MEM_REPAIR 1
#define CONFIG_RATIONAL 1
#define CONFIG_SENSORS_LM25066_REGULATOR 1
#define CONFIG_INPUT_GPIO_BEEPER_MODULE 1
#define CONFIG_NF_CONNTRACK_MODULE 1
#define CONFIG_W1_SLAVE_DS2438_MODULE 1
#define CONFIG_MEMORY_HOTPLUG 1
#define CONFIG_CRC64 1
#define CONFIG_TRUSTED_KEYS_TPM 1
#define CONFIG_KEY_DH_OPERATIONS 1
#define CONFIG_HID_GYRATION_MODULE 1
#define CONFIG_NTSYNC_MODULE 1
#define CONFIG_UBSAN 1
#define CONFIG_VIDEO_TW2804_MODULE 1
#define CONFIG_GNSS_SERIAL_MODULE 1
#define CONFIG_ARCH_SUSPEND_NONZERO_CPU 1
#define CONFIG_SND_USB_LINE6_MODULE 1
#define CONFIG_REGMAP_RAM_MODULE 1
#define CONFIG_TLS_DEVICE 1
#define CONFIG_DRM_LONTIUM_LT9611UXC_MODULE 1
#define CONFIG_DVB_USB_TTUSB2_MODULE 1
#define CONFIG_WIL6210_ISR_COR 1
#define CONFIG_SND_MAX_CARDS 32
#define CONFIG_ARCH_HAS_PMEM_API 1
#define CONFIG_IIO_MUX_MODULE 1
#define CONFIG_ARCH_MIGHT_HAVE_PC_PARPORT 1
#define CONFIG_RD_GZIP 1
#define CONFIG_VIDEO_THS7303_MODULE 1
#define CONFIG_FB_DEFERRED_IO 1
#define CONFIG_LIBWX_MODULE 1
#define CONFIG_NET_SCH_NETEM_MODULE 1
#define CONFIG_IP_NF_MATCH_RPFILTER_MODULE 1
#define CONFIG_MSDOS_PARTITION 1
#define CONFIG_B43_SDIO 1
#define CONFIG_SYNTH_EVENTS 1
#define CONFIG_DVB_USB_DTV5100_MODULE 1
#define CONFIG_POWER_SUPPLY_HWMON 1
#define CONFIG_PHYLIB_LEDS 1
#define CONFIG_NETFILTER_NETLINK_LOG_MODULE 1
#define CONFIG_DRM_PANEL_HIMAX_HX8279_MODULE 1
#define CONFIG_CPU_FREQ_GOV_CONSERVATIVE 1
#define CONFIG_ZSTD_COMMON 1
#define CONFIG_DEFAULT_SECURITY_DAC 1
#define CONFIG_TOUCHSCREEN_USB_3M 1
#define CONFIG_WATCHDOG 1
#define CONFIG_BRIDGE_NETFILTER_MODULE 1
#define CONFIG_NET_VENDOR_RDC 1
#define CONFIG_MODULE_DECOMPRESS 1
#define CONFIG_NLS_CODEPAGE_860_MODULE 1
#define CONFIG_LOGITECH_FF 1
#define CONFIG_VIDEO_VIVID_MODULE 1
#define CONFIG_TEST_VMALLOC_MODULE 1
#define CONFIG_IR_IGORPLUGUSB_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_CONNSECMARK_MODULE 1
#define CONFIG_SMT_NUM_THREADS_DYNAMIC 1
#define CONFIG_I2C_HID_CORE_MODULE 1
#define CONFIG_MWIFIEX_SDIO_MODULE 1
#define CONFIG_XFRM 1
#define CONFIG_PPC_XIVE_NATIVE 1
#define CONFIG_IPV6_PIMSM_V2 1
#define CONFIG_SCHED_CORE 1
#define CONFIG_TCG_TIS_I2C_ATMEL_MODULE 1
#define CONFIG_GNSS_MTK_SERIAL_MODULE 1
#define CONFIG_DVB_A8293_MODULE 1
#define CONFIG_PRESTERA_MODULE 1
#define CONFIG_ORANGEFS_FS_MODULE 1
#define CONFIG_MLX90614_MODULE 1
#define CONFIG_LEDS_MAX77650_MODULE 1
#define CONFIG_IRQCHIP 1
#define CONFIG_ARCH_HAS_FAST_MULTIPLIER 1
#define CONFIG_IOMMUFD_DRIVER_CORE 1
#define CONFIG_CGROUP_HUGETLB 1
#define CONFIG_FW_LOADER 1
#define CONFIG_INV_ICM42600_I2C_MODULE 1
#define CONFIG_USB_GSPCA_KINECT_MODULE 1
#define CONFIG_RTC_SYSTOHC 1
#define CONFIG_DRM_PANEL_SAMSUNG_AMS581VF01_MODULE 1
#define CONFIG_JFS_FS_MODULE 1
#define CONFIG_SENSORS_W83791D_MODULE 1
#define CONFIG_TOUCHSCREEN_RM_TS_MODULE 1
#define CONFIG_FPGA_DFL_MODULE 1
#define CONFIG_INET_IPCOMP_MODULE 1
#define CONFIG_ROMFS_FS_MODULE 1
#define CONFIG_IP6_NF_MATCH_RT_MODULE 1
#define CONFIG_SCSI_WD719X_MODULE 1
#define CONFIG_MEDIA_TUNER_MT20XX_MODULE 1
#define CONFIG_INFINIBAND_CXGB4_MODULE 1
#define CONFIG_VIDEO_IMX219_MODULE 1
#define CONFIG_L2TP_DEBUGFS_MODULE 1
#define CONFIG_SENSORS_TPS25990_REGULATOR 1
#define CONFIG_DRM_BRIDGE_CONNECTOR 1
#define CONFIG_DM_RAID_MODULE 1
#define CONFIG_EXPORTFS 1
#define CONFIG_RPR0521_MODULE 1
#define CONFIG_VIDEO_HDPVR_MODULE 1
#define CONFIG_CRYPTO_HCTR2_MODULE 1
#define CONFIG_MAC80211_HWSIM_MODULE 1
#define CONFIG_TCP_CONG_VEGAS_MODULE 1
#define CONFIG_INFINIBAND_IPOIB_DEBUG_DATA 1
#define CONFIG_GPIO_DLN2_MODULE 1
#define CONFIG_PANIC_TIMEOUT 0
#define CONFIG_VIDEO_IMX296_MODULE 1
#define CONFIG_PTP_1588_CLOCK 1
#define CONFIG_MDIO_BCM_UNIMAC_MODULE 1
#define CONFIG_DRM_SII902X_MODULE 1
#define CONFIG_FW_LOADER_COMPRESS_XZ 1
#define CONFIG_TYPEC_MUX_PS883X_MODULE 1
#define CONFIG_NET_CORE 1
#define CONFIG_DVB_LGDT330X_MODULE 1
#define CONFIG_SOFT_WATCHDOG_MODULE 1
#define CONFIG_RTW89_8852A_MODULE 1
#define CONFIG_LD_IS_BFD 1
#define CONFIG_INFINIBAND_ISERT_MODULE 1
#define CONFIG_NAMESPACES 1
#define CONFIG_SND_PCM_ELD 1
#define CONFIG_ARCH_SELECTS_CRASH_DUMP 1
#define CONFIG_DECOMPRESS_GZIP 1
#define CONFIG_RTC_DRV_OPAL_MODULE 1
#define CONFIG_HWMON 1
#define CONFIG_TYPEC_TPS6598X_MODULE 1
#define CONFIG_BRCMFMAC_MODULE 1
#define CONFIG_I2C_PCI1XXXX_MODULE 1
#define CONFIG_IP_NF_ARP_MANGLE_MODULE 1
#define CONFIG_RTW88_8821CE_MODULE 1
#define CONFIG_OPENSSL_SUPPORTS_ML_DSA 1
#define CONFIG_USB_SISUSBVGA_MODULE 1
#define CONFIG_HD44780_MODULE 1
#define CONFIG_CPU_RMAP 1
#define CONFIG_IP6_NF_RAW_MODULE 1
#define CONFIG_VFIO_PCI_CORE_MODULE 1
#define CONFIG_ARCH_SUPPORTS_NUMA_BALANCING 1
#define CONFIG_BUFFER_HEAD 1
#define CONFIG_DVB_NGENE_MODULE 1
#define CONFIG_MWIFIEX_USB_MODULE 1
#define CONFIG_NET_9P_RDMA_MODULE 1
#define CONFIG_CC_HAS_AUTO_VAR_INIT_PATTERN 1
#define CONFIG_SND_HDA_CODEC_SENARYTECH_MODULE 1
#define CONFIG_ABP2030PA_MODULE 1
#define CONFIG_VIDEO_CX23885_MODULE 1
#define CONFIG_FORTIFY_SOURCE 1
#define CONFIG_INPUT_IQS626A_MODULE 1
#define CONFIG_HWSPINLOCK 1
#define CONFIG_I2C_DESIGNWARE_CORE 1
#define CONFIG_DRM_ACCEL 1
#define CONFIG_ENCLOSURE_SERVICES_MODULE 1
#define CONFIG_BT_HCIBTUSB_MTK 1
#define CONFIG_DMA_OF 1
#define CONFIG_DVB_SI2168_MODULE 1
#define CONFIG_FB_SYSMEM_HELPERS_DEFERRED 1
#define CONFIG_MFD_SIMPLE_MFD_I2C_MODULE 1
#define CONFIG_6LOWPAN_MODULE 1
#define CONFIG_WPCM450_SOC_MODULE 1
#define CONFIG_DUMMY_MODULE 1
#define CONFIG_VIDEO_EM28XX_V4L2_MODULE 1
#define CONFIG_DM_BIO_PRISON_MODULE 1
#define CONFIG_MMC_SDHCI_IO_ACCESSORS 1
#define CONFIG_NETFILTER_FAMILY_ARP 1
#define CONFIG_DVB_USB_CXUSB_MODULE 1
#define CONFIG_IR_RCMM_DECODER_MODULE 1
#define CONFIG_TASKS_TRACE_RCU 1
#define CONFIG_6LOWPAN_NHC_MODULE 1
#define CONFIG_MHP_MEMMAP_ON_MEMORY 1
#define CONFIG_REGULATOR_RT6245_MODULE 1
#define CONFIG_HARDENED_USERCOPY_DEFAULT_ON 1
#define CONFIG_NET_VENDOR_MELLANOX 1
#define CONFIG_IR_SERIAL_MODULE 1
#define CONFIG_OCFS2_FS_MODULE 1
#define CONFIG_SKY2_MODULE 1
#define CONFIG_USB_GSPCA_SPCA501_MODULE 1
#define CONFIG_BT_LE 1
#define CONFIG_IXGBE_MODULE 1
#define CONFIG_KVM_BOOK3S_HANDLER 1
#define CONFIG_VIDEO_VD56G3_MODULE 1
#define CONFIG_MPIC 1
#define CONFIG_SENSORS_NZXT_SMART2_MODULE 1
#define CONFIG_MT76x02_LIB_MODULE 1
#define CONFIG_BACKLIGHT_LM3509_MODULE 1
#define CONFIG_DYNAMIC_DEBUG 1
#define CONFIG_ZRAM_MODULE 1
#define CONFIG_VIDEO_OV2680_MODULE 1
#define CONFIG_SERIAL_DEV_CTRL_TTYPORT 1
#define CONFIG_PCI_DOMAINS 1
#define CONFIG_RTW89_8922AE_MODULE 1
#define CONFIG_FUSION_SAS_MODULE 1
#define CONFIG_CPU_FREQ_GOV_SCHEDUTIL 1
#define CONFIG_IMA_DEFAULT_HASH_SHA256 1
#define CONFIG_BMC150_MAGN_MODULE 1
#define CONFIG_IR_PWM_TX_MODULE 1
#define CONFIG_USB_MDC800_MODULE 1
#define CONFIG_NF_CT_NETLINK_MODULE 1
#define CONFIG_IIO_TRIGGERED_BUFFER_MODULE 1
#define CONFIG_CHECKSUM_KUNIT_MODULE 1
#define CONFIG_NETFILTER_XT_NAT_MODULE 1
#define CONFIG_SERIAL_SC16IS7XX_MODULE 1
#define CONFIG_ATH10K_LEDS 1
#define CONFIG_DM9102_MODULE 1
#define CONFIG_FWCTL_PDS_MODULE 1
#define CONFIG_6LOWPAN_NHC_FRAGMENT_MODULE 1
#define CONFIG_PROC_SYSCTL 1
#define CONFIG_MIGRATION 1
#define CONFIG_DVB_PLUTO2_MODULE 1
#define CONFIG_HAVE_EFFICIENT_UNALIGNED_ACCESS 1
#define CONFIG_DRM_CHIPONE_ICN6211_MODULE 1
#define CONFIG_ATH9K_MODULE 1
#define CONFIG_XILLYBUS_PCIE_MODULE 1
#define CONFIG_BITS_TEST_MODULE 1
#define CONFIG_CRYPTO_AKCIPHER 1
#define CONFIG_HID_APPLEIR_MODULE 1
#define CONFIG_SX9360_MODULE 1
#define CONFIG_NET_SCHED 1
#define CONFIG_VIRTIO_RTC_MODULE 1
#define CONFIG_NET_CLS_FW_MODULE 1
#define CONFIG_HAVE_HARDLOCKUP_DETECTOR_BUDDY 1
#define CONFIG_NLS_CODEPAGE_861_MODULE 1
#define CONFIG_VIDEO_HEXIUM_ORION_MODULE 1
#define CONFIG_CRYPTO_CHACHA20_MODULE 1
#define CONFIG_TXGBEVF_MODULE 1
#define CONFIG_MPLS_ROUTING_MODULE 1
#define CONFIG_PAGE_POOL 1
#define CONFIG_CRYPTO_USER_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_ESP_MODULE 1
#define CONFIG_SECURITY_SELINUX_AVC_HASH_BITS 9
#define CONFIG_RT2500USB_MODULE 1
#define CONFIG_NVME_TARGET_TCP_MODULE 1
#define CONFIG_KFENCE_SAMPLE_INTERVAL 100
#define CONFIG_HID_XINMO_MODULE 1
#define CONFIG_NTFS3_FS_MODULE 1
#define CONFIG_NETPOLL 1
#define CONFIG_SCSI_DMX3191D_MODULE 1
#define CONFIG_VIDEOBUF2_MEMOPS_MODULE 1
#define CONFIG_OVERFLOW_KUNIT_TEST_MODULE 1
#define CONFIG_NET_MPLS_GSO_MODULE 1
#define CONFIG_MII_MODULE 1
#define CONFIG_DM_VERITY_VERIFY_ROOTHASH_SIG 1
#define CONFIG_BCMA_DRIVER_GMAC_CMN 1
#define CONFIG_OF_MDIO 1
#define CONFIG_NVMEM_SYSFS 1
#define CONFIG_IMA_DEFAULT_HASH "sha256"
#define CONFIG_SENSORS_ADM1266_MODULE 1
#define CONFIG_DVB_TDA18271C2DD_MODULE 1
#define CONFIG_9P_FS_POSIX_ACL 1
#define CONFIG_USB_STORAGE_ENE_UB6250_MODULE 1
#define CONFIG_B53_SRAB_DRIVER_MODULE 1
#define CONFIG_SCSI_COMMON 1
#define CONFIG_SPARSEMEM_EXTREME 1
#define CONFIG_MLX5_EN_RXNFC 1
#define CONFIG_LIST_PRIVATE_KUNIT_TEST_MODULE 1
#define CONFIG_USB_M5602_MODULE 1
#define CONFIG_RSEQ 1
#define CONFIG_SECURITY_TOMOYO_MAX_AUDIT_LOG 1024
#define CONFIG_SND_HDA_CODEC_CS421X_MODULE 1
#define CONFIG_JOYSTICK_GRIP_MODULE 1
#define CONFIG_NET_SCH_DRR_MODULE 1
#define CONFIG_PM_SLEEP_SMP_NONZERO_CPU 1
#define CONFIG_SENSORS_JC42_MODULE 1
#define CONFIG_UUID_KUNIT_TEST_MODULE 1
#define CONFIG_TOUCHSCREEN_ILITEK_MODULE 1
#define CONFIG_SYSCTL_EXCEPTION_TRACE 1
#define CONFIG_NEW_LEDS 1
#define CONFIG_UCLAMP_TASK 1
#define CONFIG_MHI_BUS_PCI_GENERIC_MODULE 1
#define CONFIG_CRYPTO_DEV_QAT_420XX_MODULE 1
#define CONFIG_RAS 1
#define CONFIG_NF_TABLES_NETDEV 1
#define CONFIG_SSB_SPROM 1
#define CONFIG_SND_USB_6FIRE_MODULE 1
#define CONFIG_HISI_HIKEY_USB_MODULE 1
#define CONFIG_CRYPTO_LIB_CURVE25519_ARCH 1
#define CONFIG_VIDEO_ADP1653_MODULE 1
#define CONFIG_BT_6LOWPAN_MODULE 1
#define CONFIG_PPC_SPLPAR 1
#define CONFIG_DVB_STV0367_MODULE 1
#define CONFIG_SERIAL_8250_PERICOM 1
#define CONFIG_PWM_DWC_MODULE 1
#define CONFIG_SCSI_IPR_MODULE 1
#define CONFIG_ARCH_WANT_IRQS_OFF_ACTIVATE_MM 1
#define CONFIG_DWMAC_MOTORCOMM_MODULE 1
#define CONFIG_SCSI_CXGB3_ISCSI_MODULE 1
#define CONFIG_PADATA 1
#define CONFIG_VIDEO_FB_IVTV_MODULE 1
#define CONFIG_HID_PXRC_MODULE 1
#define CONFIG_HD44780_COMMON_MODULE 1
#define CONFIG_ARCH_HAS_PTE_SPECIAL 1
#define CONFIG_BLK_DEV_RBD_MODULE 1
#define CONFIG_SND_DARLA20_MODULE 1
#define CONFIG_VIDEO_OV5648_MODULE 1
#define CONFIG_MEDIA_TUNER_MT2060_MODULE 1
#define CONFIG_CC_HAS_ASM_INLINE 1
#define CONFIG_VIDEO_DS90UB953_MODULE 1
#define CONFIG_HANGCHECK_TIMER_MODULE 1
#define CONFIG_SATA_AHCI_PLATFORM_MODULE 1
#define CONFIG_SENSORS_W83781D_MODULE 1
#define CONFIG_FB 1
#define CONFIG_REGULATOR_RT5190A_MODULE 1
#define CONFIG_IP_VS_FTP_MODULE 1
#define CONFIG_BT_HCIBCM4377_MODULE 1
#define CONFIG_ADXL313_MODULE 1
#define CONFIG_PATA_NS87410_MODULE 1
#define CONFIG_HID_STEELSERIES_MODULE 1
#define CONFIG_PPC_ICS_RTAS 1
#define CONFIG_USB_LEDS_TRIGGER_USBPORT_MODULE 1
#define CONFIG_SENSORS_TMP513_MODULE 1
#define CONFIG_VIDEO_CS5345_MODULE 1
#define CONFIG_F2FS_FS_LZ4 1
#define CONFIG_HAS_SECURITY_AUDIT 1
#define CONFIG_VIRT_WIFI_MODULE 1
#define CONFIG_V4L_TEST_DRIVERS 1
#define CONFIG_PDS_VDPA_MODULE 1
#define CONFIG_VIDEO_CX88_ALSA_MODULE 1
#define CONFIG_ATM_DRIVERS 1
#define CONFIG_NLS_CODEPAGE_950_MODULE 1
#define CONFIG_IR_RC5_DECODER_MODULE 1
#define CONFIG_PPC_PKEY 1
#define CONFIG_SENSORS_LM78_MODULE 1
#define CONFIG_RTW89_8852AE_MODULE 1
#define CONFIG_BT_HCIUART_INTEL 1
#define CONFIG_KPROBE_EVENTS 1
#define CONFIG_MLX4_DEBUG 1
#define CONFIG_PROC_VMCORE_DEVICE_DUMP 1
#define CONFIG_F2FS_FS_POSIX_ACL 1
#define CONFIG_CRYPTO_DEV_QAT_C3XXX_MODULE 1
#define CONFIG_CHARGER_BQ256XX_MODULE 1
#define CONFIG_TI_FPC202_MODULE 1
#define CONFIG_ATH10K_AHB 1
#define CONFIG_CC_HAS_ASM_GOTO_TIED_OUTPUT 1
#define CONFIG_ISCSI_TARGET_MODULE 1
#define CONFIG_RV_PER_TASK_MONITORS 3
#define CONFIG_TABLET_USB_KBTAB_MODULE 1
#define CONFIG_BLK_DEV_INITRD 1
#define CONFIG_CIFS_FSCACHE 1
#define CONFIG_SCSI_MPI3MR_MODULE 1
#define CONFIG_VIDEO_OV7251_MODULE 1
#define CONFIG_MEDIA_TUNER_TDA8290_MODULE 1
#define CONFIG_BE2ISCSI_MODULE 1
#define CONFIG_DWC_PCIE_PMU_MODULE 1
#define CONFIG_AFFS_FS_MODULE 1
#define CONFIG_NEED_TASKS_RCU 1
#define CONFIG_USB_SERIAL_OPTICON_MODULE 1
#define CONFIG_USB_GSPCA_SPCA500_MODULE 1
#define CONFIG_B43_PHY_LP 1
#define CONFIG_INET 1
#define CONFIG_MAX30208_MODULE 1
#define CONFIG_DL2K_MODULE 1
#define CONFIG_VIDEO_CX18_MODULE 1
#define CONFIG_PATA_HPT3X2N_MODULE 1
#define CONFIG_HID_PETALYNX_MODULE 1
#define CONFIG_BLK_DEV_SR 1
#define CONFIG_MD_AUTODETECT 1
#define CONFIG_PPC_FTRACE_OUT_OF_LINE 1
#define CONFIG_CHR_DEV_ST_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_REDIRECT_MODULE 1
#define CONFIG_POWER_RESET 1
#define CONFIG_ATA 1
#define CONFIG_ADAPTEC_STARFIRE_MODULE 1
#define CONFIG_PPC_ICP_NATIVE 1
#define CONFIG_BLK_DEV_RAM_SIZE 16384
#define CONFIG_USB_EMI26_MODULE 1
#define CONFIG_NGBEVF_MODULE 1
#define CONFIG_SENSORS_ZL6100_MODULE 1
#define CONFIG_ATH6KL_USB_MODULE 1
#define CONFIG_TYPEC_MUX_FSA4480_MODULE 1
#define CONFIG_RPS 1
#define CONFIG_PCI_P2PDMA 1
#define CONFIG_HID_PICOLCD_LCD 1
#define CONFIG_TOUCHSCREEN_ELAN_MODULE 1
#define CONFIG_CRYPTO_CAST6_MODULE 1
#define CONFIG_NF_CONNTRACK_SECMARK 1
#define CONFIG_LEDS_TRIGGER_NETDEV_MODULE 1
#define CONFIG_CGROUP_NET_PRIO 1
#define CONFIG_MPILIB 1
#define CONFIG_MT7603E_MODULE 1
#define CONFIG_MEDIA_TUNER_MC44S803_MODULE 1
#define CONFIG_GARP_MODULE 1
#define CONFIG_RTC_DRV_RV3028_MODULE 1
#define CONFIG_GPIB_CB7210_MODULE 1
#define CONFIG_FIRMWARE_TABLE 1
#define CONFIG_NET_ACT_SIMP_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_CHECKSUM_MODULE 1
#define CONFIG_VIDEO_IMX335_MODULE 1
#define CONFIG_RTC_DRV_SD3078_MODULE 1
#define CONFIG_IP_NF_SECURITY_MODULE 1
#define CONFIG_EPOLL 1
#define CONFIG_ARCH_SUPPORTS_SCHED_MC 1
#define CONFIG_LEDS_MLXREG_MODULE 1
#define CONFIG_TYPEC_MUX_GPIO_SBU_MODULE 1
#define CONFIG_RTC_DRV_DS3232_MODULE 1
#define CONFIG_FONT_SUN8x16 1
#define CONFIG_I3C_MODULE 1
#define CONFIG_CGROUPS 1
#define CONFIG_VIDEO_SAA7146_VV_MODULE 1
#define CONFIG_UDMABUF 1
#define CONFIG_CNIC_MODULE 1
#define CONFIG_NFSD_FLEXFILELAYOUT 1
#define CONFIG_PATA_JMICRON_MODULE 1
#define CONFIG_SENSORS_PLI1209BC_REGULATOR 1
#define CONFIG_NET_CLS_MATCHALL_MODULE 1
#define CONFIG_XILINX_LL_TEMAC_MODULE 1
#define CONFIG_PKCS8_PRIVATE_KEY_PARSER_MODULE 1
#define CONFIG_SERIAL_8250 1
#define CONFIG_ABP060MG_MODULE 1
#define CONFIG_NET_DSA_MT7530_MMIO_MODULE 1
#define CONFIG_HAVE_ARCH_KASAN 1
#define CONFIG_USB_SERIAL 1
#define CONFIG_CRYPTO_CHACHA20POLY1305_MODULE 1
#define CONFIG_USE_PERCPU_NUMA_NODE_ID 1
#define CONFIG_NUMA_KEEP_MEMINFO 1
#define CONFIG_HID_ELAN_MODULE 1
#define CONFIG_CRYPTO_HASH_INFO 1
#define CONFIG_DRM_AMD_ACP 1
#define CONFIG_QED_LL2 1
#define CONFIG_TARGET_CPU_BOOL 1
#define CONFIG_NETFILTER_XT_MATCH_IPVS_MODULE 1
#define CONFIG_CRC32 1
#define CONFIG_CRYPTO_DEV_QAT_DH895xCC_MODULE 1
#define CONFIG_EXT4_FS 1
#define CONFIG_SENSORS_INA238_MODULE 1
#define CONFIG_HID_EZKEY_MODULE 1
#define CONFIG_DRM_PANEL_NOVATEK_NT35950_MODULE 1
#define CONFIG_FANOTIFY_ACCESS_PERMISSIONS 1
#define CONFIG_IR_SERIAL_TRANSMITTER 1
#define CONFIG_FUEL_GAUGE_STC3117_MODULE 1
#define CONFIG_EVM_ATTR_FSUUID 1
#define CONFIG_TOUCHSCREEN_HIMAX_HX83112B_MODULE 1
#define CONFIG_TCP_CONG_HSTCP_MODULE 1
#define CONFIG_SND_HDA_INPUT_BEEP_MODE 0
#define CONFIG_GPIO_CDEV 1
#define CONFIG_CAN_F81604_MODULE 1
#define CONFIG_VIDEO_PVRUSB2_DVB 1
#define CONFIG_NETFILTER_XT_TARGET_NOTRACK_MODULE 1
#define CONFIG_ARCH_HAS_UACCESS_FLUSHCACHE 1
#define CONFIG_USB_SERIAL_XSENS_MT_MODULE 1
#define CONFIG_ATH9K_PCOEM 1
#define CONFIG_MLX5_CORE_IPOIB 1
#define CONFIG_LIBIE_MODULE 1
#define CONFIG_VIDEO_IMX290_MODULE 1
#define CONFIG_BSD_PROCESS_ACCT_V3 1
#define CONFIG_POWER_SEQUENCING_PCIE_M2_MODULE 1
#define CONFIG_FUSE_PASSTHROUGH 1
#define CONFIG_RTL8192C_COMMON_MODULE 1
#define CONFIG_IMA_WRITE_POLICY 1
#define CONFIG_HAVE_SETUP_PER_CPU_AREA 1
#define CONFIG_USBIP_VHCI_HC_PORTS 8
#define CONFIG_PCIE_PTM 1
#define CONFIG_PATA_NETCELL_MODULE 1
#define CONFIG_USB_SERIAL_SIMPLE_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_REALM_MODULE 1
#define CONFIG_CIFS_UPCALL 1
#define CONFIG_CRYPTO_DEV_QAT_C3XXXVF_MODULE 1
#define CONFIG_USB_SL811_HCD_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_RATEEST_MODULE 1
#define CONFIG_SND_MIXART_MODULE 1
#define CONFIG_SATA_MOBILE_LPM_POLICY 3
#define CONFIG_NET_CRC32C 1
#define CONFIG_DMARD10_MODULE 1
#define CONFIG_SND_HDA_GENERIC_MODULE 1
#define CONFIG_PRINT_STACK_DEPTH 64
#define CONFIG_WL18XX_MODULE 1
#define CONFIG_IIO_ADC_HELPER_MODULE 1
#define CONFIG_RV_MON_EVENTS 1
#define CONFIG_KUNIT_ALL_TESTS_MODULE 1
#define CONFIG_DECOMPRESS_BZIP2 1
#define CONFIG_RDMA_RXE_MODULE 1
#define CONFIG_MODULE_SIG 1
#define CONFIG_HAVE_DEBUG_STACKOVERFLOW 1
#define CONFIG_MODULE_SIG_FORMAT 1
#define CONFIG_BLK_DEV_RNBD_SERVER_MODULE 1
#define CONFIG_GPIO_LINE_MUX_MODULE 1
#define CONFIG_BT_HCIUART_H4 1
#define CONFIG_VIDEO_OV6211_MODULE 1
#define CONFIG_SENSORS_OCC_MODULE 1
#define CONFIG_DMABUF_HEAPS_SYSTEM 1
#define CONFIG_MIN_HEAP 1
#define CONFIG_SSB_DRIVER_GPIO 1
#define CONFIG_RTC_LIB 1
#define CONFIG_HAVE_MOVE_PUD 1
#define CONFIG_SENSORS_LM77_MODULE 1
#define CONFIG_QED_SRIOV 1
#define CONFIG_PWRSEQ_SD8787_MODULE 1
#define CONFIG_INPUT_UINPUT_MODULE 1
#define CONFIG_INPUT_GPIO_ROTARY_ENCODER_MODULE 1
#define CONFIG_SCSI_DEBUG_MODULE 1
#define CONFIG_SENSORS_TMP108_MODULE 1
#define CONFIG_PER_VMA_LOCK 1
#define CONFIG_ND_PFN_MODULE 1
#define CONFIG_NFT_REDIR_MODULE 1
#define CONFIG_USB_GSPCA_SONIXB_MODULE 1
#define CONFIG_DQL 1
#define CONFIG_RSI_DEBUGFS 1
#define CONFIG_IR_TOY_MODULE 1
#define CONFIG_TCG_TIS_I2C_MODULE 1
#define CONFIG_SECCOMP_FILTER 1
#define CONFIG_GPIO_PCI_IDIO_16_MODULE 1
#define CONFIG_MEGARAID_NEWGEN 1
#define CONFIG_TASKS_RCU 1
#define CONFIG_BNXT_SRIOV 1
#define CONFIG_GCC_VERSION 150201
#define CONFIG_6LOWPAN_NHC_IPV6_MODULE 1
#define CONFIG_HAVE_ARCH_JUMP_LABEL_RELATIVE 1
#define CONFIG_DRM_PANEL_SAMSUNG_S6E3HA8_MODULE 1
#define CONFIG_TULIP_MMIO 1
#define CONFIG_POWER_RESET_GPIO 1
#define CONFIG_SYSCTL 1
#define CONFIG_NVMEM 1
#define CONFIG_CRYPTO_LIB_SHA1 1
#define CONFIG_ZONE_DEVICE 1
#define CONFIG_SND_HDA_CODEC_HDMI_SIMPLE_MODULE 1
#define CONFIG_VIDEO_EM28XX_ALSA_MODULE 1
#define CONFIG_VLAN_8021Q_GVRP 1
#define CONFIG_HAVE_VIRT_CPU_ACCOUNTING 1
#define CONFIG_VIDEO_OS05B10_MODULE 1
#define CONFIG_IP_NF_IPTABLES_LEGACY_MODULE 1
#define CONFIG_HID_DRAGONRISE_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_STRING_MODULE 1
#define CONFIG_SND_AU8830_MODULE 1
#define CONFIG_SSB_POSSIBLE 1
#define CONFIG_VIDEO_IMX334_MODULE 1
#define CONFIG_VIDEO_IMX274_MODULE 1
#define CONFIG_SENSORS_PT5161L_MODULE 1
#define CONFIG_NET_VENDOR_XILINX 1
#define CONFIG_TIGON3_MODULE 1
#define CONFIG_MAXLINEAR_GPHY_MODULE 1
#define CONFIG_NLS_UTF8_MODULE 1
#define CONFIG_VIDEO_OV02C10_MODULE 1
#define CONFIG_SERIAL_8250_FSL 1
#define CONFIG_WLAN_VENDOR_REALTEK 1
#define CONFIG_B43_PCI_AUTOSELECT 1
#define CONFIG_USB_GSPCA_STK1135_MODULE 1
#define CONFIG_NONSTATIC_KERNEL 1
#define CONFIG_GPIO_GENERIC_MODULE 1
#define CONFIG_LPARCFG 1
#define CONFIG_SMI330_I2C_MODULE 1
#define CONFIG_BACKLIGHT_LP855X_MODULE 1
#define CONFIG_PM_SLEEP_SMP 1
#define CONFIG_ARCH_MAY_HAVE_PC_FDC 1
#define CONFIG_POWER_RESET_GPIO_RESTART 1
#define CONFIG_FIXED_PHY 1
#define CONFIG_DRM_PANEL_SAMSUNG_S6E8AA5X01_AMS561RA01_MODULE 1
#define CONFIG_CRYPTO_NULL 1
#define CONFIG_POSIX_MQUEUE_SYSCTL 1
#define CONFIG_MULTIPLEXER_MODULE 1
#define CONFIG_IP_VS_DH_MODULE 1
#define CONFIG_INPUT_QNAP_MCU_MODULE 1
#define CONFIG_HAPPYMEAL_MODULE 1
#define CONFIG_VIDEO_TW686X_MODULE 1
#define CONFIG_RTW89_8851BE_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_OSF_MODULE 1
#define CONFIG_SND_HDA_CODEC_CS8409_MODULE 1
#define CONFIG_IPMI_SI_MODULE 1
#define CONFIG_IWL4965_MODULE 1
#define CONFIG_HID_FT260_MODULE 1
#define CONFIG_SERIAL_CORE_CONSOLE 1
#define CONFIG_NET_VENDOR_CISCO 1
#define CONFIG_VIDEO_OV8858_MODULE 1
#define CONFIG_XXHASH 1
#define CONFIG_NET_L3_MASTER_DEV 1
#define CONFIG_TASK_XACCT 1
#define CONFIG_NET_SOCK_MSG 1
#define CONFIG_PAGE_OWNER 1
#define CONFIG_SENSORS_ADM9240_MODULE 1
#define CONFIG_CRYPTO_LIB_DES_MODULE 1
#define CONFIG_INPUT_MAX77693_HAPTIC_MODULE 1
#define CONFIG_EVM 1
#define CONFIG_CRYPTO_ECDH 1
#define CONFIG_ATA_PIIX 1
#define CONFIG_RTC_DRV_DS2404_MODULE 1
#define CONFIG_RTC_DRV_MAX31335_MODULE 1
#define CONFIG_OCXL_MODULE 1
#define CONFIG_DRM_NOUVEAU_BACKLIGHT 1
#define CONFIG_DVB_DYNAMIC_MINORS 1
#define CONFIG_HID_PICOLCD_MODULE 1
#define CONFIG_IRQ_DOMAIN_HIERARCHY 1
#define CONFIG_SND_JACK 1
#define CONFIG_IWLDVM_MODULE 1
#define CONFIG_NET_ACT_BPF_MODULE 1
#define CONFIG_CRC_KUNIT_TEST_MODULE 1
#define CONFIG_HID_LETSKETCH_MODULE 1
#define CONFIG_CAN_CTUCANFD_MODULE 1
#define CONFIG_DVB_USB_DIB0700_MODULE 1
#define CONFIG_SENSORS_TMP401_MODULE 1
#define CONFIG_CONTEXT_TRACKING_USER 1
#define CONFIG_CRYPTO_LIB_POLY1305 1
#define CONFIG_MTD_NAND_CORE_MODULE 1
#define CONFIG_NET_DSA_XRS700X_I2C_MODULE 1
#define CONFIG_CRYPTO_ECRDSA_MODULE 1
#define CONFIG_TCP_CONG_VENO_MODULE 1
#define CONFIG_9P_FS_MODULE 1
#define CONFIG_USB_GSPCA_MODULE 1
#define CONFIG_SQUASHFS_LZO 1
#define CONFIG_FRAMEBUFFER_CONSOLE_DETECT_PRIMARY 1
#define CONFIG_NET_SCH_TAPRIO_MODULE 1
#define CONFIG_SENSORS_W83795_MODULE 1
#define CONFIG_SENSORS_LTC3815_MODULE 1
#define CONFIG_CRYPTO_KRB5ENC_MODULE 1
#define CONFIG_FRAME_WARN 2048
#define CONFIG_DYNAMIC_FTRACE 1
#define CONFIG_NLS_MAC_CENTEURO_MODULE 1
#define CONFIG_VIDEO_DW9768_MODULE 1
#define CONFIG_LD_VERSION 24600
#define CONFIG_NET_CLS_U32_MODULE 1
#define CONFIG_SENSORS_MP2869_MODULE 1
#define CONFIG_VT_CONSOLE 1
#define CONFIG_INET_AH_MODULE 1
#define CONFIG_I2C 1
#define CONFIG_HOTPLUG_PCI_SHPC 1
#define CONFIG_NET_SCH_ETS_MODULE 1
#define CONFIG_CC_HAS_UBSAN_BOUNDS_STRICT 1
#define CONFIG_BATTERY_RT5033_MODULE 1
#define CONFIG_VIDEO_V4L2_TPG_MODULE 1
#define CONFIG_ADXL313_I2C_MODULE 1
#define CONFIG_SCSI_MVUMI_MODULE 1
#define CONFIG_MD_RAID10_MODULE 1
#define CONFIG_LEDS_CLASS 1
#define CONFIG_MEDIA_RADIO_SUPPORT 1
#define CONFIG_LSI_ET1011C_PHY_MODULE 1
#define CONFIG_CRYPTO_LIB_CURVE25519_KUNIT_TEST_MODULE 1
#define CONFIG_LEDS_TRIGGER_PANIC 1
#define CONFIG_BT_HCIUART_AG6XX 1
#define CONFIG_RESOURCE_KUNIT_TEST_MODULE 1
#define CONFIG_PM_SLEEP 1
#define CONFIG_VIDEO_IMX415_MODULE 1
#define CONFIG_NET_SCH_CODEL_MODULE 1
#define CONFIG_MODULE_SIG_ALL 1
#define CONFIG_SENSORS_ADM1029_MODULE 1
#define CONFIG_VIDEO_DW9807_VCM_MODULE 1
#define CONFIG_DRM_BRIDGE 1
#define CONFIG_REED_SOLOMON_DEC8 1
#define CONFIG_MKISS_MODULE 1
#define CONFIG_VIDEO_SAA7134_RC 1
#define CONFIG_USB_GSPCA_SPCA506_MODULE 1
#define CONFIG_SENSORS_ADT7410_MODULE 1
#define CONFIG_IWLWIFI_DEBUG 1
#define CONFIG_HID_MAGICMOUSE 1
#define CONFIG_NET_SCH_FQ_MODULE 1
#define CONFIG_VIDEO_BT848_MODULE 1
#define CONFIG_RT2800USB_RT53XX 1
#define CONFIG_VIDEO_M52790_MODULE 1
#define CONFIG_SYSTEM_DATA_VERIFICATION 1
#define CONFIG_LOCK_TORTURE_TEST_MODULE 1
#define CONFIG_HID_ZEROPLUS_MODULE 1
#define CONFIG_SND_HDA_DSP_LOADER 1
#define CONFIG_NET_PTP_CLASSIFY 1
#define CONFIG_ICE_MODULE 1
#define CONFIG_PM 1
#define CONFIG_CRYPTO_LIB_SHA1_KUNIT_TEST_MODULE 1
#define CONFIG_DST_CACHE 1
#define CONFIG_CRYPTO_DEV_VIRTIO_MODULE 1
#define CONFIG_MEMORY_FAILURE 1
#define CONFIG_CRYPTO_SHA512 1
#define CONFIG_ADXL367_I2C_MODULE 1
#define CONFIG_IRQ_POLL 1
#define CONFIG_LEDS_LP8864_MODULE 1
#define CONFIG_NET_VENDOR_GOOGLE 1
#define CONFIG_ARCH_SPARSEMEM_ENABLE 1
#define CONFIG_SCSI_AIC7XXX_MODULE 1
#define CONFIG_ARCH_ENABLE_THP_MIGRATION 1
#define CONFIG_USB_EHCI_ROOT_HUB_TT 1
#define CONFIG_USB_NET_AX88179_178A_MODULE 1
#define CONFIG_NLS_CODEPAGE_775_MODULE 1
#define CONFIG_CMA_SYSFS 1
#define CONFIG_MARVELL_PHY_MODULE 1
#define CONFIG_HID_PRIMAX_MODULE 1
#define CONFIG_NLS 1
#define CONFIG_USB_S2255_MODULE 1
#define CONFIG_IP_SET_HASH_NETPORTNET_MODULE 1
#define CONFIG_OID_REGISTRY 1
#define CONFIG_IIO_KX022A_MODULE 1
#define CONFIG_MEDIATEK_GE_PHY_MODULE 1
#define CONFIG_SND_CTXFI_MODULE 1
#define CONFIG_BT_NXPUART_MODULE 1
#define CONFIG_SENSORS_MAX77705_MODULE 1
#define CONFIG_SCHED_TRACER 1
#define CONFIG_B43_PHY_G 1
#define CONFIG_INPUT_CMA3000_I2C_MODULE 1
#define CONFIG_BNXT_FLOWER_OFFLOAD 1
#define CONFIG_TOUCHSCREEN_WACOM_I2C_MODULE 1
#define CONFIG_VIDEO_OG01A1B_MODULE 1
#define CONFIG_GENERIC_IRQ_MIGRATION 1
#define CONFIG_SQUASHFS_ZLIB 1
#define CONFIG_MLX4_CORE_MODULE 1
#define CONFIG_MLX5_DPLL_MODULE 1
#define CONFIG_NET_VENDOR_WIZNET 1
#define CONFIG_CAN_CAN327_MODULE 1
#define CONFIG_SPEAKUP_SYNTH_APOLLO_MODULE 1
#define CONFIG_SPEAKUP_SYNTH_DECTLK_MODULE 1
#define CONFIG_XFRM_IPTFS_MODULE 1
#define CONFIG_SENSORS_LM87_MODULE 1
#define CONFIG_BACKLIGHT_PWM_MODULE 1
#define CONFIG_RPCSEC_GSS_KRB5_ENCTYPES_CAMELLIA 1
#define CONFIG_USB_VIDEO_CLASS_INPUT_EVDEV 1
#define CONFIG_DVB_STV0297_MODULE 1
#define CONFIG_IPMI_IPMB_MODULE 1
#define CONFIG_NET_DSA_TAG_BRCM_PREPEND_MODULE 1
#define CONFIG_HVCS_MODULE 1
#define CONFIG_SENSORS_LTC4215_MODULE 1
#define CONFIG_NETFILTER_FAMILY_BRIDGE 1
#define CONFIG_CC_NO_ARRAY_BOUNDS 1
#define CONFIG_6LOWPAN_GHC_EXT_HDR_ROUTE_MODULE 1
#define CONFIG_MLX90632_MODULE 1
#define CONFIG_FIB_RULES 1
#define CONFIG_NFT_XFRM_MODULE 1
#define CONFIG_CONTEXT_TRACKING_IDLE 1
#define CONFIG_BLK_DEBUG_FS 1
#define CONFIG_JBD2 1
#define CONFIG_VIDEO_IMX208_MODULE 1
#define CONFIG_FONT_SUPPORT 1
#define CONFIG_KUNIT_TEST_MODULE 1
#define CONFIG_NXP_CBTX_PHY_MODULE 1
#define CONFIG_SENSORS_LTC4222_MODULE 1
#define CONFIG_WIRELESS 1
#define CONFIG_THERMAL_GOV_FAIR_SHARE 1
#define CONFIG_BLK_DEV_BSGLIB 1
#define CONFIG_9P_FS_SECURITY 1
#define CONFIG_VIDEO_ADV7170_MODULE 1
#define CONFIG_NET_SCH_FQ_PIE_MODULE 1
#define CONFIG_MLXSW_SPECTRUM_MODULE 1
#define CONFIG_SATA_SIL24_MODULE 1
#define CONFIG_MAX1363_MODULE 1
#define CONFIG_SCSI_DH_ALUA_MODULE 1
#define CONFIG_VIDEO_IMX283_MODULE 1
#define CONFIG_SENSORS_MCP3021_MODULE 1
#define CONFIG_SND_CS46XX_MODULE 1
#define CONFIG_NETFILTER 1
#define CONFIG_LIBERTAS_THINFIRM_USB_MODULE 1
#define CONFIG_MEDIA_TUNER_MT2063_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_HL_MODULE 1
#define CONFIG_DVB_STV6111_MODULE 1
#define CONFIG_NVME_RDMA_MODULE 1
#define CONFIG_NET_ACT_GATE_MODULE 1
#define CONFIG_CXL_PCI_MODULE 1
#define CONFIG_SND_VIRMIDI_MODULE 1
#define CONFIG_USB_GSPCA_SN9C2028_MODULE 1
#define CONFIG_NET_IPGRE_MODULE 1
#define CONFIG_PPC_WATCHDOG 1
#define CONFIG_INFINIBAND_IRDMA_MODULE 1
#define CONFIG_DYNAMIC_FTRACE_WITH_CALL_OPS 1
#define CONFIG_SND_USB_AUDIO_USE_MEDIA_CONTROLLER 1
#define CONFIG_BSD_DISKLABEL 1
#define CONFIG_USB_STORAGE_SDDR09_MODULE 1
#define CONFIG_VIDEO_OV9650_MODULE 1
#define CONFIG_BLK_WBT_MQ 1
#define CONFIG_DVB_CXD2841ER_MODULE 1
#define CONFIG_FUNCTION_GRAPH_RETVAL 1
#define CONFIG_CRYPTO_LIB_MD5 1
#define CONFIG_USB_SPEEDTOUCH_MODULE 1
#define CONFIG_KVM_BOOK3S_HV_POSSIBLE 1
#define CONFIG_DRM_DISPLAY_DP_HELPER 1
#define CONFIG_VIDEO_TVP5150_MODULE 1
#define CONFIG_HFSPLUS_FS_MODULE 1
#define CONFIG_CRYPTO_SKCIPHER 1
#define CONFIG_MLX5_EN_TLS 1
#define CONFIG_SENSORS_LM75_MODULE 1
#define CONFIG_USB_SERIAL_MOS7840_MODULE 1
#define CONFIG_BITOPS_KUNIT_MODULE 1
#define CONFIG_NETDEV_ADDR_LIST_TEST_MODULE 1
#define CONFIG_IR_SONY_DECODER_MODULE 1
#define CONFIG_SERIO_ARC_PS2_MODULE 1
#define CONFIG_HID_ITE_MODULE 1
#define CONFIG_HAVE_FUNCTION_GRAPH_FREGS 1
#define CONFIG_VIDEO_IMX355_MODULE 1
#define CONFIG_IBMVNIC_MODULE 1
#define CONFIG_PCP_BATCH_SCALE_MAX 5
#define CONFIG_VIDEO_ST_MIPID02_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_NFACCT_MODULE 1
#define CONFIG_VIDEO_IMX412_MODULE 1
#define CONFIG_SLAB_FREELIST_HARDENED 1
#define CONFIG_PM_GENERIC_DOMAINS 1
#define CONFIG_CRYPTO_LRW_MODULE 1
#define CONFIG_XFS_ONLINE_SCRUB 1
#define CONFIG_USB_GSPCA_SPCA505_MODULE 1
#define CONFIG_TAP_MODULE 1
#define CONFIG_HAVE_PREEMPT_DYNAMIC 1
#define CONFIG_USB_DEFAULT_PERSIST 1
#define CONFIG_SENSORS_ADT7411_MODULE 1
#define CONFIG_TCP_CONG_CUBIC 1
#define CONFIG_HID_THINGM_MODULE 1
#define CONFIG_ATM_BR2684_MODULE 1
#define CONFIG_XDP_SOCKETS 1
#define CONFIG_CRYPTO_ECHAINIV_MODULE 1
#define CONFIG_MEGARAID_MAILBOX_MODULE 1
#define CONFIG_RTW89_8922A_MODULE 1
#define CONFIG_ARCH_HAS_UBSAN 1
#define CONFIG_HID_PICOLCD_LEDS 1
#define CONFIG_DVB_DIB7000P_MODULE 1
#define CONFIG_CW1200_MODULE 1
#define CONFIG_JFFS2_SUMMARY 1
#define CONFIG_VIDEO_CCS_PLL_MODULE 1
#define CONFIG_IP_SET_BITMAP_IP_MODULE 1
#define CONFIG_SENSORS_TPS25990_MODULE 1
#define CONFIG_ARCH_WANT_DEFAULT_TOPDOWN_MMAP_LAYOUT 1
#define CONFIG_HAVE_SOFTIRQ_ON_OWN_STACK 1
#define CONFIG_VIRT_CPU_ACCOUNTING_GEN 1
#define CONFIG_BPF_JIT 1
#define CONFIG_CRYPTO_LIB_SHA3 1
#define CONFIG_ARCH_HAS_MEM_ENCRYPT 1
#define CONFIG_SUNRPC_DEBUG 1
#define CONFIG_HID_MALTRON_MODULE 1
#define CONFIG_CRYPTO_CAST5_MODULE 1
#define CONFIG_GPIB_AGILENT_82357A_MODULE 1
#define CONFIG_PM_TEST_SUSPEND 1
#define CONFIG_NET_TEAM_MODE_ACTIVEBACKUP_MODULE 1
#define CONFIG_MT7921E_MODULE 1
#define CONFIG_DVB_AS102_MODULE 1
#define CONFIG_HID_PANTHERLORD_MODULE 1
#define CONFIG_BATMAN_ADV_TRACING 1
#define CONFIG_HAVE_CLK_PREPARE 1
#define CONFIG_BOOTX_TEXT 1
#define CONFIG_SERIAL_NONSTANDARD 1
#define CONFIG_UNIXWARE_DISKLABEL 1
#define CONFIG_PTP_1588_CLOCK_MOCK_MODULE 1
#define CONFIG_BLK_DEV_BSG_COMMON 1
#define CONFIG_CFG80211_CRDA_SUPPORT 1
#define CONFIG_NVRAM 1
#define CONFIG_IP_ADVANCED_ROUTER 1
#define CONFIG_HMM_MIRROR 1
#define CONFIG_JOYSTICK_IFORCE_MODULE 1
#define CONFIG_IP_VS_OVF_MODULE 1
#define CONFIG_CRYPTO_GENIV 1
#define CONFIG_TRACER_SNAPSHOT 1
#define CONFIG_VIDEO_EM28XX_RC_MODULE 1
#define CONFIG_MMU_GATHER_MERGE_VMAS 1
#define CONFIG_CRC4_MODULE 1
#define CONFIG_USB_ONBOARD_DEV_MODULE 1
#define CONFIG_AUDIT_ARCH 1
#define CONFIG_FTRACE 1
#define CONFIG_BAYCOM_SER_HDX_MODULE 1
#define CONFIG_RTL8192EE_MODULE 1
#define CONFIG_DNS_RESOLVER_MODULE 1
#define CONFIG_SUNRPC_GSS_MODULE 1
#define CONFIG_PAGE_BLOCK_MAX_ORDER 8
#define CONFIG_VIDEOMODE_HELPERS 1
#define CONFIG_SQUASHFS_LZ4 1
#define CONFIG_EDAC_SUPPORT 1
#define CONFIG_USB_PRINTER_MODULE 1
#define CONFIG_AFS_DEBUG 1
#define CONFIG_B44_PCICORE_AUTOSELECT 1
#define CONFIG_NETFILTER_XT_TARGET_NETMAP_MODULE 1
#define CONFIG_VDPA_SIM_NET_MODULE 1
#define CONFIG_VIDEO_SAA7110_MODULE 1
#define CONFIG_F2FS_FS_LZ4HC 1
#define CONFIG_USB_GSPCA_SE401_MODULE 1
#define CONFIG_STRIP_ASM_SYMS 1
#define CONFIG_SENSORS_SIS5595_MODULE 1
#define CONFIG_PWM_GPIO_MODULE 1
#define CONFIG_MDIO_MVUSB_MODULE 1
#define CONFIG_EFI_PARTITION 1
#define CONFIG_SENSORS_MAX6620_MODULE 1
#define CONFIG_SND_AC97_POWER_SAVE 1
#define CONFIG_PCIE_BUS_DEFAULT 1
#define CONFIG_SENSORS_LTC2990_MODULE 1
#define CONFIG_MDIO_GPIO_MODULE 1
#define CONFIG_MARVELL_10G_PHY_MODULE 1
#define CONFIG_ST_UVIS25_MODULE 1
#define CONFIG_MACB_PCI_MODULE 1
#define CONFIG_VIDEO_SAA7127_MODULE 1
#define CONFIG_MT792x_USB_MODULE 1
#define CONFIG_IP_NF_MATCH_TTL_MODULE 1
#define CONFIG_MMC_TIFM_SD_MODULE 1
#define CONFIG_MMIOWB 1
#define CONFIG_FHANDLE 1
#define CONFIG_SND_USB_HIFACE_MODULE 1
#define CONFIG_XFS_SUPPORT_ASCII_CI 1
#define CONFIG_NETFILTER_XT_MATCH_CGROUP_MODULE 1
#define CONFIG_USB_SERIAL_SSU100_MODULE 1
#define CONFIG_SENSORS_G762_MODULE 1
#define CONFIG_ARCH_MHP_MEMMAP_ON_MEMORY_ENABLE 1
#define CONFIG_FPGA_DFL_FME_BRIDGE_MODULE 1
#define CONFIG_USB_SERIAL_F81232_MODULE 1
#define CONFIG_NVME_TARGET_FCLOOP_MODULE 1
#define CONFIG_USB_GSPCA_SPCA561_MODULE 1
#define CONFIG_SND_DARLA24_MODULE 1
#define CONFIG_MAC80211_KUNIT_TEST_MODULE 1
#define CONFIG_SGL_ALLOC 1
#define CONFIG_PPC_BOOK3S_IDLE 1
#define CONFIG_TYPEC_HD3SS3220_MODULE 1
#define CONFIG_GENERIC_PCI_IOMAP 1
#define CONFIG_CRYPTO_USER_API_RNG 1
#define CONFIG_DVB_STV6110_MODULE 1
#define CONFIG_MLXFW_MODULE 1
#define CONFIG_NF_CT_PROTO_GRE 1
#define CONFIG_BT_HCIBFUSB_MODULE 1
#define CONFIG_KVM_BOOK3S_64_HV_MODULE 1
#define CONFIG_SOUND_MODULE 1
#define CONFIG_SENSORS_W83793_MODULE 1
#define CONFIG_NET_DSA_TAG_TRAILER_MODULE 1
#define CONFIG_NLS_MAC_TURKISH_MODULE 1
#define CONFIG_GENERIC_ALLOCATOR 1
#define CONFIG_DVB_TDA1004X_MODULE 1
#define CONFIG_HAVE_ARCH_KGDB 1
#define CONFIG_DRM_PANEL_RAYDIUM_RM692E5_MODULE 1
#define CONFIG_MTD_NAND_ECC 1
#define CONFIG_SND_USB_USX2Y_MODULE 1
#define CONFIG_NET_INGRESS 1
#define CONFIG_CRYPTO_AEAD2 1
#define CONFIG_INFINIBAND_VIRT_DMA 1
#define CONFIG_VIDEO_HI846_MODULE 1
#define CONFIG_DM_PERSISTENT_DATA_MODULE 1
#define CONFIG_NET_ACT_MPLS_MODULE 1
#define CONFIG_GENERIC_BUG_RELATIVE_POINTERS 1
#define CONFIG_SND_YMFPCI_MODULE 1
#define CONFIG_ARCH_STACKWALK 1
#define CONFIG_BNXT_MODULE 1
#define CONFIG_NODES_SHIFT 8
#define CONFIG_LCD_PLATFORM_MODULE 1
#define CONFIG_USB_HIDDEV 1
#define CONFIG_CEC_CH7322_MODULE 1
#define CONFIG_TCM_FILEIO_MODULE 1
#define CONFIG_DEBUG_INFO_NONE 1
#define CONFIG_HID_U2FZERO_MODULE 1
#define CONFIG_RV_MON_SSSW 1
#define CONFIG_BACKLIGHT_KTD253_MODULE 1
#define CONFIG_TOUCHSCREEN_ZFORCE_MODULE 1
#define CONFIG_WIZNET_W5100_MODULE 1
#define CONFIG_KUNIT_DEBUGFS 1
#define CONFIG_HFS_FS_MODULE 1
#define CONFIG_HAVE_ARCH_MMAP_RND_BITS 1
#define CONFIG_SND_FM801_MODULE 1
#define CONFIG_VIDEO_MT9V032_MODULE 1
#define CONFIG_RTAS_FLASH 1
#define CONFIG_HAVE_PERF_REGS 1
#define CONFIG_HW_RANDOM_XIPHERA_MODULE 1
#define CONFIG_RTW89_USB_MODULE 1
#define CONFIG_CXL_FEATURES 1
#define CONFIG_CRYPTO_DF80090A 1
#define CONFIG_MOTORCOMM_PHY_MODULE 1
#define CONFIG_VIDEO_AD5820_MODULE 1
#define CONFIG_PPS_GENERATOR_MODULE 1
#define CONFIG_VIDEO_MT9T112_MODULE 1
#define CONFIG_IMA_MEASURE_ASYMMETRIC_KEYS 1
#define CONFIG_DVB_STB0899_MODULE 1
#define CONFIG_PAGE_SIZE_LESS_THAN_256KB 1
#define CONFIG_IPV6_SIT_6RD 1
#define CONFIG_FUSE_DAX 1
#define CONFIG_HAS_IOPORT 1
#define CONFIG_REGULATOR_RT5759_MODULE 1
#define CONFIG_STRICT_MODULE_RWX 1
#define CONFIG_IXGBEVF_MODULE 1
#define CONFIG_CXL_BUS_MODULE 1
#define CONFIG_MAC802154_MODULE 1
#define CONFIG_DATA_SHIFT 24
#define CONFIG_USB_NET_SR9700_MODULE 1
#define CONFIG_PCS_MTK_LYNXI_MODULE 1
#define CONFIG_FXOS8700_MODULE 1
#define CONFIG_PCI 1
#define CONFIG_SENSORS_PMBUS_MODULE 1
#define CONFIG_USB_OHCI_HCD_PCI 1
#define CONFIG_SND_OPL3_LIB_SEQ_MODULE 1
#define CONFIG_USB_GSPCA_CPIA1_MODULE 1
#define CONFIG_CLANG_VERSION 0
#define CONFIG_DEV_DAX_MODULE 1
#define CONFIG_POWERNV_CPUIDLE 1
#define CONFIG_CEC_NOTIFIER 1
#define CONFIG_BRIDGE_EBT_MARK_MODULE 1
#define CONFIG_BATTERY_UG3105_MODULE 1
#define CONFIG_B43_MODULE 1
#define CONFIG_PPC_XICS 1
#define CONFIG_DM_CACHE_MODULE 1
#define CONFIG_GPIB_NEC7210_MODULE 1
#define CONFIG_HVC_RTAS 1
#define CONFIG_B43_SSB 1
#define CONFIG_HV_PERF_CTRS 1
#define CONFIG_SCD30_I2C_MODULE 1
#define CONFIG_SECURITY_APPARMOR_INTROSPECT_POLICY 1
#define CONFIG_ARCH_HAS_ELF_RANDOMIZE 1
#define CONFIG_HAVE_ARCH_WITHIN_STACK_FRAMES 1
#define CONFIG_NFS_V4_1_IMPLEMENTATION_ID_DOMAIN "kernel.org"
#define CONFIG_SCSI 1
#define CONFIG_DVB_ZL10036_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_IPRANGE_MODULE 1
#define CONFIG_NF_CONNTRACK_TIMESTAMP 1
#define CONFIG_INPUT_LEDS 1
#define CONFIG_USB_RTL8153_ECM_MODULE 1
#define CONFIG_MAGIC_SYSRQ 1
#define CONFIG_PPC_SUBPAGE_PROT 1
#define CONFIG_CRYPTO_KDF800108_CTR 1
#define CONFIG_REGULATOR_RT5739_MODULE 1
#define CONFIG_MAGIC_SYSRQ_SERIAL_SEQUENCE ""
#define CONFIG_BT_BNEP_MC_FILTER 1
#define CONFIG_SENSORS_MAX34440_MODULE 1
#define CONFIG_CRC7_MODULE 1
#define CONFIG_MEDIA_ANALOG_TV_SUPPORT 1
#define CONFIG_HAVE_KVM_IRQCHIP 1
#define CONFIG_LD_ORPHAN_WARN_LEVEL "warn"
#define CONFIG_AIX_PARTITION 1
#define CONFIG_RTW88_8703B_MODULE 1
#define CONFIG_DVB_TDA10071_MODULE 1
#define CONFIG_FONTS 1
#define CONFIG_MEDIA_TUNER_FC2580_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_JASTEC 1
#define CONFIG_CC_HAS_ASM_GOTO_OUTPUT 1
#define CONFIG_64BIT 1
#define CONFIG_I2C_MUX_LTC4306_MODULE 1
#define CONFIG_DVB_IX2505V_MODULE 1
#define CONFIG_USB_OHCI_LITTLE_ENDIAN 1
#define CONFIG_ISO9660_FS_MODULE 1
#define CONFIG_UNIX_DIAG 1
#define CONFIG_QTNFMAC_MODULE 1
#define CONFIG_PHY_PACKAGE_MODULE 1
#define CONFIG_NFS_SWAP 1
#define CONFIG_SND_HDA_GENERIC_LEDS 1
#define CONFIG_DVB_VES1820_MODULE 1
#define CONFIG_SENSORS_LM85_MODULE 1
#define CONFIG_APDS9306_MODULE 1
#define CONFIG_DMA_OPS_HELPERS 1
#define CONFIG_VIDEOBUF2_DVB_MODULE 1
#define CONFIG_DVB_STV0299_MODULE 1
#define CONFIG_IP_SCTP_MODULE 1
#define CONFIG_IP_VS_PROTO_SCTP 1
#define CONFIG_IP_VS_TWOS_MODULE 1
#define CONFIG_SENSORS_LTC2991_MODULE 1
#define CONFIG_RADIO_MAXIRADIO_MODULE 1
#define CONFIG_PM_DEBUG 1
#define CONFIG_VIDEOBUF2_DMA_SG_MODULE 1
#define CONFIG_NET_DSA_SMSC_LAN9303_MDIO_MODULE 1
#define CONFIG_MMC_SDHCI_XENON_MODULE 1
#define CONFIG_NET_VENDOR_TI 1
#define CONFIG_BACKLIGHT_LED_MODULE 1
#define CONFIG_BLK_CGROUP 1
#define CONFIG_BNX2X_SRIOV 1
#define CONFIG_SND_HDA_CODEC_REALTEK_LIB_MODULE 1
#define CONFIG_HAVE_ARCH_KFENCE 1
#define CONFIG_AL3010_MODULE 1
#define CONFIG_HID_CORSAIR_MODULE 1
#define CONFIG_IPV6_VTI_MODULE 1
#define CONFIG_RTL8XXXU_UNTESTED 1
#define CONFIG_HID_NTI_MODULE 1
#define CONFIG_DECOMPRESS_ZSTD 1
#define CONFIG_NF_LOG_ARP_MODULE 1
#define CONFIG_MTD_MAP_BANK_WIDTH_4 1
#define CONFIG_NF_NAT_AMANDA_MODULE 1
#define CONFIG_SND_UTIMER 1
#define CONFIG_NET_TEAM_MODULE 1
#define CONFIG_LEDS_MAX77705_MODULE 1
#define CONFIG_SSB_PCIHOST_POSSIBLE 1
#define CONFIG_SND_HDA_CODEC_ANALOG_MODULE 1
#define CONFIG_XPS 1
#define CONFIG_SPARSEMEM_MANUAL 1
#define CONFIG_6LOWPAN_GHC_EXT_HDR_FRAG_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_STATISTIC_MODULE 1
#define CONFIG_ZRAM_BACKEND_LZ4 1
#define CONFIG_VIRTIO_VDPA_MODULE 1
#define CONFIG_COMPAT_32BIT_TIME 1
#define CONFIG_MPTCP_KUNIT_TEST_MODULE 1
#define CONFIG_HOTPLUG_CPU 1
#define CONFIG_NFS_FSCACHE 1
#define CONFIG_PAGE_POOL_STATS 1
#define CONFIG_WANT_DEV_COREDUMP 1
#define CONFIG_NET_IPGRE_DEMUX_MODULE 1
#define CONFIG_SENSORS_MPQ7932_MODULE 1
#define CONFIG_JOYSTICK_GF2K_MODULE 1
#define CONFIG_DVB_PT1_MODULE 1
#define CONFIG_DRM_PANEL_JDI_R63452_MODULE 1
#define CONFIG_MISC_ALCOR_PCI_MODULE 1
#define CONFIG_NET_VENDOR_PENSANDO 1
#define CONFIG_VIDEO_HI847_MODULE 1
#define CONFIG_SERIAL_8250_DFL_MODULE 1
#define CONFIG_F2FS_FS_ZSTD 1
#define CONFIG_HARDLOCKUP_DETECTOR_ARCH 1
#define CONFIG_BLK_DEV_NVME 1
#define CONFIG_IBMVETH_MODULE 1
#define CONFIG_SENSORS_LM73_MODULE 1
#define CONFIG_DVB_USB_VP7045_MODULE 1
#define CONFIG_ARCH_HAS_SET_MEMORY 1
#define CONFIG_DVB_USB_DTT200U_MODULE 1
#define CONFIG_HID_CHERRY_MODULE 1
#define CONFIG_MAC80211_MESH 1
#define CONFIG_BLK_WBT 1
#define CONFIG_SATA_SIS_MODULE 1
#define CONFIG_TASKS_RUDE_RCU 1
#define CONFIG_AS21XXX_PHY_MODULE 1
#define CONFIG_MTD_UBI_WL_THRESHOLD 4096
#define CONFIG_USB_SERIAL_WWAN_MODULE 1
#define CONFIG_IP_SET_HASH_NETPORT_MODULE 1
#define CONFIG_DRM_PANEL_HIMAX_HX83102_MODULE 1
#define CONFIG_HUGETLB_PAGE_SIZE_VARIABLE 1
#define CONFIG_RTC_DRV_GENERIC 1
#define CONFIG_NET_ACT_SAMPLE_MODULE 1
#define CONFIG_USB_ARMLINUX 1
#define CONFIG_ZEROPLUS_FF 1
#define CONFIG_KUNIT_AUTORUN_ENABLED 1
#define CONFIG_HID_KEYTOUCH_MODULE 1
#define CONFIG_RTW89_8852CE_MODULE 1
#define CONFIG_IMA 1
#define CONFIG_REGULATOR_MP886X_MODULE 1
#define CONFIG_SMSC_PHY_MODULE 1
#define CONFIG_BLK_INLINE_ENCRYPTION 1
#define CONFIG_NETFILTER_XT_MATCH_ADDRTYPE_MODULE 1
#define CONFIG_SENSORS_LTC4260_MODULE 1
#define CONFIG_CC_CAN_LINK 1
#define CONFIG_BT_MTKSDIO_MODULE 1
#define CONFIG_B53_MODULE 1
#define CONFIG_JOYSTICK_IFORCE_USB_MODULE 1
#define CONFIG_MT7925_COMMON_MODULE 1
#define CONFIG_CHARGER_MAX77976_MODULE 1
#define CONFIG_NF_LOG_IPV6_MODULE 1
#define CONFIG_CC_HAS_AUTO_VAR_INIT_ZERO_BARE 1
#define CONFIG_MMC_VUB300_MODULE 1
#define CONFIG_IPMI_PLAT_DATA 1
#define CONFIG_KERNFS 1
#define CONFIG_DRM_MGAG200_MODULE 1
#define CONFIG_PARAVIRT_SPINLOCKS 1
#define CONFIG_DM_MULTIPATH_HST_MODULE 1
#define CONFIG_6LOWPAN_NHC_UDP_MODULE 1
#define CONFIG_SERIO_SERPORT_MODULE 1
#define CONFIG_RD_XZ 1
#define CONFIG_DRM_AMDGPU_CIK 1
#define CONFIG_PAGE_SIZE_64KB 1
#define CONFIG_VIDEO_V4L2_I2C 1
#define CONFIG_MSDOS_FS_MODULE 1
#define CONFIG_USB_ULPI_BUS_MODULE 1
#define CONFIG_PPC_POWERNV 1
#define CONFIG_DM_EBS_MODULE 1
#define CONFIG_SECURITY_TOMOYO_POLICY_LOADER "/sbin/tomoyo-init"
#define CONFIG_ARCH_PKEY_BITS 5
#define CONFIG_XMON_DISASSEMBLY 1
#define CONFIG_IP_VS_SH_TAB_BITS 8
#define CONFIG_BE2NET_SKYHAWK 1
#define CONFIG_SND_MTPAV_MODULE 1
#define CONFIG_DVB_S5H1411_MODULE 1
#define CONFIG_ATH6KL_SDIO_MODULE 1
#define CONFIG_HAVE_SCHED_AVG_IRQ 1
#define CONFIG_CRC_ITU_T_MODULE 1
#define CONFIG_PFCP_MODULE 1
#define CONFIG_SCSI_SRP_ATTRS_MODULE 1
#define CONFIG_BALLOON 1
#define CONFIG_SND_ECHO3G_MODULE 1
#define CONFIG_KEYBOARD_QT1050_MODULE 1
#define CONFIG_SSB_SDIOHOST_POSSIBLE 1
#define CONFIG_SENSORS_UCD9200_MODULE 1
#define CONFIG_SENSORS_RM3100_MODULE 1
#define CONFIG_INFINIBAND_RTRS_MODULE 1
#define CONFIG_LEDS_POWERNV_MODULE 1
#define CONFIG_PROC_PAGE_MONITOR 1
#define CONFIG_FUN_ETH_MODULE 1
#define CONFIG_HAVE_ARCH_SECCOMP_FILTER 1
#define CONFIG_NET_DSA_TAG_OCELOT_MODULE 1
#define CONFIG_DEBUG_BUGVERBOSE 1
#define CONFIG_PLAYSTATION_FF 1
#define CONFIG_SENSORS_MP2888_MODULE 1
#define CONFIG_ATH9K_COMMON_DEBUG 1
#define CONFIG_EXT4_USE_FOR_EXT2 1
#define CONFIG_USB_PEGASUS_MODULE 1
#define CONFIG_PTP_1588_CLOCK_IDTCM_MODULE 1
#define CONFIG_TRANSPARENT_HUGEPAGE_SHMEM_HUGE_NEVER 1
#define CONFIG_UIO_PCI_GENERIC_MODULE 1
#define CONFIG_OF_IOMMU 1
#define CONFIG_SLAB_FREELIST_RANDOM 1
#define CONFIG_MFD_MAX77650_MODULE 1
#define CONFIG_ARCH_SUSPEND_POSSIBLE 1
#define CONFIG_USB_IDMOUSE_MODULE 1
#define CONFIG_KPROBES 1
#define CONFIG_TEST_IOV_ITER_MODULE 1
#define CONFIG_TOUCHSCREEN_IQS7211_MODULE 1
#define CONFIG_RT_MUTEXES 1
#define CONFIG_UPROBE_EVENTS 1
#define CONFIG_PCI_DOE 1
#define CONFIG_BLK_CGROUP_IOLATENCY 1
#define CONFIG_CEC_PIN 1
#define CONFIG_CHELSIO_T4VF_MODULE 1
#define CONFIG_NET_IFE_MODULE 1
#define CONFIG_RD_LZ4 1
#define CONFIG_USB_SERIAL_EDGEPORT_MODULE 1
#define CONFIG_SECURITY_TOMOYO 1
#define CONFIG_VIDEO_ADV7175_MODULE 1
#define CONFIG_IPMI_SSIF_MODULE 1
#define CONFIG_NET_DSA_XRS700X_MDIO_MODULE 1
#define CONFIG_SENSORS_LIS3_I2C_MODULE 1
#define CONFIG_DVB_MXL692_MODULE 1
#define CONFIG_VIDEO_CX231XX_ALSA_MODULE 1
#define CONFIG_MFD_SY7636A_MODULE 1
#define CONFIG_DEFAULT_HOSTNAME "gentoo"
#define CONFIG_VFIO_PCI_INTX 1
#define CONFIG_MLX5_MACSEC 1
#define CONFIG_GENERIC_NET_UTILS 1
#define CONFIG_LMP91000_MODULE 1
#define CONFIG_DM_ERA_MODULE 1
#define CONFIG_RTW88_8812AU_MODULE 1
#define CONFIG_RTC_DRV_RX8010_MODULE 1
#define CONFIG_GENERIC_ISA_DMA 1
#define CONFIG_IP_VS_WRR_MODULE 1
#define CONFIG_LEDS_TRIGGER_ONESHOT_MODULE 1
#define CONFIG_NVDIMM_PFN 1
#define CONFIG_SECURITY_SELINUX 1
#define CONFIG_BLK_DEV_DRBD_MODULE 1
#define CONFIG_INPUT_JOYSTICK 1
#define CONFIG_BLK_DEV_DM_BUILTIN 1
#define CONFIG_DRM_DISPLAY_DSC_HELPER 1
#define CONFIG_SMARTJOYPLUS_FF 1
#define CONFIG_SCSI_LOWLEVEL 1
#define CONFIG_DRM_PANEL_NOVATEK_NT35510_MODULE 1
#define CONFIG_SENSORS_LM63_MODULE 1
#define CONFIG_INTERVAL_TREE 1
#define CONFIG_RTC_DRV_PCF8563_MODULE 1
#define CONFIG_FUSION_FC_MODULE 1
#define CONFIG_W1_SLAVE_DS2805_MODULE 1
#define CONFIG_SPARSEMEM 1
#define CONFIG_CXL_PMU_MODULE 1
#define CONFIG_BCACHE_MODULE 1
#define CONFIG_CRYPTO_DEV_ATMEL_SHA204A_MODULE 1
#define CONFIG_INPUT_IQS7222_MODULE 1
#define CONFIG_HVC_DRIVER 1
#define CONFIG_BLK_CGROUP_IOCOST 1
#define CONFIG_NLS_MAC_GAELIC_MODULE 1
#define CONFIG_DRM_PANEL_SHARP_LQ079L1SX01_MODULE 1
#define CONFIG_NFP_NET_IPSEC 1
#define CONFIG_NET_VENDOR_BROADCOM 1
#define CONFIG_SPEAKUP_SYNTH_SPKOUT_MODULE 1
#define CONFIG_RTC_SYSTOHC_DEVICE "rtc0"
#define CONFIG_INPUT_GPIO_VIBRA_MODULE 1
#define CONFIG_OSNOISE_TRACER 1
#define CONFIG_RTC_INTF_PROC 1
#define CONFIG_RC_LOOPBACK_MODULE 1
#define CONFIG_AUTOFS_FS 1
#define CONFIG_DVB_CX24123_MODULE 1
#define CONFIG_SND_HDA_CODEC_ALC861_MODULE 1
#define CONFIG_LD_CAN_USE_KEEP_IN_OVERLAY 1
#define CONFIG_I2C_SIMTEC_MODULE 1
#define CONFIG_IPV6_MROUTE_MULTIPLE_TABLES 1
#define CONFIG_FUNCTION_PROFILER 1
#define CONFIG_BT_BREDR 1
#define CONFIG_TYPEC_DP_ALTMODE_MODULE 1
#define CONFIG_MODULE_COMPRESS_XZ 1
#define CONFIG_XFRM_ESPINTCP 1
#define CONFIG_LEDS_LP3952_MODULE 1
#define CONFIG_NET_DSA_QCA8K_MODULE 1
#define CONFIG_USB_HID 1
#define CONFIG_SMSC911X_MODULE 1
#define CONFIG_PHY_CADENCE_TORRENT_MODULE 1
#define CONFIG_USB_HSO_MODULE 1
#define CONFIG_NFT_BRIDGE_REJECT_MODULE 1
#define CONFIG_SND_HDA_RECONFIG 1
#define CONFIG_PARMAN_MODULE 1
#define CONFIG_IIO_BUFFER_CB_MODULE 1
#define CONFIG_CRYPTO_SKCIPHER2 1
#define CONFIG_MTD_MODULE 1
#define CONFIG_SENSORS_LTC4261_MODULE 1
#define CONFIG_PCI_MSI 1
#define CONFIG_UBIFS_ATIME_SUPPORT 1
#define CONFIG_RMI4_CORE_MODULE 1
#define CONFIG_PATA_PDC2027X_MODULE 1
#define CONFIG_HAVE_GENERIC_VDSO 1
#define CONFIG_SND_HDA_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_PANJIT 1
#define CONFIG_VIDEO_DW9719_MODULE 1
#define CONFIG_NF_TABLES_ARP 1
#define CONFIG_TMPFS_POSIX_ACL 1
#define CONFIG_VIDEO_OV08D10_MODULE 1
#define CONFIG_MODULES_TREE_LOOKUP 1
#define CONFIG_CGROUP_DMEM 1
#define CONFIG_FW_LOADER_PAGED_BUF 1
#define CONFIG_AIC79XX_DEBUG_MASK 0
#define CONFIG_IRQ_MSI_LIB 1
#define CONFIG_NILFS2_FS_MODULE 1
#define CONFIG_IP_VS_PE_SIP_MODULE 1
#define CONFIG_USB_SERIAL_KOBIL_SCT_MODULE 1
#define CONFIG_BATMAN_ADV_MODULE 1
#define CONFIG_NXP_C45_TJA11XX_PHY_MODULE 1
#define CONFIG_SATA_INIC162X_MODULE 1
#define CONFIG_NVMEM_RMEM_MODULE 1
#define CONFIG_USB_NET_CDC_EEM_MODULE 1
#define CONFIG_GENTOO_PRINT_FIRMWARE_INFO 1
#define CONFIG_BMI160_I2C_MODULE 1
#define CONFIG_MEDIA_TUNER_E4000_MODULE 1
#define CONFIG_CPUMASK_KUNIT_TEST_MODULE 1
#define CONFIG_RTC_DRV_DS1286_MODULE 1
#define CONFIG_RTL8723AE_MODULE 1
#define CONFIG_EEH 1
#define CONFIG_SND_HDA_CODEC_HDMI_TEGRA_MODULE 1
#define CONFIG_DRM_CLIENT_DEFAULT_FBDEV 1
#define CONFIG_SND_AU8820_MODULE 1
#define CONFIG_SENSORS_TDA38640_REGULATOR 1
#define CONFIG_CPU_FREQ_GOV_USERSPACE 1
#define CONFIG_TOUCHSCREEN_USB_ETT_TC45USB 1
#define CONFIG_SCSI_SYM53C8XX_DEFAULT_TAGS 16
#define CONFIG_BLK_DEV_NULL_BLK_MODULE 1
#define CONFIG_IP_VS_PROTO_AH_ESP 1
#define CONFIG_JOLIET 1
#define CONFIG_V4L_MEM2MEM_DRIVERS 1
#define CONFIG_JOYSTICK_IFORCE_232_MODULE 1
#define CONFIG_INFINIBAND_OCRDMA_MODULE 1
#define CONFIG_EROFS_FS_ZIP 1
#define CONFIG_DVB_S5H1409_MODULE 1
#define CONFIG_SENSORS_LM83_MODULE 1
#define CONFIG_SQUASHFS_DECOMP_MULTI_PERCPU 1
#define CONFIG_PSERIES_PLPKS 1
#define CONFIG_B43LEGACY_PIO 1
#define CONFIG_HAVE_DYNAMIC_FTRACE_WITH_DIRECT_CALLS 1
#define CONFIG_ARCH_HAS_PREEMPT_LAZY 1
#define CONFIG_SERIAL_8250_DW 1
#define CONFIG_ACENIC_MODULE 1
#define CONFIG_MD_RAID456_MODULE 1
#define CONFIG_I6300ESB_WDT_MODULE 1
#define CONFIG_PATA_PDC_OLD_MODULE 1
#define CONFIG_IMA_KEXEC 1
#define CONFIG_HOTPLUG_PCI_RPA_MODULE 1
#define CONFIG_W1_SLAVE_DS2433_CRC 1
#define CONFIG_MMC_WBSD_MODULE 1
#define CONFIG_PPC_SMLPAR 1
#define CONFIG_HW_RANDOM_TPM 1
#define CONFIG_ILLEGAL_POINTER_VALUE 0x5deadbeef0000000
#define CONFIG_IWLEGACY_DEBUG 1
#define CONFIG_CRYPTO_DEV_NX 1
#define CONFIG_INPUT_MATRIXKMAP_MODULE 1
#define CONFIG_IP_NF_MATCH_AH_MODULE 1
#define CONFIG_GROUP_SCHED_WEIGHT 1
#define CONFIG_MEDIA_TUNER_SIMPLE_MODULE 1
#define CONFIG_UPROBES 1
#define CONFIG_VP_VDPA_MODULE 1
#define CONFIG_SENSORS_OCC_P9_SBE_MODULE 1
#define CONFIG_DW_XDATA_PCIE_MODULE 1
#define CONFIG_NFT_NAT_MODULE 1
#define CONFIG_ZSWAP_COMPRESSOR_DEFAULT "lzo"
#define CONFIG_DRM_PANEL_RAYDIUM_RM67200_MODULE 1
#define CONFIG_VIDEO_CS53L32A_MODULE 1
#define CONFIG_MAX31827_MODULE 1
#define CONFIG_NF_CONNTRACK_ZONES 1
#define CONFIG_DRM_CLIENT 1
#define CONFIG_SCSI_AM53C974_MODULE 1
#define CONFIG_PPC_DAWR 1
#define CONFIG_USB_UHCI_HCD 1
#define CONFIG_SENSORS_VIA686A_MODULE 1
#define CONFIG_MT76x0E_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_MULTIPORT_MODULE 1
#define CONFIG_IIO_KX022A_I2C_MODULE 1
#define CONFIG_GENERIC_PHY 1
#define CONFIG_FCOE_MODULE 1
#define CONFIG_NETFILTER_XT_MATCH_LIMIT_MODULE 1
#define CONFIG_IPV6_MROUTE 1
#define CONFIG_USB_RAINSHADOW_CEC_MODULE 1
#define CONFIG_INET6_TUNNEL_MODULE 1
#define CONFIG_MCP4728_MODULE 1
#define CONFIG_DVB_BT8XX_MODULE 1
#define CONFIG_NET_DSA_TAG_MXL_862XX_MODULE 1
#define CONFIG_VIDEO_ET8EK8_MODULE 1
#define CONFIG_MT7925U_MODULE 1
#define CONFIG_IWL3945_MODULE 1
#define CONFIG_DUMMY_CONSOLE 1
#define CONFIG_FB_TILEBLITTING 1
#define CONFIG_RTL_CARDS_MODULE 1
#define CONFIG_PANTHERLORD_FF 1
#define CONFIG_DVB_TUNER_DIB0090_MODULE 1
#define CONFIG_HAVE_ARCH_TRACEHOOK 1
#define CONFIG_IXGBE_HWMON 1
#define CONFIG_DVB_PT3_MODULE 1
#define CONFIG_NET_DSA_SMSC_LAN9303_MODULE 1
#define CONFIG_USB_SERIAL_XR_MODULE 1
#define CONFIG_IP_SET_MODULE 1
#define CONFIG_IP_SET_BITMAP_PORT_MODULE 1
#define CONFIG_TCP_CONG_NV_MODULE 1
#define CONFIG_VIDEO_CCS_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_TCPMSS_MODULE 1
#define CONFIG_SLHC_MODULE 1
#define CONFIG_VIDEO_WM8775_MODULE 1
#define CONFIG_SLUB_DEBUG 1
#define CONFIG_PPS_CLIENT_LDISC_MODULE 1
#define CONFIG_USB_GSPCA_SQ905_MODULE 1
#define CONFIG_HW_RANDOM_POWERNV_MODULE 1
#define CONFIG_EDAC_SCRUB 1
#define CONFIG_TOUCHSCREEN_USB_GENERAL_TOUCH 1
#define CONFIG_DVB_CX24120_MODULE 1
#define CONFIG_ATH10K_SDIO_MODULE 1
#define CONFIG_DVB_USB_DIBUSB_MB_MODULE 1
#define CONFIG_BRIDGE_CFM 1
#define CONFIG_BAREUDP_MODULE 1
#define CONFIG_PATA_HPT37X_MODULE 1
#define CONFIG_LEDS_BLINKM_MODULE 1
#define CONFIG_RADIO_TEA575X_MODULE 1
#define CONFIG_MMC_RICOH_MMC 1
#define CONFIG_CRYPTO_SHA256 1
#define CONFIG_ATH10K_MODULE 1
#define CONFIG_SND_CTL_LED_MODULE 1
#define CONFIG_NUMA_BALANCING 1
#define CONFIG_VIRTIO_MMIO_CMDLINE_DEVICES 1
#define CONFIG_ATM_LANE_MODULE 1
#define CONFIG_FWCTL_MODULE 1
#define CONFIG_DRM_PANEL_SEIKO_43WVF1G_MODULE 1
#define CONFIG_USB_GSPCA_KONICA_MODULE 1
#define CONFIG_NET_EMATCH_CMP_MODULE 1
#define CONFIG_QUOTACTL 1
#define CONFIG_BRIDGE_NF_EBTABLES_MODULE 1
#define CONFIG_EXFAT_DEFAULT_IOCHARSET "utf8"
#define CONFIG_VIDEOBUF2_V4L2_MODULE 1
#define CONFIG_NET_VENDOR_ARC 1
#define CONFIG_WIL6210_DEBUGFS 1
#define CONFIG_BT_HCIUART_BCM 1
#define CONFIG_NET_VENDOR_NVIDIA 1
#define CONFIG_UNIX 1
#define CONFIG_MEDIA_TUNER_M88RS6000T_MODULE 1
#define CONFIG_IRQ_WORK 1
#define CONFIG_IPV6_RPL_LWTUNNEL 1
#define CONFIG_ARCH_ENABLE_MEMORY_HOTREMOVE 1
#define CONFIG_NETFILTER_XT_MATCH_COMMENT_MODULE 1
#define CONFIG_EROFS_FS_XATTR 1
#define CONFIG_NEED_SRCU_NMI_SAFE 1
#define CONFIG_TRACING_MAP 1
#define CONFIG_NLS_MAC_CROATIAN_MODULE 1
#define CONFIG_SCSI_HPSA_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_TPROXY_MODULE 1
#define CONFIG_SGI_PARTITION 1
#define CONFIG_DUMMY_CONSOLE_COLUMNS 80
#define CONFIG_SUNGEM_MODULE 1
#define CONFIG_NF_LOG_IPV4_MODULE 1
#define CONFIG_TLS_MODULE 1
#define CONFIG_NET_DEVMEM 1
#define CONFIG_ATH9K_PCI 1
#define CONFIG_B43LEGACY_LEDS 1
#define CONFIG_TCP_CONG_HTCP_MODULE 1
#define CONFIG_SND_LAYLA24_MODULE 1
#define CONFIG_HID_PICOLCD_BACKLIGHT 1
#define CONFIG_STACKTRACE 1
#define CONFIG_REGULATOR_MAX5970_MODULE 1
#define CONFIG_WCN36XX_MODULE 1
#define CONFIG_DTPM 1
#define CONFIG_DRM_TI_SN65DSI86_MODULE 1
#define CONFIG_DVB_USB_CINERGY_T2_MODULE 1
#define CONFIG_BT_HCIBTUSB_AUTOSUSPEND 1
#define CONFIG_TYPHOON_MODULE 1
#define CONFIG_8139TOO_8129 1
#define CONFIG_6LOWPAN_NHC_HOP_MODULE 1
#define CONFIG_HAVE_ARCH_TRANSPARENT_HUGEPAGE_PUD 1
#define CONFIG_TUN_MODULE 1
#define CONFIG_SCSI_IPR_DUMP 1
#define CONFIG_NET_CLS_BASIC_MODULE 1
#define CONFIG_TCP_CONG_LP_MODULE 1
#define CONFIG_DVB_USB_TECHNISAT_USB2_MODULE 1
#define CONFIG_KUNIT_DEFAULT_TIMEOUT 300
#define CONFIG_SATA_QSTOR_MODULE 1
#define CONFIG_RTC_DRV_RV3029_HWMON 1
#define CONFIG_SENSORS_IBMPEX_MODULE 1
#define CONFIG_MAC80211_DEBUGFS 1
#define CONFIG_IXGBE_DCB 1
#define CONFIG_MOUSE_BCM5974_MODULE 1
#define CONFIG_ATH5K_MODULE 1
#define CONFIG_SENSORS_MAX6639_MODULE 1
#define CONFIG_SENSORS_MAX31790_MODULE 1
#define CONFIG_CRYPTO_XCBC_MODULE 1
#define CONFIG_FSI_MODULE 1
#define CONFIG_AR5523_MODULE 1
#define CONFIG_MLX4_CORE_GEN2 1
#define CONFIG_HW_RANDOM 1
#define CONFIG_GPIO_FXL6408_MODULE 1
#define CONFIG_IP_NF_TARGET_MASQUERADE_MODULE 1
#define CONFIG_RCU_NEED_SEGCBLIST 1
#define CONFIG_SENSORS_LM80_MODULE 1
#define CONFIG_NET_DSA_TAG_BRCM_LEGACY_FCS_MODULE 1
#define CONFIG_SENSORS_INA233_MODULE 1
#define CONFIG_USB_NET_CX82310_ETH_MODULE 1
#define CONFIG_BT_QCA_MODULE 1
#define CONFIG_CC_HAS_PCREL 1
#define CONFIG_LEDS_MAX5970_MODULE 1
#define CONFIG_SLIP_SMART 1
#define CONFIG_NEED_DMA_MAP_STATE 1
#define CONFIG_BT_MTKUART_MODULE 1
#define CONFIG_TCM_FC_MODULE 1
#define CONFIG_UBIFS_FS_MODULE 1
#define CONFIG_RCU_EXP_CPU_STALL_TIMEOUT 0
#define CONFIG_OF 1
#define CONFIG_ZL3073X_MODULE 1
#define CONFIG_MODULE_SIG_KEY "/var/tmp/portage/sys-kernel/gentoo-kernel-7.0.13/temp/kernel_key.pem"
#define CONFIG_ARCH_WANT_OPTIMIZE_DAX_VMEMMAP 1
#define CONFIG_ARCH_MMAP_RND_BITS_MIN 14
#define CONFIG_UBSAN_SHIFT 1
#define CONFIG_USB_SERIAL_SYMBOL_MODULE 1
#define CONFIG_OPENVSWITCH_VXLAN_MODULE 1
#define CONFIG_VFIO_PCI_DMABUF 1
#define CONFIG_CRYPTO_BLAKE2B 1
#define CONFIG_NVDIMM_DAX 1
#define CONFIG_SENSORS_PLI1209BC_MODULE 1
#define CONFIG_GFS2_FS_MODULE 1
#define CONFIG_INFINIBAND_ADDR_TRANS_CONFIGFS 1
#define CONFIG_UIO_DFL_MODULE 1
#define CONFIG_ATA_FORCE 1
#define CONFIG_NET_VENDOR_DEC 1
#define CONFIG_IP_SET_HASH_IPMAC_MODULE 1
#define CONFIG_SND_HDA_CODEC_CA0132_DSP 1
#define CONFIG_SENSORS_DS1621_MODULE 1
#define CONFIG_RT2800USB_UNKNOWN 1
#define CONFIG_SYSVIPC 1
#define CONFIG_LOCKDEP_SUPPORT 1
#define CONFIG_VIDEO_OV5640_MODULE 1
#define CONFIG_NFT_FIB_IPV6_MODULE 1
#define CONFIG_MEDIA_TUNER_TDA18250_MODULE 1
#define CONFIG_MEDIA_TUNER_QM1D1B0004_MODULE 1
#define CONFIG_IIO_ST_GYRO_3AXIS_MODULE 1
#define CONFIG_BT_HCIUART_LL 1
#define CONFIG_TCP_SIGPOOL 1
#define CONFIG_RT2X00_LIB_PCI_MODULE 1
#define CONFIG_FSCACHE 1
#define CONFIG_MFD_SM501_MODULE 1
#define CONFIG_DM_INTEGRITY_MODULE 1
#define CONFIG_MEDIA_SUPPORT_FILTER 1
#define CONFIG_CRYPTO 1
#define CONFIG_NF_TABLES_BRIDGE_MODULE 1
#define CONFIG_PARAVIRT 1
#define CONFIG_PHY_NXP_PTN3222_MODULE 1
#define CONFIG_HID_ELECOM_MODULE 1
#define CONFIG_LOGO 1
#define CONFIG_FPGA_M10_BMC_SEC_UPDATE_MODULE 1
#define CONFIG_HAVE_ARCH_SECCOMP 1
#define CONFIG_SND_CORE_TEST_MODULE 1
#define CONFIG_VCPU_STALL_DETECTOR_MODULE 1
#define CONFIG_DVB_TUNER_CX24113_MODULE 1
#define CONFIG_CRYPTO_AES_GCM_P10_MODULE 1
#define CONFIG_HID_MCP2221_MODULE 1
#define CONFIG_VFIO_GROUP 1
#define CONFIG_VIDEO_OV9640_MODULE 1
#define CONFIG_CHARGER_MAX77650_MODULE 1
#define CONFIG_VIDEO_VIVID_MAX_DEVS 64
#define CONFIG_BLK_CGROUP_FC_APPID 1
#define CONFIG_PROC_FS 1
#define CONFIG_USB_ALI_M5632 1
#define CONFIG_RSI_SDIO_MODULE 1
#define CONFIG_SND_CTL_FAST_LOOKUP 1
#define CONFIG_DVB_USB_DIBUSB_MC_MODULE 1
#define CONFIG_TXGBE_MODULE 1
#define CONFIG_SENSORS_MAX16064_MODULE 1
#define CONFIG_CRYPTO_MICHAEL_MIC_MODULE 1
#define CONFIG_CRYPTO_DEV_ATMEL_ECC_MODULE 1
#define CONFIG_DRM_CLIENT_SELECTION_MODULE 1
#define CONFIG_NET_VENDOR_TEHUTI 1
#define CONFIG_NF_CT_PROTO_SCTP 1
#define CONFIG_BLK_DEV_PMEM_MODULE 1
#define CONFIG_USB_GSPCA_SPCA508_MODULE 1
#define CONFIG_SX_COMMON_MODULE 1
#define CONFIG_BPF_EVENTS 1
#define CONFIG_TEST_LKM_MODULE 1
#define CONFIG_MEDIA_PLATFORM_SUPPORT 1
#define CONFIG_TMPFS_QUOTA 1
#define CONFIG_VIDEO_VISL_MODULE 1
#define CONFIG_POSIX_TIMERS 1
#define CONFIG_HID_BELKIN_MODULE 1
#define CONFIG_DVB_ISL6421_MODULE 1
#define CONFIG_SND_OXYGEN_MODULE 1
#define CONFIG_I2C_ATR_MODULE 1
#define CONFIG_USB_SERIAL_KLSI_MODULE 1
#define CONFIG_BME680_MODULE 1
#define CONFIG_PCS_LYNX_MODULE 1
#define CONFIG_HAVE_FTRACE_REGS_HAVING_PT_REGS 1
#define CONFIG_NOP_USB_XCEIV_MODULE 1
#define CONFIG_GENERIC_TRACER 1
#define CONFIG_NET_DSA_MT7530_MDIO_MODULE 1
#define CONFIG_ATA_GENERIC_MODULE 1
#define CONFIG_SQUASHFS_XZ 1
#define CONFIG_SCSI_SMARTPQI_MODULE 1
#define CONFIG_CC_HAS_COUNTED_BY 1
#define CONFIG_PPP_ASYNC_MODULE 1
#define CONFIG_OSF_PARTITION 1
#define CONFIG_DVB_MAX_ADAPTERS 16
#define CONFIG_NET_VENDOR_SIS 1
#define CONFIG_BCMA_BLOCKIO 1
#define CONFIG_V4L2_MEM2MEM_DEV_MODULE 1
#define CONFIG_LEDS_REGULATOR_MODULE 1
#define CONFIG_NOUVEAU_DEBUG_DEFAULT 3
#define CONFIG_RTC_DRV_RS5C372_MODULE 1
#define CONFIG_HAVE_PREEMPT_DYNAMIC_KEY 1
#define CONFIG_XFRM_AH_MODULE 1
#define CONFIG_DVB_DIB8000_MODULE 1
#define CONFIG_INET_DIAG 1
#define CONFIG_JOYSTICK_A3D_MODULE 1
#define CONFIG_ATM_ENI_MODULE 1
#define CONFIG_FS_IOMAP 1
#define CONFIG_IO_EVENT_IRQ 1
#define CONFIG_FB_IOMEM_HELPERS 1
#define CONFIG_NATIONAL_PHY_MODULE 1
#define CONFIG_SENSORS_STEF48H28_MODULE 1
#define CONFIG_DRM_QXL_MODULE 1
#define CONFIG_INPUT_VIVALDIFMAP_MODULE 1
#define CONFIG_THERMAL_STATISTICS 1
#define CONFIG_USB_GSPCA_DTCS033_MODULE 1
#define CONFIG_IP_VS_PROTO_TCP 1
#define CONFIG_USB_GL860_MODULE 1
#define CONFIG_ALLOW_DEV_COREDUMP 1
#define CONFIG_ASN1 1
#define CONFIG_NF_NAT_PPTP_MODULE 1
#define CONFIG_STAGING_MEDIA 1
#define CONFIG_RAID_ATTRS_MODULE 1
#define CONFIG_MMC_CQHCI_MODULE 1
#define CONFIG_DRM_DISPLAY_HDMI_AUDIO_HELPER 1
#define CONFIG_DRM_PANEL_DSI_CM_MODULE 1
#define CONFIG_DEVTMPFS_MOUNT 1
#define CONFIG_REGULATOR_BD9571MWV_MODULE 1
#define CONFIG_NFC_ST21NFCA_I2C_MODULE 1
#define CONFIG_ARCH_HAS_DMA_OPS 1
#define CONFIG_HID_LOGITECH_DJ_MODULE 1
#define CONFIG_USB_SERIAL_EDGEPORT_TI_MODULE 1
#define CONFIG_CONSOLE_LOGLEVEL_QUIET 3
#define CONFIG_DRM_PANEL_SAMSUNG_AMS639RQ08_MODULE 1
#define CONFIG_VIDEO_BT819_MODULE 1
#define CONFIG_NETFILTER_XT_TARGET_NFLOG_MODULE 1
#define CONFIG_SCSI_EFCT_MODULE 1
#define CONFIG_XILINX_PR_DECOUPLER_MODULE 1
#define CONFIG_INET_TUNNEL_MODULE 1
#define CONFIG_CC_HAS_ELFV2 1
#define CONFIG_MLX5_EN_PSP 1
#define CONFIG_MPROFILE_KERNEL 1
#define CONFIG_VIDEO_WM8739_MODULE 1
#define CONFIG_PCI_SYSCALL 1
#define CONFIG_LZO_DECOMPRESS 1
#define CONFIG_DM_UNSTRIPED_MODULE 1
#define CONFIG_VIRTIO_VSOCKETS_MODULE 1
#define CONFIG_JUMP_LABEL 1
#define CONFIG_GENERIC_SMP_IDLE_THREAD 1
#define CONFIG_USB_ARCH_HAS_HCD 1
#define CONFIG_REED_SOLOMON_ENC8 1
#define CONFIG_NET_EMATCH_STACK 32
#define CONFIG_NF_NAT_FTP_MODULE 1
#define CONFIG_CRYPTO_AEGIS128_MODULE 1
#define CONFIG_DPOT_DAC_MODULE 1
#define CONFIG_LDM_PARTITION 1
#define CONFIG_EEPROM_EE1004_MODULE 1
#define CONFIG_DRM_PANEL_RENESAS_R61307_MODULE 1
#define CONFIG_IAVF_MODULE 1
#define CONFIG_UBSAN_BOUNDS_STRICT 1
#define CONFIG_VIDEO_UDA1342_MODULE 1
#define CONFIG_ARCH_MMAP_RND_COMPAT_BITS_MIN 7
#define CONFIG_NET_EMATCH_CANID_MODULE 1
#define CONFIG_RTC_HCTOSYS 1
#define CONFIG_EXTRA_FIRMWARE ""
#define CONFIG_IP_MULTICAST 1
#define CONFIG_LATENCYTOP 1
#define CONFIG_DVB_BUDGET_MODULE 1
#define CONFIG_ASYMMETRIC_PUBLIC_KEY_SUBTYPE 1
#define CONFIG_DVB_STV0288_MODULE 1
#define CONFIG_PHY_CADENCE_SIERRA_MODULE 1
#define CONFIG_SENSORS_MAX6650_MODULE 1
#define CONFIG_LOCKUP_DETECTOR 1
#define CONFIG_FUSE_FS_MODULE 1
#define CONFIG_CRYPTO_LIB_SHA256_KUNIT_TEST_MODULE 1
#define CONFIG_USB_STV06XX_MODULE 1
#define CONFIG_ABP2030PA_I2C_MODULE 1
#define CONFIG_NFC_PN533_USB_MODULE 1
#define CONFIG_IO_URING_BPF 1
#define CONFIG_SENSORS_LIS3LV02D_MODULE 1
#define CONFIG_SERIAL_8250_DWLIB 1
#define CONFIG_ZRAM_BACKEND_ZSTD 1
#define CONFIG_OF_KOBJ 1
#define CONFIG_KSZ884X_PCI_MODULE 1
#define CONFIG_RTW88_CORE_MODULE 1
#define CONFIG_PSTORE_RAM_MODULE 1
#define CONFIG_MEDIA_TUNER_TUA9001_MODULE 1
#define CONFIG_SENSORS_ISL28022_MODULE 1
#define CONFIG_BT_HCIUART_NOKIA_MODULE 1
#define CONFIG_NOP_TRACER 1
#define CONFIG_MICREL_PHY_MODULE 1
#define CONFIG_CLK_GATE_KUNIT_TEST_MODULE 1
#define CONFIG_LZ4_COMPRESS_MODULE 1
#define CONFIG_TOUCHSCREEN_USB_ETURBO 1
#define CONFIG_NETFILTER_XT_MATCH_SOCKET_MODULE 1
#define CONFIG_ARCH_HAS_FORCE_DMA_UNENCRYPTED 1
#define CONFIG_IP_VS_LBLCR_MODULE 1
#define CONFIG_REGULATOR_AW37503_MODULE 1
#define CONFIG_BTREE 1
#define CONFIG_DRM_SII9234_MODULE 1
#define CONFIG_PNFS_FLEXFILE_LAYOUT_MODULE 1
#define CONFIG_AMD_QDMA_MODULE 1
#define CONFIG_FW_CS_DSP_KUNIT_TEST_MODULE 1
#define CONFIG_VETH_MODULE 1
#define CONFIG_USB_GSPCA_CONEX_MODULE 1
#define CONFIG_ARCH_HAS_DMA_MAP_DIRECT 1
#define CONFIG_TLAN_MODULE 1
#define CONFIG_RV_MON_WWNR 1
#define CONFIG_RPCSEC_GSS_KRB5_KUNIT_TEST_MODULE 1
#define CONFIG_IP_VS_NFCT 1
#define CONFIG_NSM_MODULE 1
#define CONFIG_CAN_CTUCANFD_PLATFORM_MODULE 1
#define CONFIG_DVB_CXD2099_MODULE 1
#define CONFIG_WATCHDOG_RTAS_MODULE 1
#define CONFIG_SDIO_UART_MODULE 1
#define CONFIG_SSB_B43_PCI_BRIDGE 1
#define CONFIG_NET_SCH_PIE_MODULE 1
#define CONFIG_UCLAMP_TASK_GROUP 1
#define CONFIG_USB_SERIAL_MOS7720_MODULE 1
#define CONFIG_NET_VENDOR_LITEX 1
#define CONFIG_RTW88_8822CE_MODULE 1
#define CONFIG_VIDEO_V4L2_SUBDEV_API 1
#define CONFIG_SCSI_SAS_ATA 1
#define CONFIG_IKHEADERS_MODULE 1
#define CONFIG_NVME_TARGET_FC_MODULE 1
#define CONFIG_LEDS_LM3692X_MODULE 1
#define CONFIG_SENSORS_MAX16065_MODULE 1
#define CONFIG_VIRTIO_BALLOON_MODULE 1
#define CONFIG_BCMA_HOST_PCI_POSSIBLE 1
#define CONFIG_VHOST_RING_MODULE 1
#define CONFIG_CPU_ISOLATION 1
#define CONFIG_GENERIC_CMOS_UPDATE 1
#define CONFIG_SERIAL_8250_PCI 1
#define CONFIG_NET_VENDOR_QLOGIC 1
#define CONFIG_USB_GSPCA_SONIXJ_MODULE 1
#define CONFIG_LWTUNNEL_BPF 1
#define CONFIG_CRYPTO_LIB_NH_MODULE 1
#define CONFIG_NET_ACT_CTINFO_MODULE 1
#define CONFIG_DLM_MODULE 1
#define CONFIG_SERIO 1
#define CONFIG_CHR_DEV_SCH_MODULE 1
#define CONFIG_SC92031_MODULE 1
#define CONFIG_MCP47FEB02_MODULE 1
#define CONFIG_USB_GSPCA_OV519_MODULE 1
#define CONFIG_NET_VRF_MODULE 1
#define CONFIG_DRM_GEM_SHMEM_HELPER_MODULE 1
#define CONFIG_ARCH_SUPPORTS_ATOMIC_RMW 1
#define CONFIG_DRM_LIB_RANDOM 1
#define CONFIG_DVB_TDA826X_MODULE 1
#define CONFIG_PPC64_ELF_ABI_V2 1
#define CONFIG_BT_INTEL_PCIE_MODULE 1
#define CONFIG_CRYPTO_LIB_AES 1
#define CONFIG_IPMI_POWERNV_MODULE 1
#define CONFIG_BPF_LIRC_MODE2 1
#define CONFIG_SND_GINA24_MODULE 1
#define CONFIG_DM_VERITY_FEC 1
#define CONFIG_MLX5_VDPA_NET_MODULE 1
#define CONFIG_REGULATOR_SY7636A_MODULE 1
#define CONFIG_NF_LOG_SYSLOG_MODULE 1
#define CONFIG_LTRF216A_MODULE 1
#define CONFIG_USB_OHCI_HCD 1
#define CONFIG_VFIO_VIRQFD 1
#define CONFIG_FB_IOMEM_FOPS 1
#define CONFIG_FW_CACHE 1
#define CONFIG_REGMAP_I2C 1
#define CONFIG_IP_NF_TARGET_REDIRECT_MODULE 1
#define CONFIG_MTD_MTDRAM_MODULE 1
#define CONFIG_HID_TOPRE_MODULE 1
#define CONFIG_SCSI_MPT2SAS_MODULE 1
#define CONFIG_MMC_SDHCI_MODULE 1
#define CONFIG_IOMMUFD_DRIVER 1
#define CONFIG_PRINTK 1
#define CONFIG_LEDS_TRIGGER_DISK 1
#define CONFIG_DM_SWITCH_MODULE 1
#define CONFIG_VIDEO_EM28XX_DVB_MODULE 1
#define CONFIG_NETLABEL 1
#define CONFIG_MEMCG_V1 1
#define CONFIG_PCI_STUB 1
#define CONFIG_CRYPTO_RMD160_MODULE 1
#define CONFIG_USB_NET_CDC_NCM_MODULE 1
#define CONFIG_DAMON_STAT 1
#define CONFIG_CC_VERSION_TEXT "powerpc64le-unknown-linux-gnu-gcc (Gentoo 15.2.1_p20260214 p5) 15.2.1 20260214"
#define CONFIG_EQUALIZER_MODULE 1
#define CONFIG_VIDEO_IVTV_MODULE 1
#define CONFIG_DVB_BUDGET_AV_MODULE 1
#define CONFIG_IP6_NF_MATCH_SRH_MODULE 1
#define CONFIG_BINFMT_ELF 1
#define CONFIG_BLK_DEV_RNBD 1
#define CONFIG_HIGH_RES_TIMERS 1
#define CONFIG_DEBUG_FS_ALLOW_ALL 1
#define CONFIG_NULL_TTY_MODULE 1
#define CONFIG_QED_RDMA 1
#define CONFIG_PPC_INDIRECT_PIO 1
#define CONFIG_HAVE_SAMPLE_FTRACE_DIRECT_MULTI 1
#define CONFIG_WIREGUARD_MODULE 1
#define CONFIG_RTW89_8852BU_MODULE 1
#define CONFIG_MEMFD_CREATE 1
#define CONFIG_PPC_XIVE 1
#define CONFIG_USB_HSIC_USB3503_MODULE 1
#define CONFIG_MTD_OF_PARTS_MODULE 1
#define CONFIG_DRM_AMD_DC_SI 1
